--[[--------------------------------------------------------------------------------
Copyright (c) Ulf Dornheck Busscher (Duugu) <duugu@duugu.com>
All rights reserved.
----------------------------------------------------------------------------------]]
local _, BAB = ...
local _G = _G
local floor = math.floor
local getn = table.getn
local pairs = pairs

local hooksecurefunc = hooksecurefunc
local SetPortraitToTexture = SetPortraitToTexture
local CreateFrame = CreateFrame

--
function BAB:AutoButtons(pBarName)
	if BAB.DB.Bars[pBarName].autoButton == 1 then
		if BAB:GetMaxButtonsOnBar(pBarName) > BAB:GetNumberOfButtons(pBarName) + 1 then
			for x = 1, BAB:GetMaxButtonsOnBar(pBarName) - BAB:GetNumberOfButtons(pBarName) + 1 do
				BAB:AddButton(pBarName)
			end
		end
	end
end
---
function BAB:CreateExtraButton(pName, pNumber)
	if not _G[pName..pNumber] then
		local button = _G[pName..pNumber] or BAB:CreateButtonFrame(pName..pNumber)
		hooksecurefunc(_G[button:GetName().."Icon"], "SetTexture", function(obj, texture) BAB:ButtonShapeRoundHook(obj, texture) end)
		button:Hide()
		return button
	else
		_G[pName..pNumber]:Hide()
		return _G[pName..pNumber]
	end
end
function BAB:CreateSecureButtonFrame()
	local cancelButton = CreateFrame("Button", "BABAurasCancelButton", UIParent, "SecureActionButtonTemplate")
	cancelButton:SetPoint("CENTER")
	cancelButton:SetSize(64, 64)
	cancelButton:SetFrameLevel(100)
	cancelButton:SetFrameStrata("HIGH")
--[[local tex = cancelButton:CreateTexture("cbtex", "OVERLAY")
tex:SetTexture("Interface\\AddOns\\BABAuras\\textures\\ButtonHilight-Square.tga")
tex:ClearAllPoints()
tex:SetPoint("TOPLEFT", cancelButton, "TOPLEFT")
tex:SetPoint("BOTTOMRIGHT", cancelButton, "BOTTOMRIGHT")
tex:SetDrawLayer("OVERLAY", 7)]]
	cancelButton:Hide()

	cancelButton:RegisterForClicks("RightButtonDown")
	cancelButton:SetAttribute("*type2", "macro")
	cancelButton:SetAttribute("*macrotext2", "")
	cancelButton:SetScript("OnLeave", function(self) 
		if not InCombatLockdown() then
			BAB.DB.CancelButton:SetAttribute("type2", "empty")
			BAB.DB.CancelButton:SetAttribute("*macrotext2", nil)
			self:ClearAllPoints()
			self:Hide()
		end
		GameTooltip:Hide()
	end)
	--cancelButton.renew = false
	
	BAB.DB.CancelButton = cancelButton
end

BABAurasLastAura = {}
function BAB:CreateButtonFrame(name)
	local button = CreateFrame("Frame", name, UIParent, "BABAurasAuraTemplate")
	button:SetScript("OnEnter", function(self) 
		local tData = BAB.DB.Bars[self.bar].Buttons[self.number].AuraData
		if tData and tData.spellId and tData.spellId > 0 then 
			local tIndex
			if BAB.DB.CurrentAurasTable[tData.type] then 
				for i, v in pairs(BAB.DB.CurrentAurasTable[tData.type]) do
					if tData.index == i then tIndex = v.x end
				end
			end

			if tData.name == "OH" or  tData.name == "MH" then
				GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
				GameTooltip:SetItemByID(tData.spellId)
				GameTooltip:Show()
			else
				if tIndex then
					GameTooltip:SetOwner(self, "ANCHOR_BOTTOMLEFT")
					GameTooltip:SetUnitAura("player", tIndex, tData.type)
					GameTooltip:Show()
				end
			end

			if not InCombatLockdown() then
				if BAB:IsAuraCancelable(tData.spellId, tData.type) then
					BAB.DB.CancelButton:ClearAllPoints()
					BAB.DB.CancelButton:SetPoint("TOPLEFT", BAB.DB.Bars[self.bar].Buttons[self.number].frame, "TOPLEFT")
					BAB.DB.CancelButton:SetPoint("BOTTOMRIGHT", BAB.DB.Bars[self.bar].Buttons[self.number].frame, "BOTTOMRIGHT")
					BAB.DB.CancelButton:Show()
				
					BAB.DB.CancelButton:SetAttribute("type2", "macro")

					local tBtn = 0
					if tData.type == "MH" then tBtn = 1 end
					if tData.type == "OH" then tBtn = 2 end

					if tBtn > 0 then
						BAB.DB.CancelButton:SetAttribute("*macrotext2", "/click TempEnchant"..tBtn.. " RightButton")
					elseif not BAB.DB.ProtectedBuffs[tData.spellId] then
						BAB.DB.CancelButton:SetAttribute("*macrotext2", "/script if not InCombatLockdown() then CancelUnitBuff('player', "..tIndex..", 'HELPFUL') BABAurasLastAura['"..self:GetName().."'] = true end")
					else
						BAB.DB.CancelButton:SetAttribute("*macrotext2", "/cancelaura "..tData.name.." \n/script  if not InCombatLockdown() then BABAurasLastAura['"..self:GetName().."'] = true end")
					end
				else
					BAB.DB.CancelButton:SetAttribute("type2", "empty")
					BAB.DB.CancelButton:SetAttribute("*macrotext2", nil)
				end
			end
		end	end)

	button:SetScript("OnLeave", function(self) 
		if InCombatLockdown() then
			GameTooltip:Hide()
		end
		local tData = BAB.DB.Bars[self.bar].Buttons[self.number].AuraData
		if tData and tData.spellId and tData.spellId > 0 then 
			if tData.name == "OH" or  tData.name == "MH" then
				GameTooltip:Hide()
			end
		end
	end)

	button:SetScript("OnUpdate", function(self, elapsed) 
		if BAB.DB.CancelButton and (not (BAB.DB.CancelButton:IsVisible() and InCombatLockdown())) then
			if BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.spellId or BAB.DB.MainOptions.configMode == true then
				if BAB.MSQ then 
					if BAB.MSQ:GetGloss(BAB.DB.Bars[self.bar].Buttons[self.number].frame) then
						BAB.MSQ:GetGloss(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Show()
					end
				end
				if BAB.DB.MainOptions.configMode == false then button:EnableMouse(true) end
				if BAB.DB.Bars[self.bar].showBorder == 1 and BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.spellId then
					if BAB.MSQ then 
						if BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame) then
							BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Show()
							button.border:Hide()
						else
							button.border:Show()
							BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Hide()
						end
					else
						button.border:Show()
					end

				else
					if BAB.MSQ then 
						if BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame) then
							BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Hide()
						end
					end
					button.border:Hide()
				end
				if BAB.DB.MainOptions.configMode == true then
				
					_G[BAB.DB.Bars[self.bar].Buttons[self.number].name.."FloatingBG"]:Show()
				else
					_G[BAB.DB.Bars[self.bar].Buttons[self.number].name.."FloatingBG"]:Hide()
				end
				if (BAB.DB.Bars[self.bar].showFlash == 1 and BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.expirationTime  and math.floor(BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.expirationTime - GetTime()) < 5 and math.floor(BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.expirationTime - GetTime()) >= 0 )  then
					button.flash:Show()
				else
					button.flash:Hide()
				end
			else
				button:EnableMouse(false)
				if BAB.MSQ then 
					if BAB.MSQ:GetGloss(BAB.DB.Bars[self.bar].Buttons[self.number].frame) then
						BAB.MSQ:GetGloss(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Hide()
					end
					if BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame) then
						BAB.MSQ:GetNormal(BAB.DB.Bars[self.bar].Buttons[self.number].frame):Hide()
					end
				end

				button.border:Hide()
				_G[BAB.DB.Bars[self.bar].Buttons[self.number].name.."FloatingBG"]:Hide()
				button.flash:Hide()
			end
		end

		if BAB.DB.CancelButton and not InCombatLockdown() and BAB.DB.CancelButton and BAB.DB.Bars[BAB.L["Buff"].." 1"]then
			if BAB.DB.CancelButton:GetWidth() ~= BAB.DB.Bars[BAB.L["Buff"].." 1"].buttonWidth then
				BAB.DB.CancelButton:SetSize(BAB.DB.Bars[BAB.L["Buff"].." 1"].buttonWidth, BAB.DB.Bars[BAB.L["Buff"].." 1"].buttonWidth)
			end 
		end

		BAB:OnUpdateCooldown(elapsed, self)

		if (BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.duration == nil) or (BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.duration and BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.duration <= 0) then
			BAB.DB.Bars[self.bar].Buttons[self.number].BABCount.FS:SetText("")
		else
			BAB.DB.Bars[self.bar].Buttons[self.number].BABCount.FS:SetText(BAB:FormatTimeValue(math.floor(BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.expirationTime - GetTime())))
		end
		
		if (BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.count == nil) or (BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.count and BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.count <= 1) then
			BAB.DB.Bars[self.bar].Buttons[self.number].BABHotKey.FS:SetText("")
		else
			BAB.DB.Bars[self.bar].Buttons[self.number].BABHotKey.FS:SetText(BAB.DB.Bars[self.bar].Buttons[self.number].AuraData.count)
		end

		

	end)
	
	button.icon               = _G[name .. "Icon"]
	button.flash              = _G[name .. "Flash"]
	button.hotkey             = _G[name .. "HotKey"]
	button.count              = _G[name .. "Count"]
	button.actionName         = _G[name .. "Name"]
	button.border             = _G[name .. "Border"]
	button.normal             = _G[name .. "Border"]
	button.cooldown           = _G[name .. "Cooldown"]

	button.hotkey:SetFont("Fonts\\ARIALN.ttf", 13, "OUTLINE")
	button.hotkey:SetVertexColor(0.75, 0.75, 0.75)

	
	button.SetNormalTexture = function(self) 
		return self, self.border
	end

	--button.normal.SetVertexColor = function() print("SetVertexColor") end
	--button.normal.GetVertexColor = function() print("GetVertexColor") end
	--button.normal.GetTexture = function() return "" end
	button.GetNormalTexture = function(self) return self.border end--button.normal end
	--button.SetTexture = function(self) end--button.normal end
	button.cooldown.SetFrameLevel = function(self) end
	--button:SetNormalTexture("")

	return button
end
--
---
function BAB:ButtonUpdateTexture(pButton)
	if pButton then
		for iTex, vTex in pairs(BAB.DB.Settings.applyPortaitOn) do
			if _G[pButton:GetName()..vTex] then
				_G[pButton:GetName()..vTex]:SetTexture(_G[pButton:GetName()..vTex]:GetTexture())
			end
		end
	else
		for bari, barv in pairs(BAB.DB.Bars) do
			for btni, btnv in pairs(barv.Buttons) do
				if btnv.frame then
					for iTex, vTex in pairs(BAB.DB.Settings.applyPortaitOn) do
						if _G[btnv.frame:GetName()..vTex] then
							_G[btnv.frame:GetName()..vTex]:SetTexture(_G[btnv.frame:GetName()..vTex]:GetTexture())
						end
					end
				end
			end
		end
	end
end
---
function BAB:ButtonShapeRoundHook(obj, texture)
	if not texture or texture == " " or texture == "" then return end

	if obj:GetTexture() ~= "Portrait2" then
		obj.org = obj:GetTexture()
	end
	-- doh. masque re-parents the icon texture? what a brilliant idea :/
	local bar = obj:GetParent().bar or  obj:GetParent():GetParent().bar or nil

	if bar then
		if BAB.DB.Bars[bar].buttonShape ~= BABAURAS_CONST_BUTTON_SHAPE_ROUND then
			if obj:GetTexture() == "Portrait2" then
				obj:SetTexture(obj.org)
			end
		else
			if obj:GetTexture() ~= "Portrait2" then
				SetPortraitToTexture(obj, texture)
			end
		end
	end
end
---
function BAB:SetButtonShape(pButton)
	local tButtonsSizeModificator = nil

	local tBName = pButton.frame:GetName()
	local tWidth = BAB.DB.Bars[pButton.bar].buttonWidth
	local tHeight = BAB.DB.Bars[pButton.bar].buttonHeight
	local tBFRame = pButton.frame

	--STATIC SHAPE
	if BAB.DB.Bars[pButton.bar].buttonShape == BABAURAS_CONST_BUTTON_SHAPE_STATIC then
		if tBFRame then

			local tMod = 0
			local tModBottom = 0
			local tModTop = 0

			if pButton.shapeMod then tMod = -(pButton.shapeMod / 360) * 3 end
			--[[
if not string.find(tBFRame:GetName(), "Flyout") then
	local ltMod 
	if tMod < 0 then ltMod = tMod * -1 else ltMod = tMod end
	tButtonsSizeModificator = BAB.DB.MainOptions.MaxShapeFactor / ltMod
end
]]
			if tMod > BAB.DB.MainOptions.MaxShapeFactor then tMod = BAB.DB.MainOptions.MaxShapeFactor end
			if tMod < -(BAB.DB.MainOptions.MaxShapeFactor) then tMod = -(BAB.DB.MainOptions.MaxShapeFactor) end

			tModBottom = 0
			tModTop = 0
			if tMod < 0 then
				tModBottom = 0
				tModTop = tMod * -1
			elseif tMod > 0 then
				tModBottom = tMod
				tModTop = 0
			end
		
			tModBottom = tModBottom * 1.3
			tModTop = tModTop * 1.3

			if tModBottom > BAB.DB.MainOptions.MaxShapeFactor then tModBottom = BAB.DB.MainOptions.MaxShapeFactor end
			if tModTop > BAB.DB.MainOptions.MaxShapeFactor then tModTop = BAB.DB.MainOptions.MaxShapeFactor end

			if tMod < 0 then tMod = tMod * -1 end

			tBFRame:SetAttribute("tMod", tMod)
			tBFRame:SetAttribute("tModTop", tModTop)
			tBFRame:SetAttribute("tModBottom", tModBottom)





			--~ BAB Cooldown        The cool down animation frame for the button.
			if _G[tBName.."CooldownTexture"] then
				local t = _G[tBName.."CooldownTexture"]
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetTexCoord(0,1,0,1)
				t:SetHeight(tWidth * 0.9)
				t:SetWidth(tWidth * 0.9)
				--t:SetDrawLayer("BORDER", 7)
		
			end
			--[[
			--~ AutoCastable    The texture that's shown when and ability can be toggled on and off.
			if _G[tBName.."AutoCastable"]  then
				local t = _G[tBName.."AutoCastable"]
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth)
				t:SetHeight(tHeight)
				t:SetTexCoord(0,1,0,1)
			end

			-- 100 Checked 
			if tBFRame.SetCheckedTexture then
				local t = tBFRame:GetCheckedTexture()--:GetTexture()
				if t then
					if BAB:IsMSQEnabled(pButton.bar) then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth * 1)
						t:SetWidth(tWidth * 1)
					else
						tBFRame:SetCheckedTexture("Interface\\AddOns\\BABAuras\\textures\\CheckButtonHilight.tga")
						if BAB.MSQ then
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetTexCoord(0, 1, 0, 1)
							t:SetHeight(tWidth)
							t:SetWidth(tWidth)
						else
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetTexCoord(0, 1, 0, 1)
							t:SetHeight(tWidth)
							t:SetWidth(tWidth)
						end
					end
				end
			end
			]]
			-- 150 FloatingBG / masque backdrop
			--masque backdrop
			local t = _G[tBName.."FloatingBG"]
			if BAB:IsMSQEnabled(pButton.bar) then
				local tMBD = BAB.MSQ:GetBackdrop(tBFRame)
				if tMBD then
					tMBD:ClearAllPoints()
                     tMBD:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
                     tMBD:SetWidth(tWidth)
                     tMBD:SetHeight(tHeight)
                     tMBD:SetDrawLayer("BACKGROUND", -7)
                     tMBD:SetTexCoord(0,1,0,1)
				end
			else
				if t then
					t:SetTexture("Interface\\Buttons\\UI-Quickslot")
					t:SetTexCoord(0,1,0,1)
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetDrawLayer("BACKGROUND", -7)
					t:SetWidth(tWidth * 1.6)
					t:SetHeight(tHeight * 1.6)
					if BAB.MSQ and BAB.MSQ:GetBackdrop(tBFRame) then
						BAB.MSQ:GetBackdrop(tBFRame):SetDrawLayer("BACKGROUND", -7)
					end
					--t:Show()
				end
			end

			-- 200 Icon
			local t = _G[tBName.."Icon"]
			if t then
				if BAB.MSQ and BAB:IsMSQEnabled(pButton.bar) then
					t:SetWidth(tWidth * 0.9)
					t:SetHeight(tHeight * 0.9)
					t:SetTexCoord(0,1,0,1)
				else
					t:SetWidth(tWidth)
					t:SetHeight(tHeight)
					t:SetTexCoord(0,1,0,1)
				end
			end

			-- 250 border 
			local t = _G[tBName.."Border"]
			if t then
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				--masque border
				if BAB:IsMSQEnabled(pButton.bar) then
                   t:SetWidth(tWidth * 1)
                   t:SetHeight(tHeight * 1)
                   t:SetTexCoord(0,1,0,1)
				else
                   t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\UI-ActionButton-Border2px.tga")
                   t:SetWidth(tWidth * 1.0)
                   t:SetHeight(tHeight * 1.0)
                   t:SetTexCoord(0,1,0,1)
				end
			--	t:Show()
			end
			--[[
			-- 300 Normal
			local t = tBFRame:GetNormalTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					local tMNormal = BAB.MSQ:GetNormal(tBFRame)
					if tMNormal then
						tMNormal:ClearAllPoints()
						tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						tMNormal:SetWidth(tWidth * 1.0)
						tMNormal:SetHeight(tHeight * 1.00)
						tMNormal:SetTexCoord(0,1,0,1)
					end
				else
					if BAB.MSQ then
						if BAB.MSQ:GetNormal(tBFRame) then
							local tMNormal = BAB.MSQ:GetNormal(tBFRame)
							tMNormal:ClearAllPoints()
							tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							tMNormal:SetWidth(tWidth * 1.7)
							tMNormal:SetHeight(tHeight * 1.7)
							tMNormal:SetTexCoord(0,1,0,1)
						end
					else
						t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * 1.7)
						t:SetHeight(tHeight * 1.7)
						t:SetTexCoord(0,1,0,1)
					end
				end
			end

			-- 350 Disabled
			local t = tBFRame:GetDisabledTexture()
			if t  then
				t:SetWidth(tWidth)
				t:SetHeight(tHeight)
				t:SetTexCoord(0,1,0,1)
			end

			local t = tBFRame:GetPushedTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
                  tBFRame:SetPushedTexture("Interface\\AddOns\\BABAuras\\textures\\UI-Quickslot-Depress.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					end
				end
			end

			local t = tBFRame:GetHighlightTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
					tBFRame:SetHighlightTexture("Interface\\AddOns\\BABAuras\\textures\\ButtonHilight-Square.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					end
				end
			end
			]]
			-- 500 flash
			local t = _G[tBName.."Flash"]
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\UI-QuickslotRed.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
						--t:Show()
					end
				end
			end
			--[[
			-- masque gloss
			if BAB.MSQ then
				if BAB.MSQ:GetGloss(tBFRame) then
					BAB.MSQ:GetGloss(tBFRame):SetWidth(tWidth * 1.0)
					BAB.MSQ:GetGloss(tBFRame):SetHeight(tHeight * 1.0)
					BAB.MSQ:GetGloss(tBFRame):SetTexCoord(0, 1, 0, 1)
				end
			end

			-- flyout
			local t = _G[tBName.."FlyoutBorder"]
			if t then
				t:Hide()
			end
			local t = _G[tBName.."BorderShadow"]
			if t then
				t:Hide()
			end

			--~ 0 Blizzard Cooldown        The cool down animation frame for the button.
			local t = _G[tBName.."Cooldown"]
			if t then
				t:Hide()
			end

			--Glow/ants
			local t = _G[tBName.."Glow"]
			if t then
				t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\IconAlertGlow.tga")
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth * 1.7)
				t:SetHeight(tHeight * 1.7)
				t:SetTexCoord(0,1,0,1)
			end
			local t = _G[tBName.."Ants"]
			if t then
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth * 1.0)
				t:SetHeight(tHeight * 1.0)
				t:SetTexCoord(0,1,0,1)
			end
			
			if pButton.backdropTexture then
				pButton.backdropTexture:SetWidth((tWidth * (1 + (tMod * 2.5) ) + BAB.DB.Bars[pButton.bar].buttonPadding)  * 1.1)
				pButton.backdropTexture:SetHeight((tHeight + BAB.DB.Bars[pButton.bar].buttonPadding)  * 1.3)
				pButton.backdropTexture:SetTexCoord(0,1,0,1)
			end
			]]
		end
	elseif BAB.DB.Bars[pButton.bar].buttonShape == BABAURAS_CONST_BUTTON_SHAPE_ROUND then
		if tBFRame then

			local tMod = 0
			local tModBottom = 0
			local tModTop = 0

			if pButton.shapeMod then tMod = -(pButton.shapeMod / 360) * 3 end
--[[
if not string.find(tBFRame:GetName(), "Flyout") then
	local ltMod 
	if tMod < 0 then ltMod = tMod * -1 else ltMod = tMod end
	tButtonsSizeModificator = BAB.DB.MainOptions.MaxShapeFactor / ltMod
end
]]

			if tMod > BAB.DB.MainOptions.MaxShapeFactor then tMod = BAB.DB.MainOptions.MaxShapeFactor end
			if tMod < -(BAB.DB.MainOptions.MaxShapeFactor) then tMod = -(BAB.DB.MainOptions.MaxShapeFactor) end

			tModBottom = 0
			tModTop = 0
			if tMod < 0 then
				tModBottom = 0
				tModTop = tMod * -1
			elseif tMod > 0 then
				tModBottom = tMod
				tModTop = 0
			end
		
			tModBottom = tModBottom * 1.3
			tModTop = tModTop * 1.3

			if tModBottom > BAB.DB.MainOptions.MaxShapeFactor then tModBottom = BAB.DB.MainOptions.MaxShapeFactor end
			if tModTop > BAB.DB.MainOptions.MaxShapeFactor then tModTop = BAB.DB.MainOptions.MaxShapeFactor end

			if tMod < 0 then tMod = tMod * -1 end

			tBFRame:SetAttribute("tMod", tMod)
			tBFRame:SetAttribute("tModTop", tModTop)
			tBFRame:SetAttribute("tModBottom", tModBottom)





			--~ BAB Cooldown        The cool down animation frame for the button.
			if _G[tBName.."CooldownTexture"] then
				local t = _G[tBName.."CooldownTexture"]
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", -1, 0)
				t:SetTexCoord(0,1,0,1)
				t:SetHeight(tWidth * 1)
				t:SetWidth(tWidth * 1)
				--t:SetDrawLayer("BORDER", 7)
		
			end
			--[[
			--~ AutoCastable    The texture that's shown when and ability can be toggled on and off.
			if _G[tBName.."AutoCastable"]  then
				if t then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth)
					t:SetHeight(tHeight)
					t:SetTexCoord(0,1,0,1)
				end
			end

			-- 100 Checked 
			if tBFRame.SetCheckedTexture then
				local t = tBFRame:GetCheckedTexture()--:GetTexture()
				if t then
					if BAB:IsMSQEnabled(pButton.bar) then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth * 1)
						t:SetWidth(tWidth * 1)
					else
						tBFRame:SetCheckedTexture("Interface\\AddOns\\BABAuras\\textures\\round\\CheckButtonHilight.tga")
						if BAB.MSQ then
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetTexCoord(0, 1, 0, 1)
							t:SetHeight(tWidth * 1.2)
							t:SetWidth(tWidth * 1.2)
						else
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetTexCoord(0, 1, 0, 1)
							t:SetHeight(tWidth * 1.2)
							t:SetWidth(tWidth * 1.2)
						end
					end
				end
			end
			]]
			-- 150 FloatingBG / masque backdrop
			--masque backdrop
			local t = _G[tBName.."FloatingBG"]
			if BAB:IsMSQEnabled(pButton.bar) then
				local tMBD = BAB.MSQ:GetBackdrop(tBFRame)
				if tMBD then
					tMBD:ClearAllPoints()
		                     tMBD:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
		                     tMBD:SetWidth(tWidth)
		                     tMBD:SetHeight(tHeight)
		                     tMBD:SetDrawLayer("BACKGROUND", -7)
		                     tMBD:SetTexCoord(0,1,0,1)
				end
			else
				if t then
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-Quickslot")
					t:SetTexCoord(0,1,0,1)
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetDrawLayer("BACKGROUND", -7)
					t:SetWidth(tWidth * 1.7)
					t:SetHeight(tHeight * 1.7)
					--t:Hide()
					if BAB.MSQ and BAB.MSQ:GetBackdrop(tBFRame) then
						local mBack = BAB.MSQ:GetBackdrop(tBFRame)
						BAB.MSQ:GetBackdrop(tBFRame):SetDrawLayer("BACKGROUND", -7)
						mBack:SetWidth(tWidth * 1.7)
						mBack:SetHeight(tHeight * 1.7)
					end
					--t:Show()
				end
			end

			-- 200 Icon
			local t = _G[tBName.."Icon"]
			if t then
				if BAB.MSQ and BAB:IsMSQEnabled(pButton.bar) then
					t:SetWidth(tWidth * 0.9)
					t:SetHeight(tHeight * 0.9)
					t:SetTexCoord(0,1,0,1)
				else
					t:SetWidth(tWidth)
					t:SetHeight(tHeight)
					t:SetTexCoord(0,1,0,1)
				end
			end

			-- 250 border 
			local t = _G[tBName.."Border"]
			if t then
				--masque border
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				if BAB:IsMSQEnabled(pButton.bar) then
					t:SetWidth(tWidth * 1)
					t:SetHeight(tHeight * 1)
		            t:SetTexCoord(0,1,0,1)
				else
		            t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-ActionButton-Border")
					t:SetWidth(tWidth * 1.8)
					t:SetHeight(tHeight * 1.8)
		            t:SetTexCoord(0,1,0,1)
				end
				--t:Show()
			end
			--[[
			-- 300 Normal
			local t = tBFRame:GetNormalTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					local tMNormal = BAB.MSQ:GetNormal(tBFRame)
					if tMNormal then
						tMNormal:ClearAllPoints()
						tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						tMNormal:SetWidth(tWidth * 1.1)
						tMNormal:SetHeight(tHeight * 1.1)
						tMNormal:SetTexCoord(0,1,0,1)
					end
				else
					if BAB.MSQ then
						if BAB.MSQ:GetNormal(tBFRame) then
							local tMNormal = BAB.MSQ:GetNormal(tBFRame)
							tMNormal:ClearAllPoints()
							tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							tMNormal:SetWidth(tWidth * 1.7)
							tMNormal:SetHeight(tHeight * 1.7)
							tMNormal:SetTexCoord(0,1,0,1)
							tMNormal:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-Quickslot2")
						end
					else
						t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-Quickslot2")
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * 1.7)
						t:SetHeight(tHeight * 1.7)
						t:SetTexCoord(0,1,0,1)
					end
				end
			end

			-- 350 Disabled
			local t = tBFRame:GetDisabledTexture()
			if t  then
				t:SetWidth(tWidth)
				t:SetHeight(tHeight)
				t:SetTexCoord(0,1,0,1)
			end

			local t = tBFRame:GetPushedTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
					tBFRame:SetPushedTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-Quickslot-Depress.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					end
				end
			end

			local t = tBFRame:GetHighlightTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
					tBFRame:SetHighlightTexture("Interface\\AddOns\\BABAuras\\textures\\round\\ButtonHilight-Square.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					end
				end
			end
			]]
			-- 500 flash
			local t = _G[tBName.."Flash"]
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetTexCoord(0, 1, 0, 1)
					t:SetHeight(tWidth * 1)
					t:SetWidth(tWidth * 1)
				else
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\UI-QuickslotRed.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth)
						t:SetWidth(tWidth)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetTexCoord(0, 1, 0, 1)
						t:SetHeight(tWidth * 1.1)
						t:SetWidth(tWidth * 1.1)
					end
					--t:Show()
				end
			end
			--[[
			-- masque gloss
			if BAB.MSQ then
				if BAB.MSQ:GetGloss(tBFRame) then
					BAB.MSQ:GetGloss(tBFRame):SetWidth(tWidth * 1.0)
					BAB.MSQ:GetGloss(tBFRame):SetHeight(tHeight * 1.0)
					BAB.MSQ:GetGloss(tBFRame):SetTexCoord(0, 1, 0, 1)
				end
			end

			-- flyout
			local t = _G[tBName.."FlyoutBorder"]
			if t then
				t:Hide()
			end
			local t = _G[tBName.."BorderShadow"]
			if t then
				t:Hide()
			end

			--~ 0 Blizzard Cooldown        The cool down animation frame for the button.
			local t = _G[tBName.."Cooldown"]
			if t then
				t:Hide()
			end

			--Glow/ants
			local t = _G[tBName.."Glow"]
			if t then
				if BAB:IsMSQEnabled(pButton.bar) then
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\IconAlertGlow.tga")
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * 1.4)
					t:SetHeight(tHeight * 1.4)
					t:SetTexCoord(0,1,0,1)
				else
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\IconAlertGlow.tga")
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * 1.7)
					t:SetHeight(tHeight * 1.7)
					t:SetTexCoord(0,1,0,1)
				end
			end
			local t = _G[tBName.."Ants"]
			if t then
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * 0.9)
					t:SetHeight(tHeight * 0.9)
					t:SetTexCoord(0,1,0,1)
				else
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * 1.1)
					t:SetHeight(tHeight * 1.1)
					t:SetTexCoord(0,1,0,1)
				end
			end

			if pButton.backdropTexture then
				pButton.backdropTexture:SetWidth((tWidth * (1 + (tMod * 2.5) ) + BAB.DB.Bars[pButton.bar].buttonPadding)  * 1.1)
				pButton.backdropTexture:SetHeight((tHeight + BAB.DB.Bars[pButton.bar].buttonPadding)  * 1.3)
				pButton.backdropTexture:SetTexCoord(0,1,0,1)
			end
			]]

		end

	--DYNAMIC SHAPE
	elseif BAB.DB.Bars[pButton.bar].buttonShape == BABAURAS_CONST_BUTTON_SHAPE_DYNAMIC then
		if tBFRame then
			-- -90 0 90 (bottom center top)
			local tMod = 0
			local tModBottom = 0
			local tModTop = 0

			if pButton.shapeMod then tMod = -(pButton.shapeMod / 360) * 3 end

--[[
if not string.find(tBFRame:GetName(), "Flyout") then
	local ltMod 
	if tMod < 0 then ltMod = tMod * -1 else ltMod = tMod end
	tButtonsSizeModificator = BAB.DB.MainOptions.MaxShapeFactor / ltMod
end
]]
			if tMod > BAB.DB.MainOptions.MaxShapeFactor then tMod = BAB.DB.MainOptions.MaxShapeFactor end
			if tMod < -(BAB.DB.MainOptions.MaxShapeFactor) then tMod = -(BAB.DB.MainOptions.MaxShapeFactor) end

			tModBottom = 0
			tModTop = 0
			if tMod < 0 then
				tModBottom = 0
				tModTop = tMod * -1
			elseif tMod > 0 then
				tModBottom = tMod
				tModTop = 0
			end
		
			tModBottom = tModBottom * 1.3
			tModTop = tModTop * 1.3

			if tModBottom > BAB.DB.MainOptions.MaxShapeFactor then tModBottom = BAB.DB.MainOptions.MaxShapeFactor end
			if tModTop > BAB.DB.MainOptions.MaxShapeFactor then tModTop = BAB.DB.MainOptions.MaxShapeFactor end

			if tMod < 0 then tMod = tMod * -1 end

			tBFRame:SetAttribute("tMod", tMod)
			tBFRame:SetAttribute("tModTop", tModTop)
			tBFRame:SetAttribute("tModBottom", tModBottom)



			--~ BAB Cooldown        The cool down animation frame for the button.
			if _G[tBName.."CooldownTexture"] then
				local t = _G[tBName.."CooldownTexture"]
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				--t:SetWidth(tWidth * (0.95 +  (tMod )))
				--t:SetHeight(tHeight * (1.5  ))
					t:SetWidth((tWidth * 1.0) * (1 +  (tMod )))
					t:SetHeight((tHeight * 1) * (1.5  ))

				t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				--t:SetDrawLayer("BORDER", 7)
		
			end
			--[[
			--~ AutoCastable    The texture that's shown when and ability can be toggled on and off.
			if _G[tBName.."AutoCastable"]  then
				local t = _G[tBName.."AutoCastable"]
				if t then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5 ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				end
			end

			-- 100 Checked 
			if tBFRame.SetCheckedTexture then
				local t = tBFRame:GetCheckedTexture()--:GetTexture()
				if t then
					if BAB:IsMSQEnabled(pButton.bar) then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (0.94 +  (tMod )))
						t:SetHeight(tHeight * (1.47 ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					else
						tBFRame:SetCheckedTexture("")
						if BAB.MSQ then
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetWidth(tWidth * (0.93 +  (tMod )))
							t:SetHeight(tHeight * (1.44 ))
							t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
						else
							t:ClearAllPoints()
							t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							t:SetWidth(tWidth * (1.05 +  (tMod )))
							t:SetHeight(tHeight * (1.55 ))
							t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
						end
					end
					t:Hide()
				end
			end
			]]
			-- 150 FloatingBG / masque backdrop
			--masque backdrop
			local t = _G[tBName.."FloatingBG"]
			if BAB:IsMSQEnabled(pButton.bar) then
				local tMBD = BAB.MSQ:GetBackdrop(tBFRame)
				if tMBD then
					tMBD:ClearAllPoints()
					tMBD:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					tMBD:SetWidth(tWidth * (1 +  (tMod )))
					tMBD:SetHeight(tHeight * (1.5  ))
					tMBD:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				end
			else
				if t then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1.7 +  (tMod * 2)))
					t:SetHeight(tHeight * ( 2.55  ))
					local tAdd = tModTop / 2.5
					local tAdd1 = tModBottom / 2.5
					t:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
					--t:SetVertexColor(1,1,1,1)
					--t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\UI-Quickslot")
					t:SetDrawLayer("BACKGROUND", -7)
					if BAB.MSQ and BAB.MSQ:GetBackdrop(tBFRame) then
						BAB.MSQ:GetBackdrop(tBFRame):SetDrawLayer("BACKGROUND", -7)
					end
					--t:Show()
				end
			end

			-- 200 Icon
			local t = _G[tBName.."Icon"]
			if t then
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (0.85 +  (tMod )))
					t:SetHeight(tHeight * (1.30  ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					t:SetDrawLayer("BORDER", 7)
				else
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5  ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					t:SetDrawLayer("BORDER", 7)
				end
			end

			-- 250 border 
			local t = _G[tBName.."Border"]
			if t then
				--masque border
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.50 ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				else
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\UI-ActionButton-Border2px.tga")
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5  ))
					local tAdd = tModTop / 2.5
					local tAdd1 = tModBottom / 2.5
					t:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
					--t:Show()
				end
			end
			--[[
			-- 300 Normal
			local t = tBFRame:GetNormalTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					local tMNormal = BAB.MSQ:GetNormal(tBFRame)
					if tMNormal then
						tMNormal:ClearAllPoints()
						tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						tMNormal:SetWidth(tWidth * (1 +  (tMod )))
						tMNormal:SetHeight(tHeight * (1.55 ))
						tMNormal:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					end
				else
					if BAB.MSQ then
						if BAB.MSQ:GetNormal(tBFRame) then
							local tMNormal = BAB.MSQ:GetNormal(tBFRame)
							tMNormal:ClearAllPoints()
							tMNormal:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
							tMNormal:SetWidth(tWidth * (1.7 +  (tMod * 2)))
							tMNormal:SetHeight(tHeight * ( 2.55  ))
							local tAdd = tModTop / 2.5
							local tAdd1 = tModBottom / 2.5
							tMNormal:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
						end
					else
						t:SetTexture("")
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1.7 +  (tMod * 2)))
						t:SetHeight(tHeight * ( 2.55  ))
						local tAdd = tModTop / 2.5
						local tAdd1 = tModBottom / 2.5
						t:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
						--t:Hide()
					end
				end
			end

			-- 350 Disabled
			local t = tBFRame:GetDisabledTexture()
			if t  then
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth * (1 +  (tMod )))
				t:SetHeight(tHeight * (1.50 ))
				t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				--t:Hide()
			end

			local t = tBFRame:GetPushedTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5 ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				else
					tBFRame:SetPushedTexture("")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1.1 +  (tMod )))
						t:SetHeight(tHeight * ( 1.6  ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1.1 +  (tMod )))
						t:SetHeight(tHeight * ( 1.6  ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
						--t:Hide()
					end
				end
			end

			local t = tBFRame:GetHighlightTexture()
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5 ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				else
					tBFRame:SetHighlightTexture("")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1 +  (tMod )))
						t:SetHeight(tHeight * (1.55 ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1 +  (tMod )))
						t:SetHeight(tHeight * (1.55 ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
						--t:Hide()
					end
				end
			end
			]]
			-- 500 flash
			local t = _G[tBName.."Flash"]
			if t then
				--masque normal
				if BAB:IsMSQEnabled(pButton.bar) then
					t:ClearAllPoints()
					t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
					t:SetWidth(tWidth * (1 +  (tMod )))
					t:SetHeight(tHeight * (1.5 ))
					t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				else
					t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\UI-QuickslotRed.tga")
					if BAB.MSQ then
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1.1 +  (tMod )))
						t:SetHeight(tHeight * (1.6 ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					else
						t:ClearAllPoints()
						t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
						t:SetWidth(tWidth * (1.1 +  (tMod )))
						t:SetHeight(tHeight * (1.6 ))
						t:SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
					end
					--t:Show()
				end
			end
			--[[
			-- masque gloss
			if BAB.MSQ then
				if BAB.MSQ:GetGloss(tBFRame) then
					BAB.MSQ:GetGloss(tBFRame):SetWidth(tWidth * (1.1 + tMod ))
					BAB.MSQ:GetGloss(tBFRame):SetHeight(tHeight * (1.7  ))
					BAB.MSQ:GetGloss(tBFRame):SetTexCoord(-tModTop, -0.25, -tModBottom , 1.25,  1 + tModTop, -0.25 , 1 + tModBottom , 1.25)
				end
			end

			-- flyout
			local t = _G[tBName.."FlyoutBorder"]
			if t then
				t:Hide()
				t:SetTexCoord(0, -(tMod), -(tMod), 1 + (tMod), 1, -(tMod), 1 + (tMod), 1 + (tMod))
			end
			local t = _G[tBName.."BorderShadow"]
			if t then
				t:Hide()
				t:SetTexCoord(0, -(tMod), -(tMod), 1 + (tMod), 1, -(tMod), 1 + (tMod), 1 + (tMod))
			end

			--~ 0 Blizzard Cooldown        The cool down animation frame for the button.
			local t = _G[tBName.."Cooldown"]
			if t then
				t:Hide()
			end

			--Glow/ants
			local t = _G[tBName.."Glow"]
			if t then
				t:SetTexture("")
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth * (1.7 +  (tMod * 2)))
				t:SetHeight(tHeight * ( 2.55  ))
				local tAdd = tModTop / 2.5
				local tAdd1 = tModBottom / 2.5
				t:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
				--t:Hide()
			end
			local t = _G[tBName.."Ants"]
			if t then
				t:ClearAllPoints()
				t:SetPoint("CENTER", tBFRame, "CENTER", 0, 0)
				t:SetWidth(tWidth * (1.1 +  (tMod )))
				t:SetHeight(tHeight * ( 1.55  ))
				local tAdd = tModTop / 2.5
				local tAdd1 = tModBottom / 2.5
				t:SetTexCoord(-tModTop  - tAdd , -0.25, -tModBottom - tAdd, 1.25,  1 + tModTop + tAdd1, -0.25 , 1 + tModBottom + tAdd1, 1.25)
				--t:Hide()
			end

			if pButton.backdropTexture then
				pButton.backdropTexture:SetWidth((tWidth * (1 + (tMod * 2.5) ) + BAB.DB.Bars[pButton.bar].buttonPadding)  * 1.1)
				local tAdd = tModTop / 2.5
				local tAdd1 = tModBottom / 2.5
				pButton.backdropTexture:SetTexCoord(-tModTop , -0.25, -tModBottom , 1.25,  (1 + tModTop), -0.25 , 1 + tModBottom , 1.25)
				--t:Hide()
			end
			]]
		end
	end

	return tButtonsSizeModificator
end

---
function BAB:OnUpdateCooldown(elapsed, self)
	self.barName = self.bar
	self.buttonNumber = self.number
    if self.barName then
        if not BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer then 
            BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer = 0 
        end
        BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer = BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer + elapsed
        if BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer > BAB.DB.Settings.CooldownUpdateRate and (BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownstart and BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownDuration) then
            local duration = BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownDuration
            local start = BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownstart
            local left = duration - (GetTime() - start)
			if duration > 0 and BAB.DB.Bars[self.barName].showCooldown == 1  then
				BAB:SetCooldownTexture(BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownTexture, floor(((left / duration ) * 100)))
			else
				BAB:SetCooldownTexture(BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cooldownTexture, 0)
			end
            BAB.DB.Bars[self.barName].Buttons[self.buttonNumber].cTimer = 0
        end
    end
end
----
function BAB:UpdateButtons()
    for a, b in pairs(BAB.DB.Bars) do
        for i, v in pairs(b.Buttons) do
			if not _G[v.name] then
				BAB:CreateButton(a, BAB:CreateExtraButton(a, i):GetName(), i) 
				BAB:HideButton(a, i)
			else
				v.frame = _G[v.name]
				v.frame.bar = a
				v.cooldown = _G[v.name.."Cooldown"]
				v.BABCenterFS = v.frame.BABCenterFS
				v.cooldownTexture = v.frame.cooldownTexture            
				v.BABHotKey = _G[v.frame:GetName().."BABHotKey"] or BAB:CreateCustomFS(v, "BABHotKey")
				v.BABCount = _G[v.frame:GetName().."BABCount"] or BAB:CreateCustomFS(v, "BABCount")
				v.BABActionName = _G[v.frame:GetName().."BABActionName"] or BAB:CreateCustomFS(v, "BABActionName")
				v.backdropTexture = BAB:GetButtonBackgroundTexture(v.frame, v)
				BAB:HideButton(a, i)
			end
        end
    end
end
----
function BAB:CreateCustomFS(pButton, pFSFrameName)
	local tFRame = _G[pButton.frame:GetName()..pFSFrameName] or CreateFrame("Frame", pButton.frame:GetName()..pFSFrameName, BAB.DB.Bars[pButton.bar].textOverlay)         
	pButton[pFSFrameName] = tFRame
    pButton.frame[pFSFrameName] = tFRame
    tFRame:SetWidth(1) 
    tFRame:SetHeight(1) 
    tFRame:ClearAllPoints() 
	tFRame:SetParent(BAB.DB.Bars[pButton.bar].textOverlay)
    tFRame:Hide()

	local tFS = tFRame:CreateFontString(pButton.frame:GetName()..pFSFrameName.."FS", "OVERLAY")  
	tFRame.FS = tFS   
    tFS:SetPoint("CENTER", tFRame, "CENTER") 
    --tFS:SetFontObject(GameFontNormalLarge)
	--tFS:SetFont("Fonts\\ARIALN.TTF", 12)
	tFS:SetFont("Fonts\\ARIALN.ttf", 13, "OUTLINE")
	tFS:SetVertexColor(0.75, 0.75, 0.75)
    tFS:SetText("") 
    tFS:SetTextColor(1, 1, 1, 1)
    tFS:SetJustifyH("CENTER") 
    tFS:SetJustifyV("TOP") 

    local ag = tFRame.ag or tFRame:CreateAnimationGroup()
    tFS.ag = ag
    local a1 = tFRame.a1 or ag:CreateAnimation("Rotation")
    tFS.a1 = a1
	tFS.a1:SetDegrees(360)
    tFS.a1:SetDuration(0.001)
    tFS.a1:SetEndDelay(10)
    tFS.a1:SetScript("OnFinished", function(self, elapsed) end)    
    tFS.a1:SetScript("OnUpdate", function(self, elapsed) if self:IsPlaying() then if self:IsDelaying() then self:Pause() end end end)    
	--tFS.ag:Play()

    tFS:Show()

	return tFRame
end
----
function BAB:CreateCustomButtonElements(pButton)
	local tFrame = pButton.frame
	local tFName = tFrame:GetName()

    local ag = tFrame:CreateAnimationGroup()
	tFrame.y = 200
    tFrame.ag = ag
    local a1 = ag:CreateAnimation("Rotation")
    tFrame.a1 = a1
    a1:SetDuration(0.0000000)
    a1:SetEndDelay(2)
    a1:SetScript("OnFinished", function(self, elapsed) end)    
    a1:SetScript("OnUpdate", function(self, elapsed) if self:IsPlaying() then if self:IsDelaying() then self:Pause() end end end)    

    local tCFS = _G[tFName.."BABCenterFS"] or tFrame:CreateFontString(tFName.."BABCenterFS", "OVERLAY")  
	       
    pButton.BABCenterFS = tCFS
    tFrame.BABCenterFS = tCFS
    tCFS.a2 = ag:CreateAnimation("Rotation")
    tCFS:SetFontObject(GameFontNormalLarge)
    tCFS:SetText("") 
    tCFS:SetTextColor(1, 1, 1, 1)
    tCFS:SetJustifyH("CENTER") 
    tCFS:SetJustifyV("MIDDLE") 
    tCFS:ClearAllPoints() 
	tCFS:SetParent(BAB.DB.Bars[pButton.bar].frame)
    tCFS:SetPoint("CENTER", tFrame, "CENTER") 
    tCFS:Show()

	--tFrame.ag:Play()

    pButton.BABHotKey = _G[tFName.."BABHotKey"] or BAB:CreateCustomFS(pButton, "BABHotKey")
	tFrame.hotkey.BABHotKey = pButton.BABHotKey
	hooksecurefunc(tFrame.hotkey, "SetText", function(self, text) 
		if self.BABHotKey and text then 
			if text ~= RANGE_INDICATOR then 
				local tString = text
				if string.len(text) < 3 then
					--tString = tString..string.rep(" ", (3 - string.len(text)))
				end
				self.BABHotKey.FS:SetText(tString) 
				--self.BABHotKey:Show() 
			else
				self.BABHotKey.FS:SetText("") 
				--self.BABHotKey:Hide() 
			end
		end 
		self:Hide() 
	end)

    pButton.BABCount = _G[tFName.."BABCount"] or BAB:CreateCustomFS(pButton, "BABCount")
	tFrame.count.BABCount = pButton.BABCount
	hooksecurefunc(tFrame.count, "SetText", function(self, text) 
		if self.BABCount and text then 
			if text ~= RANGE_INDICATOR then 
				local tString = text
				if string.len(text) < 2 then
					--tString = tString..string.rep(" ", (2 - string.len(text)))
				end
				self.BABCount.FS:SetText(tString) 
			else
				self.BABCount.FS:SetText("") 
			end
		end 
		self:Hide() 
	end)

	pButton.BABActionName = _G[tFName.."BABActionName"] or BAB:CreateCustomFS(pButton, "BABActionName")
	tFrame.actionName.BABActionName = pButton.BABActionName
	hooksecurefunc(tFrame.actionName, "SetText", function(self, text) 
		if self.BABActionName and text then 
			if text ~= RANGE_INDICATOR then 

				local tString = string.sub(text, 1, 5)
				if string.len(tString) < 5 then
					--tString = tString..string.rep(" ", (5 - string.len(tString)))
				end
				self.BABActionName.FS:SetText(tString) 
--				self.BABActionName:Show() 
			else
				self.BABActionName.FS:SetText("") 
--				self.BABActionName:Hide() 
			end
		end 
		self:Hide() 
	end)

	local tGlow = _G[tFName.."Glow"] or tFrame:CreateTexture(tFName.."Glow","OVERLAY")
	pButton.Glow = tGlow
	tGlow:SetTexture("Interface\\AddOns\\BABAuras\\textures\\IconAlertGlow.tga")
    tGlow:ClearAllPoints()
    tGlow:SetPoint("TOPLEFT", tFrame, "TOPLEFT")
    tGlow:SetPoint("BOTTOMRIGHT", tFrame, "BOTTOMRIGHT")
	tGlow:SetDrawLayer("OVERLAY", 2)
    tGlow:Hide()

	local tAnts = _G[tFName.."Ants"] or tFrame:CreateTexture(tFName.."Ants","OVERLAY")
	pButton.Ants = tAnts
	tAnts:SetTexture("Interface\\AddOns\\BABAuras\\textures\\IconAlertAnts1.tga")
    tAnts:ClearAllPoints()
    tAnts:SetPoint("TOPLEFT", tFrame, "TOPLEFT")
    tAnts:SetPoint("BOTTOMRIGHT", tFrame, "BOTTOMRIGHT")
	tAnts:SetDrawLayer("OVERLAY", 2)
	tAnts:Hide()
    
	local tTex = _G[pButton.name.."CooldownTexture"] or tFrame:CreateTexture(pButton.name.."CooldownTexture","OVERLAY")
    pButton.cooldownTexture = tTex
    tFrame.cooldownTexture = tTex
    tTex:SetTexture("Interface\\AddOns\\BABAuras\\textures\\BABcooldown.tga")
    tTex:SetVertexColor(0, 0, 0, 1)
    --tTex:SetTexCoord(0,0,0,0)
    tTex:ClearAllPoints()
    tTex:SetPoint("TOPLEFT", tFrame, "TOPLEFT")
    tTex:SetPoint("BOTTOMRIGHT", tFrame, "BOTTOMRIGHT")
    tTex:SetAlpha(BAB.DB.Settings.CooldownTextureAlpha)
	tTex:SetDrawLayer("OVERLAY", 1)
    tTex:Hide()   
	_G[tFName.."Cooldown"]:SetAlpha(0)


	pButton.FloatingBG = _G[tFName.."FloatingBG"] or tFrame:CreateTexture(tFName.."FloatingBG","BACKGROUND")
	pButton.FloatingBG:SetTexture("Interface\\Buttons\\UI-Quickslot")
	pButton.FloatingBG:SetAlpha(0.4)
	pButton.FloatingBG:SetDrawLayer("BACKGROUND")
end
----
function BAB:CreateButton(pBarName, pButtonFrameName, pButtonNumber)
    if not pBarName or not pButtonFrameName then return false end
    local tCurrent = getn(BAB.DB.Bars[pBarName].Buttons)

    BAB.DB.Bars[pBarName].Buttons[pButtonNumber] = {
        frame = _G[pButtonFrameName],
        bar = pBarName,
        name = pButtonFrameName,
        x = 0,
        y = 0,
        degree = 0,
        cooldown = _G[pButtonFrameName.."Cooldown"],
        cooldownTexture = nil,
		backdropTexture = nil,
		number = pButtonNumber,
    }
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame.bar = pBarName
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame.number = pButtonNumber
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].unused = false
	SecureHandler_OnLoad(BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame)
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame:SetFrameRef("BarFrame", BAB.DB.Bars[pBarName].frame)
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame.degree = 0
	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].backdropTexture = BAB:GetButtonBackgroundTexture(BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame, BAB.DB.Bars[pBarName].Buttons[pButtonNumber])

	BAB.DB.Bars[pBarName].Buttons[pButtonNumber].AuraData = {
		index = nil,
		name = nil, 
		type = nil,
		rank = nil, 
		texture = nil, 
		count = nil, 
		debuffType = nil, 
		duration = nil, 
		expirationTime = nil, 
		caster = nil, 
		isStealable = nil, 
		shouldConsolidate = nil, 
		spellId = nil,
		}

    local tButtonData = BAB.DB.Bars[pBarName].Buttons[pButtonNumber]
    BAB:CreateCustomButtonElements(tButtonData)

    if _G[tButtonData.frame:GetName().."Cooldown"] then
        _G[tButtonData.frame:GetName().."Cooldown"].buttonName = tButtonData.frame:GetName()
        _G[tButtonData.frame:GetName().."Cooldown"].barName = pBarName
        _G[tButtonData.frame:GetName().."Cooldown"].buttonNumber = pButtonNumber

        if _G[tButtonData.frame:GetName().."Cooldown"]:IsVisible() then
            tButtonData.cooldownTexture:Show()
        end
    end

    BAB:SetupButton(pBarName, pButtonNumber)
    BAB:ShowButton(pBarName, pButtonNumber)

end
---
function BAB:CalculateFSRotation(pButton, posX, posY)
-- Not even that the fontstring rotation is broken if the fs object is not achored to center/center ... nono ... the 
-- fontstring additionaly rotates around its top. ITS $%0#& TOP. NOT THE CENTER. Which means we have to fix the 
-- wrong position after rotating the fontstring object. Thank you blizzard for wasting hours of my life to work around 
-- something that could be fixed in minutes if you would care a minimum about addon authors. -.-

	posX, posY = posX, posY or 0, 0

	local tDegree = pButton.degree
		if tDegree < 0 then tDegree = 270 + (tDegree + 90) end
	local tHalfDegree = 180 - tDegree
		if tHalfDegree < 0 then tHalfDegree = tHalfDegree * -1 end
	local tHalfDegree1 = tDegree - 315
		if tHalfDegree1 < 0 then tHalfDegree1 = 360 + (tDegree - 315) end
	local tHalfDegree2 = 180 - tHalfDegree1
		if tHalfDegree2 < 0 then tHalfDegree2 = tHalfDegree2 * -1 end

	local tsx, tsy, tex, tey = pButton.x, pButton.y, pButton.x - (BAB.DB.Bars[pButton.bar].buttonHeight / 2) + 17 - ((180 - tHalfDegree2) / 18) + posX, pButton.y + (BAB.DB.Bars[pButton.bar].buttonHeight / 2) + 4 + posY
	local new_x = floor(tsx + cos(tDegree) * (tex - tsx) - sin(tDegree) * (tey - tsy))
	local new_y = floor(tsy + sin(tDegree) * (tex - tsx) + cos(tDegree) * (tey - tsy))
	tsx, tsy, tex, tey = new_x, new_y, new_x, new_y - (tHalfDegree / 15) 

	return floor(tsx + cos(tDegree) * (tex - tsx) - sin(tDegree) * (tey - tsy)), floor(tsy + sin(tDegree) * (tex - tsx) + cos(tDegree) * (tey - tsy))
end
---
function BAB:SetupButton(pBarName, pButtonNumber, pRotate, pFin)
	if BAB.DB.Bars[pBarName].isActive == 0 then
		--		
	else
		if BAB.DB.Bars[pBarName] then
			local tButton = BAB.DB.Bars[pBarName].Buttons[pButtonNumber]
			if tButton then
				local tButtonsSizeModificator 
				local tFrame = tButton.frame
				--tFrame:ClearAllPoints()
				tFrame:SetHeight(BAB.DB.Bars[pBarName].buttonHeight)
				tFrame:SetWidth(BAB.DB.Bars[pBarName].buttonWidth)
				tFrame:SetPoint("CENTER", BAB.DB.Bars[pBarName].frame, "BOTTOMLEFT", tButton.x, tButton.y) 

				--tFrame.hotkey.BABHotKey:ClearAllPoints()
if tFrame.hotkey.BABHotKey.FS.ag:IsPaused() == true then 
	tFrame.hotkey.BABHotKey.FS.ag:Stop() 
	tFrame.hotkey.BABHotKey.FS.a1:Stop() 
end
				tFrame.hotkey.BABHotKey:SetPoint("CENTER",BAB.DB.Bars[pBarName].frame, "BOTTOMLEFT", BAB:CalculateFSRotation(tButton, BAB.DB.Bars[pBarName].buttonWidth / 8, -3)) 
				tFrame.hotkey.BABHotKey.FS.a1:SetDegrees(tButton.degree)
				if BAB.DB.Bars[pBarName].showHotKey == 1 then
					tFrame.hotkey.BABHotKey:Show()
				else
					tFrame.hotkey.BABHotKey:Hide()
				end
if tFrame.hotkey.BABHotKey.FS.ag:IsPlaying() == false then 
	tFrame.hotkey.BABHotKey.FS.ag:Play() 
	tFrame.hotkey.BABHotKey.FS.a1:Play() 
end



if tFrame.count.BABCount.FS.ag:IsPaused() == true then 
	tFrame.count.BABCount.FS.ag:Stop() 
	tFrame.count.BABCount.FS.a1:Stop() 
end
				--tFrame.count.BABCount:ClearAllPoints()
				tFrame.count.BABCount:SetPoint("CENTER",BAB.DB.Bars[pBarName].frame, "BOTTOMLEFT", BAB:CalculateFSRotation(tButton, BAB.DB.Bars[pBarName].buttonWidth / 6, -(BAB.DB.Bars[pBarName].buttonHeight / 1.8))) 
				tFrame.count.BABCount.FS.a1:SetDegrees(tButton.degree)
				if BAB.DB.Bars[pBarName].showCount == 1 then
					tFrame.count.BABCount:Show()
				else
					tFrame.count.BABCount:Hide()
				end
if tFrame.count.BABCount.FS.ag:IsPlaying() == false then 
	tFrame.count.BABCount.FS.ag:Play() 
	tFrame.count.BABCount.FS.a1:Play() 
end

				--tFrame.actionName.BABActionName:ClearAllPoints()
if tFrame.actionName.BABActionName.FS.ag:IsPaused() == true then 
	tFrame.actionName.BABActionName.FS.ag:Stop() 
	tFrame.actionName.BABActionName.FS.a1:Stop() 
end
				tFrame.actionName.BABActionName:SetPoint("CENTER",BAB.DB.Bars[pBarName].frame, "BOTTOMLEFT", BAB:CalculateFSRotation(tButton, 5, -(BAB.DB.Bars[pBarName].buttonHeight / 1.8))) 
				tFrame.actionName.BABActionName.FS.a1:SetDegrees(tButton.degree)
				if BAB.DB.Bars[pBarName].showActionName == 1 then
					tFrame.actionName.BABActionName:Show()
				else
					tFrame.actionName.BABActionName:Hide()
				end

				if BAB.DB.MasqueGroups[pBarName] then 
					BAB.DB.MasqueGroups[pBarName]:AddButton(tButton.frame, {}) 
				end
if tFrame.actionName.BABActionName.FS.ag:IsPlaying() == false then 
	tFrame.actionName.BABActionName.FS.ag:Play() 
	tFrame.actionName.BABActionName.FS.a1:Play() 
end


				local tBD = tButton.backdropTexture
if tBD.ag:IsPaused() == true then 
	--tBD.ag:Stop() 
	tBD.a1:Stop() 
end
				if BAB.DB.Bars[pBarName].showBackdropTexture == 1 then
					tBD.a1:SetDegrees(tButton.degree)
					tBD:SetTexture("")--"Interface\\AddOns\\BABAuras\\Textures\\ButtonBackdrop")
					tBD:SetHeight(BAB.DB.Bars[pBarName].buttonHeight * 2)
					tBD:SetWidth(BAB.DB.Bars[pBarName].buttonWidth + (BAB.DB.Bars[pBarName].buttonPadding * 1.0))
					tBD:SetPoint("CENTER", tButton.frame, "CENTER") 
					--tBD:SetDrawLayer("BACKGROUND", -8)
					tBD:SetVertexColor(BAB.DB.Bars[pBarName].backdropColor.r, BAB.DB.Bars[pBarName].backdropColor.g, BAB.DB.Bars[pBarName].backdropColor.b) 
					tBD:Show()
				else
					tBD:Hide() 
				end
if tBD.ag:IsPlaying() == false then 
	--tBD.ag:Play() 
	tBD.a1:Play() 
end


				
				tButtonsSizeModificator = BAB:SetButtonShape(tButton)

				if tButton.x == 0 and tButton.y == 0 and tButton.degree == 0 then
					BAB:HideButton(pBarName, pButtonNumber)
				else
					BAB:ShowButton(pBarName, pButtonNumber)
				end
				
				return tButtonsSizeModificator 
			end
		end
	end
end
----
function BAB:ShowButton(pBarName, pButtonNumber)
	if BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame then
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame:Show()
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABCenterFS:Show()
		if BAB.DB.Bars[pBarName].showHotKey == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABHotKey:Show()
		end
		if BAB.DB.Bars[pBarName].showCount == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABCount:Show()
		end
		if BAB.DB.Bars[pBarName].showActionName == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABActionName:Show()
		end
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].backdropTexture:Show()

	end
end
----
function BAB:HideButton(pBarName, pButtonNumber)
	if BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame then
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].frame:Hide()
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABCenterFS:Hide()
		if BAB.DB.Bars[pBarName].showHotKey == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABHotKey:Hide()
		end
		if BAB.DB.Bars[pBarName].showCount == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABCount:Hide()
		end
		if BAB.DB.Bars[pBarName].showActionName == 1 then
			BAB.DB.Bars[pBarName].Buttons[pButtonNumber].BABActionName:Hide()
		end
		BAB.DB.Bars[pBarName].Buttons[pButtonNumber].backdropTexture:Hide()
	end
end
---
function BAB:SetCooldownTexture(pCooldownFrame, pPercentLeft)
	if pPercentLeft > 0 then
		local n = floor(pPercentLeft / 0.401)
		if BAB.DB.Bars[pCooldownFrame:GetParent().bar].buttonShape == BABAURAS_CONST_BUTTON_SHAPE_ROUND then
			if pCooldownFrame:GetTexture() ~= "Interface\\AddOns\\BABAuras\\textures\\round\\CD\\"..n then
				pCooldownFrame:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\CD\\"..n)
				if pCooldownFrame:IsVisible() ~= true then pCooldownFrame:Show() end
			end
		else
			if pCooldownFrame:GetTexture() ~= "Interface\\AddOns\\BABAuras\\textures\\CD\\"..n..".tga" then
				pCooldownFrame:SetTexture("Interface\\AddOns\\BABAuras\\textures\\CD\\"..n..".tga")
				if pCooldownFrame:IsVisible() ~= true then pCooldownFrame:Show() end
			end
		end


    else
        if pCooldownFrame:IsVisible() ~= false then pCooldownFrame:Hide() end
        
    end
end
