--[[--------------------------------------------------------------------------------
Copyright (c) Ulf Dornheck Busscher (Duugu) <duugu@duugu.com>
All rights reserved.
----------------------------------------------------------------------------------]]
local _, BAB = ...
local _G = _G
local floor = math.floor
local getn = table.getn
local tinsert = table.insert
local atan2 = math.atan2
local deg = math.deg 
local pow = math.pow
local pairs = pairs

local GetUnitName = GetUnitName
local GetRealmName = GetRealmName
local CreateFrame = CreateFrame

---
function BAB:ResizeButtonTextLabels(pBarName)
	for ib, vb in pairs(BAB.DB.Bars[pBarName].Buttons) do
		if BAB.DB.Bars[pBarName].autoFontSize == 1  then
			local filename, fontHeight, flags = vb.BABHotKey.FS:GetFont()
			fontHeight = floor(BAB.DB.Bars[pBarName].buttonWidth / 3.8)
			if fontHeight < 8 then fontHeight = 8 end
			if fontHeight > 20 then fontHeight = 20 end
			vb.BABHotKey.FS:SetFont(filename, fontHeight, flags)
		
			filename, fontHeight, flags = vb.BABCount.FS:GetFont()
			fontHeight = floor(BAB.DB.Bars[pBarName].buttonWidth / 3.8)
			if fontHeight < 8 then fontHeight = 8 end
			if fontHeight > 20 then fontHeight = 20 end
			vb.BABCount.FS:SetFont(filename, fontHeight, flags)
		
			filename, fontHeight, flags = vb.BABActionName.FS:GetFont()
			fontHeight = floor(BAB.DB.Bars[pBarName].buttonWidth / 3.8)
			if fontHeight < 8 then fontHeight = 8 end
			if fontHeight > 20 then fontHeight = 20 end
			vb.BABActionName.FS:SetFont(filename, fontHeight, flags)
		else
			local filename, fontHeight, flags = vb.BABHotKey.FS:GetFont()
			fontHeight = 13
			vb.BABHotKey.FS:SetFont(filename, fontHeight, flags)
		
			filename, fontHeight, flags = vb.BABCount.FS:GetFont()
			fontHeight = 13
			vb.BABCount.FS:SetFont(filename, fontHeight, flags)
		
			filename, fontHeight, flags = vb.BABActionName.FS:GetFont()
			fontHeight = 13
			vb.BABActionName.FS:SetFont(filename, fontHeight, flags)
		end
	end
end
---
function BAB:StoreMasqueData()
	if MasqueDB then
		local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
		local tProfile = MasqueDB.profiles[MasqueDB.profileKeys[tUnit]]

		--local tAddonData = tProfile.Groups["BAB"]
		--BAB.DB.MasqueBarData = BAB:TableCopy(tAddonData, true)   

		for i, v in pairs(BAB.DB.Bars) do
			local tBarData = tProfile.Groups["BABAURAS_"..v.shownName]
			BAB.DB.Bars[i].MasqueBarData = BAB:TableCopy_test(tBarData, true)   
			if BAB.DB.Bars[i].MasqueBarData.SkinID == "Blizzard" or BAB.DB.Bars[i].MasqueBarData.SkinID == "Dream" or BAB.DB.Bars[i].MasqueBarData.SkinID == "Zoomed"  then
				BAB.DB.Bars[i].MasqueBarData.SkinID = "BABAURAS_"..BAB.DB.Bars[i].MasqueBarData.SkinID
			end
		end
	end
end
---
function BAB:LoadMasqueData()

	if MasqueDB then
		local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
		local tProfile = MasqueDB.profiles[MasqueDB.profileKeys[tUnit]]

		--local tAddonData = tProfile.Groups["BAB"]
		--if BAB.DB.MasqueBarData then
			--tAddonData = BAB:TableCopy(BAB.DB.MasqueBarData, true)   
		--else
			--BAB.DB.MasqueBarData = BAB:TableCopy(tAddonData, true)   
		--end

		for i, v in pairs(BAB.DB.Bars) do
			if BAB.DB.Bars[i].MasqueBarData then
				local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
				local tProfile = MasqueDB.profiles[MasqueDB.profileKeys[tUnit]]
				local tBarData = tProfile.Groups["BABAURAS_"..v.shownName]
				--tBarData = BAB:TableCopy_test(BAB.DB.Bars[i].MasqueBarData, true)  
				tBarData.SkinID = BAB.DB.Bars[i].MasqueBarData.SkinID
				tBarData.Disabled = BAB.DB.Bars[i].MasqueBarData.Disabled
				tBarData.Inherit = BAB.DB.Bars[i].MasqueBarData.Inherit
				tBarData.Gloss = BAB.DB.Bars[i].MasqueBarData.Gloss
				tBarData.Backdrop = BAB.DB.Bars[i].MasqueBarData.Backdrop
				tBarData.Colors = BAB.DB.Bars[i].MasqueBarData.Colors

			else
				local tBarData = tProfile.Groups["BABAURAS_"..v.shownName]
				BAB.DB.Bars[i].MasqueBarData = BAB:TableCopy_test(tBarData, true)   
			end
		end
	end
end
---
function BAB:LoadMasqueGroups()
	if MasqueDB then
		for i, v in pairs(BAB.DB.Bars) do
			BAB.DB.MasqueGroups[i] = BAB.MSQ:Group("BABAuras", v.shownName) 
			for bi, bv in pairs(v.Buttons) do
				BAB.DB.MasqueGroups[i]:AddButton(bv.frame) 
			end
		end
	end
end
function BAB:DeleteMasqueGroups()
	if MasqueDB then
		for i, v in pairs(BAB.DB.Bars) do
			for bi, bv in pairs(v.Buttons) do
				BAB.DB.MasqueGroups[i]:RemoveButton(bv.frame, true)
			end
			BAB.DB.MasqueGroups[i]:Delete(true)
			BAB.DB.MasqueGroups[i] = nil
			local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
			local tProfile = MasqueDB.profiles[MasqueDB.profileKeys[tUnit]]
			tProfile.Groups["BABAURAS_"..v.shownName] = nil
		end
	end
end
--
function BAB:ApplyMasque(pBarname)
	if MasqueDB then
		if pBarname then
			BAB.DB.MasqueGroups[pBarname]:ReSkin()  
		else
			for i, v in pairs(BAB.DB.Bars) do
				BAB.DB.MasqueGroups[i]:ReSkin()  
			end
		end
	end
end


---
local tModifiers = {[2] = {a = 1.628, b = 3.38, c = 0.58},
					[3] = {a = 1.278, b = 3.20, c = 0.75},
					[4] = {a = 1.140, b = 3.00, c = 0.85},
					[5] = {a = 1.100, b = 3.00, c = 0.90},
					[6] = {a = 1.060, b = 3.00, c = 0.95},
					[7] = {a = 1.047, b = 3.00, c = 0.96},
					[8] = {a = 1.037, b = 3.00, c = 0.97}}
function BAB:AutoSetupBarShape(tBarname, tShape, tArg1, pUserInitiated) 
	if not tBarname or not tShape then return nil end

	if tShape == BABAURAS_CONST_CIRCLE then 
		if not tArg1 then return end
		if tArg1 < 20 then tArg1 = 2 end
		if BAB.DB.Bars[tBarname].calcStyle == 1 then
			local tKnots = getn(BAB.DB.Bars[tBarname].Sections)
			if tKnots == 1 then
				BAB:AddSection(tBarname)		
				BAB:SetupSection(tBarname)
				tKnots = getn(BAB.DB.Bars[tBarname].Sections)
			end

			local r = tArg1
			local centerx, centery = BAB.DB.Bars[tBarname].width / 2, BAB.DB.Bars[tBarname].height / 2
	
			for q = 1, tKnots do
				local new_x = cos((360 / tKnots) * q) * r + centerx
				local new_y = sin((360 / tKnots) * q) * r + centery
				if q == 1 then
					BAB.DB.Bars[tBarname].Sections[q].Knot1.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Knot1.y = new_y
					BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x = new_x
					BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y = new_y
				elseif q == tKnots then
					BAB.DB.Bars[tBarname].Sections[q].Knot1.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Knot1.y = new_y
					BAB.DB.Bars[tBarname].Sections[q-1].Knot2.x = new_x
					BAB.DB.Bars[tBarname].Sections[q-1].Knot2.y = new_y
				else
					BAB.DB.Bars[tBarname].Sections[q].Knot1.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Knot1.y = new_y
					BAB.DB.Bars[tBarname].Sections[q - 1].Knot2.x = new_x
					BAB.DB.Bars[tBarname].Sections[q - 1].Knot2.y = new_y
				end
			end

			local tMod = tModifiers[tKnots].b
			r = r * tModifiers[tKnots].a --230

			for q = 1, tKnots do
				local new_x = cos(((360 / tKnots ) * q) + ( ((360 / tKnots ) / tMod))) * r + centerx
				local new_y = sin(((360 / tKnots ) * q) + ( ((360 / tKnots ) / tMod))) * r + centery
				BAB.DB.Bars[tBarname].Sections[q].Vector1.x = new_x
				BAB.DB.Bars[tBarname].Sections[q].Vector1.y = new_y
				if q > 1 then
					local tCenterX = BAB.DB.Bars[tBarname].Sections[q].Vector1.frame.db.x - BAB.DB.Bars[tBarname].Sections[q].Knot1.frame.db.x
					local tCenterY = BAB.DB.Bars[tBarname].Sections[q].Vector1.frame.db.y - BAB.DB.Bars[tBarname].Sections[q].Knot1.frame.db.y
					BAB.DB.Bars[tBarname].Sections[q - 1].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[tBarname].Sections[q].Knot1.frame.db.x
					BAB.DB.Bars[tBarname].Sections[q - 1].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[tBarname].Sections[q].Knot1.frame.db.y
				end
			end
			local tCenterX = BAB.DB.Bars[tBarname].Sections[1].Vector1.frame.db.x - BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.x
			local tCenterY = BAB.DB.Bars[tBarname].Sections[1].Vector1.frame.db.y - BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.y
			BAB.DB.Bars[tBarname].Sections[tKnots].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.x
			BAB.DB.Bars[tBarname].Sections[tKnots].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.y
			if BAB.DB.Bars[tBarname].linked ~= 1 then
				local new_x = BAB.DB.Bars[tBarname].Sections[1].Knot1.x - BAB.DB.Bars[tBarname].Sections[1].Vector1.x
				local new_y = BAB.DB.Bars[tBarname].Sections[1].Knot1.y - BAB.DB.Bars[tBarname].Sections[1].Vector1.y
				BAB.DB.Bars[tBarname].Sections[tKnots].Vector2.x = BAB.DB.Bars[tBarname].Sections[1].Knot1.x + new_x
				BAB.DB.Bars[tBarname].Sections[tKnots].Vector2.y = BAB.DB.Bars[tBarname].Sections[1].Knot1.y + new_y
			end

		else
			local tKnots = getn(BAB.DB.Bars[tBarname].Sections)
			if tKnots == 1 then
				BAB:AddSection(tBarname)		
				BAB:SetupSection(tBarname)
				tKnots = getn(BAB.DB.Bars[tBarname].Sections)
			end

			local r = tArg1
			local centerx, centery = BAB.DB.Bars[tBarname].width / 2, BAB.DB.Bars[tBarname].height / 2
		
			local tNumberOfPoints = 360 / (4 + ((tKnots - 2) * 3) + 2)
			local tCount = 0

			for q = 1, tKnots do
				if q == 1 then
					local new_x = cos(tCount * tNumberOfPoints) * (r * tModifiers[tKnots].c) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r * tModifiers[tKnots].c) + centery
					BAB.DB.Bars[tBarname].Sections[1].Knot1.x = new_x
					BAB.DB.Bars[tBarname].Sections[1].Knot1.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[1].Vector1.x = new_x
					BAB.DB.Bars[tBarname].Sections[1].Vector1.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[1].Vector2.x = new_x
					BAB.DB.Bars[tBarname].Sections[1].Vector2.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[1].Knot2.x = new_x
					BAB.DB.Bars[tBarname].Sections[1].Knot2.y = new_y
				elseif q == tKnots then
					local new_x = cos(tCount * tNumberOfPoints) * (r * 1.5) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r * 1.5) + centery
					BAB.DB.Bars[tBarname].Sections[q].Knot1.x = BAB.DB.Bars[tBarname].Sections[q-1].Knot2.x
					BAB.DB.Bars[tBarname].Sections[q].Knot1.y = BAB.DB.Bars[tBarname].Sections[q-1].Knot2.y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[q].Vector1.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Vector1.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[q].Vector2.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Vector2.y = new_y

					BAB.DB.Bars[tBarname].Sections[q].Knot2.x = BAB.DB.Bars[tBarname].Sections[1].Knot1.x
					BAB.DB.Bars[tBarname].Sections[q].Knot2.y = BAB.DB.Bars[tBarname].Sections[1].Knot1.y
				else
					BAB.DB.Bars[tBarname].Sections[q].Knot1.x = BAB.DB.Bars[tBarname].Sections[q-1].Knot2.x
					BAB.DB.Bars[tBarname].Sections[q].Knot1.y = BAB.DB.Bars[tBarname].Sections[q-1].Knot2.y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[q].Vector1.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Vector1.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[q].Vector2.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Vector2.y = new_y
					tCount = tCount + 1
					local new_x = cos(tCount * tNumberOfPoints) * (r) + centerx
					local new_y = sin(tCount * tNumberOfPoints) * (r) + centery
					BAB.DB.Bars[tBarname].Sections[q].Knot2.x = new_x
					BAB.DB.Bars[tBarname].Sections[q].Knot2.y = new_y
				end
			end
		end
		BAB:SetupSection(tBarname)
        BAB:CalculatePoints(tBarname)
        BAB:DrawPoints(tBarname)
		BAB:RotateButtonsOnBar(tBarname)
		BAB:AutoButtons(tBarname)
	elseif tShape == BABAURAS_CONST_LINE then 
		local tKnots = getn(BAB.DB.Bars[tBarname].Sections)

		if BAB.DB.Bars[tBarname].Sections[1].Knot1.x - BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x < 50 and BAB.DB.Bars[tBarname].Sections[1].Knot1.x - BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x > -50 then
			BAB.DB.Bars[tBarname].Sections[1].Knot1.x = 0
			BAB.DB.Bars[tBarname].Sections[1].Knot1.y = 0
			BAB.DB.Bars[tBarname].Sections[tKnots].Knot1.x = BAB.DB.Settings.BarWidth
			BAB.DB.Bars[tBarname].Sections[tKnots].Knot1.y = 0
		end

		local tLength = BAB:LineLenghtBresenham(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y)
		if tLength / (tKnots * 3) < BAB.DB.Settings.minLineLengthForReShape then tLength = (tKnots * 3) * BAB.DB.Settings.minLineLengthForReShape end
		tLength = tLength / (tKnots * 3)


		BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, 0)
		BAB.DB.Bars[tBarname].Sections[1].Vector1.x, BAB.DB.Bars[tBarname].Sections[1].Vector1.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength )
		BAB.DB.Bars[tBarname].Sections[1].Vector2.x, BAB.DB.Bars[tBarname].Sections[1].Vector2.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength * 2)
		BAB.DB.Bars[tBarname].Sections[1].Knot2.x, BAB.DB.Bars[tBarname].Sections[1].Knot2.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength * 3)
		for q = 2, tKnots do
			BAB.DB.Bars[tBarname].Sections[q].Knot1.x, BAB.DB.Bars[tBarname].Sections[q].Knot1.y = BAB.DB.Bars[tBarname].Sections[q - 1].Knot2.x, BAB.DB.Bars[tBarname].Sections[q - 1].Knot2.y
			BAB.DB.Bars[tBarname].Sections[q].Vector1.x, BAB.DB.Bars[tBarname].Sections[q].Vector1.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength * (1 + ((q-1) * 3)))
			BAB.DB.Bars[tBarname].Sections[q].Vector2.x, BAB.DB.Bars[tBarname].Sections[q].Vector2.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength * (2 + ((q-1) * 3)))
			BAB.DB.Bars[tBarname].Sections[q].Knot2.x, BAB.DB.Bars[tBarname].Sections[q].Knot2.y = BAB:FindPointOnLine(BAB.DB.Bars[tBarname].Sections[1].Knot1.x, BAB.DB.Bars[tBarname].Sections[1].Knot1.y, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.x, BAB.DB.Bars[tBarname].Sections[tKnots].Knot2.y, tLength * (3 + ((q-1) * 3)))

		end

		BAB:SetupSection(tBarname)
        BAB:CalculatePoints(tBarname)
        BAB:DrawPoints(tBarname)
		BAB:RotateButtonsOnBar(tBarname)
		BAB:AutoButtons(tBarname)
	end
end
---
function BAB:LinkBar(tBarname, tLink) 
	if tLink == true then
		local tP = 0
		if BAB.DB.Bars[tBarname].CirclePermanent == 1 then 
			BAB.DB.Bars[tBarname].CirclePermanent = 0 
			tP = 1 
		end

		BAB.DB.Bars[tBarname].minSections = 2
		if getn(BAB.DB.Bars[tBarname].Sections) < 2 then BAB:AddSection(tBarname) end
		local tSectionNumber = getn(BAB.DB.Bars[tBarname].Sections)

		if (BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y - BAB.DB.Bars[tBarname].Sections[1].Knot1.y) < 50 then
			if BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y < BAB.DB.Bars[tBarname].Sections[1].Knot1.y then
				BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y = BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y - 100
				BAB.DB.Bars[tBarname].Sections[1].Knot1.y = BAB.DB.Bars[tBarname].Sections[1].Knot1.y + 100
			else
				BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y = BAB.DB.Bars[tBarname].Sections[getn(BAB.DB.Bars[tBarname].Sections)].Knot2.y + 100
				BAB.DB.Bars[tBarname].Sections[1].Knot1.y = BAB.DB.Bars[tBarname].Sections[1].Knot1.y - 100
			end
		end

		BAB:SetupSection(tBarname)

		BAB.DB.Bars[tBarname].linked = 1 
        BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.isMoving = true
		BAB.DB.Bars[tBarname].Sections[1].Knot1.frame:GetScript("OnUpdate")(BAB.DB.Bars[tBarname].Sections[1].Knot1.frame, true, true)
        BAB.DB.Bars[tBarname].Sections[1].Knot1.frame.db.isMoving = false
        BAB.DB.Bars[tBarname].Sections[1].Vector1.frame.db.isMoving = true
		BAB.DB.Bars[tBarname].Sections[1].Vector1.frame:GetScript("OnUpdate")(BAB.DB.Bars[tBarname].Sections[1].Vector1.frame, true, true)
        BAB.DB.Bars[tBarname].Sections[1].Vector1.frame.db.isMoving = false

		if tP == 1 then
			BAB.DB.Bars[tBarname].CirclePermanent = 1
			BAB:AutoSetupBarShape(tBarname, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[tBarname].CircleRadius, true) 
		end 

	else
		if BAB.DB.Bars[tBarname].CirclePermanent == 0 then
			BAB.DB.Bars[tBarname].minSections = 1
		end
		if getn(BAB.DB.Bars[tBarname].Sections) > 1 then
			--BAB:RemoveSection(tBarname)
		end
		BAB.DB.Bars[tBarname].linked = 0
	end

	if getn(BAB.DB.Bars[tBarname].Sections) > BAB.DB.Bars[tBarname].minSections then
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Enable()
	else
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Disable()
	end
	if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) > 7 then
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Disable()
	else
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Enable()
	end
end
---
function BAB:ResetBar(pBarname)
	local pName, pNumberButtons = BAB.DB.Bars[pBarname].name, BAB:GetNumberOfButtons(pBarname)
	BAB:RemoveBar(pBarname)
	BAB:SelectBar()
	BAB:SelectBar(BAB:AddBar(pName, pNumberButtons))
	BAB:SaveProfile("resetbar", BABAurasCurrentProfile)
end
---
function BAB:CreateBaseProfile()
    local tY = 0
    local tBarname = BAB.L["Buff"].." 1"
    tY = tY + BAB.DB.Settings.BarHeight * 1.1
    BAB:CreateBar(tBarname, (UIParent:GetWidth() / 2) + (BAB.DB.Settings.BarWidth / 2), UIParent:GetHeight() - tY, tBarname)
    BAB:SetupBar(tBarname)
    BAB:AddSection(tBarname)
    BAB.DB.Bars[tBarname].buttonWidth = BAB.DB.Settings.baseButtonWith
    BAB.DB.Bars[tBarname].buttonHeight = BAB.DB.Settings.baseButtonWith
	for x = 1, 12 do
		BAB:CreateButton(tBarname, BAB:CreateExtraButton(tBarname, x):GetName(), x) 
	end

    local tY = BAB.DB.Settings.baseButtonWith
    local tBarname = BAB.L["Debuff"].." 1"
    tY = tY + BAB.DB.Settings.BarHeight * 1.1
    BAB:CreateBar(tBarname, (UIParent:GetWidth() / 2) + (BAB.DB.Settings.BarWidth / 2) , UIParent:GetHeight() - tY, tBarname)
    BAB:SetupBar(tBarname)
    BAB:AddSection(tBarname)
    BAB.DB.Bars[tBarname].buttonWidth = BAB.DB.Settings.baseButtonWith
    BAB.DB.Bars[tBarname].buttonHeight = BAB.DB.Settings.baseButtonWith
	for x = 1, 12 do
		BAB:CreateButton(tBarname, BAB:CreateExtraButton(tBarname, x):GetName(), x) 
	end	

    local tY = 0
    local tBarname = BAB.L["Enchants"]
    tY = tY + BAB.DB.Settings.BarHeight * 1.1
	BAB.DB.Settings.BarWidth = 150
    BAB:CreateBar(tBarname, (UIParent:GetWidth() / 2) + (215 + 430), UIParent:GetHeight() - tY, tBarname)
    BAB:SetupBar(tBarname)
    BAB:AddSection(tBarname)
    BAB.DB.Bars[tBarname].buttonWidth = BAB.DB.Settings.baseButtonWith
    BAB.DB.Bars[tBarname].buttonHeight = BAB.DB.Settings.baseButtonWith
	for x = 1, 3 do
		BAB:CreateButton(tBarname, BAB:CreateExtraButton(tBarname, x):GetName(), x) 
	end
	BAB.DB.Settings.BarWidth = 430
end
---
BABTT = {}
function BAB:OnEvent(pSelf, pEventName, ...)
table.insert(BABTT, pEventName)
    if pEventName == "UNIT_AURA" or pEventName == "UNIT_INVENTORY_CHANGED" then
		BAB:UpdateAuras()
    elseif pEventName == "VARIABLES_LOADED" then
		if not BABTexturePackLoaded then
			print(BAB.L["BAB error: 'BABTexturePack' is not available. Download BABTexturePack from http://www.wowinterface.com/downloads/info22792-BABTexturePack.html to use BAB."])
			return
		end
		if BABTexturePackLoaded < BAB.DB.Settings.requiredTexturePackVersion then
			print(string.format(BAB.L["BAB warning: You are using an old BABTexturePack version (%s). The newest version (%s) is available from http://www.wowinterface.com/downloads/info22792-BABTexturePack.html."], BABTexturePackLoaded, BAB.DB.Settings.requiredTexturePackVersion))
		end

        --masque support
        if MasqueDB then
            BAB.MSQ = LibStub("Masque")
            BAB.MSQ:Register("BABAuras", BAB.MasqueCallback, BAB)
        end




        BAB:CreateBaseProfile()



    elseif pEventName == "PET_BATTLE_OPENING_START" then
		if BAB.DB.MainOptions.configMode == true then
			BAB:CloseOptions()
		    BABAurasOptionSettings.MainOptions.frame:Hide()
	        BAB.DB.MainOptions.configMode = false
			BAB:SaveProfile("pet batt open")
		end
		BAB:HideBar()

    elseif pEventName == "PET_BATTLE_CLOSE" then
		BAB:ShowBar()

    elseif pEventName == "PLAYER_REGEN_ENABLED" then
		BAB.DB.InCombatLockdown = false

    elseif pEventName == "PLAYER_REGEN_DISABLED" then
		BAB.DB.CancelButton:ClearAllPoints()
		BAB.DB.CancelButton:Hide()
		BAB.DB.InCombatLockdown = true
		if BAB.DB.MainOptions.configMode == true then
			BAB:CloseOptions()
	        BAB.DB.MainOptions.configMode = false
		    BABAurasOptionSettings.MainOptions.frame:Hide()
			BAB:SaveProfile("plr regen dis")
		end
    elseif pEventName == "PLAYER_ENTERING_WORLD" then
        local tFirstStart = false

		BAB:CreateSecureButtonFrame()
		BAB:SetupAll()
		BAB:SelectBar()


		--create the base profile
        if not BABAurasProfiles["Base"] then
            tFirstStart = true
            BABAurasProfiles["Base"] = {}
            BABAurasProfiles["Base"] = BAB:TableCopy(BAB.DB, true)    
        end

        
        if not BABAurasCurrentProfile then
            BAB:LoadProfile(BAB.L["Default"])
            --BAB:LoadProfile(BAB.L["Default"])
        else
            BAB:LoadProfile(BABAurasCurrentProfile)
            --BAB:LoadProfile(BABAurasCurrentProfile)
        end
            
		if BAB.MSQ then
			if not BABAurasProfiles["Base"].Skins then
				BABAurasProfiles["Base"].Skins = {}
			end
			if not BAB.Skins then
				BAB.Skins = {}
			end
			BAB.Skins[1]()
			BAB.Skins[2]()
			BAB.Skins[3]()
			BAB.Skins[4]()
			BAB.Skins[5]()
			BAB.Skins[6]()
			BAB.Skins[7]()
			BAB.Skins[8]()
			BAB.Skins[9]()
		end




		SlashCmdList["BABAURAS"] = function(msg)
			if not InCombatLockdown() then
				if BAB.DB.MainOptions.configMode == false then BAB:OpenOptions() else BAB:CloseOptions() end
			end
		end
		SLASH_BABAURAS1 = "/babauras"

		InterfaceOptionsFrame:SetMovable(true)
		InterfaceOptionsFrame:RegisterForDrag("LeftButton")
		InterfaceOptionsFrame:SetScript("OnDragStart", function(self) self:StartMoving() end)
		InterfaceOptionsFrame:SetScript("OnDragStop", function(self) self:StopMovingOrSizing() end)
		InterfaceOptionsFrame:SetClampedToScreen(true)

		if _G["BuffFrame"] then _G["BuffFrame"]:Hide() end
		if _G["TemporaryEnchantFrame"] then _G["TemporaryEnchantFrame"]:Hide() end

        local tObj = _G["BABAurasOptions"] or CreateFrame("Frame", "BABAurasOptions" ,UIParent)
        tObj:SetFrameStrata("MEDIUM")
        tObj:SetHeight(BABAurasOptionSettings.MainOptions.height)
        tObj:SetWidth(BABAurasOptionSettings.MainOptions.width)
        tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
        tile = false, tileSize = 1, edgeSize = 10, insets = { left = 0, right = 0, top = 0, bottom = 0 }})
        tObj:SetBackdropColor(0, 0, 0, 1)
        tObj:EnableMouse(true)
        tObj:SetMovable(true)
		
		-- this is to save the bab profile on masque changes. its a bad hack as it don't gets changes if the user don't close the options. But ... it is masque's fault. period.
		InterfaceOptionsFrame:SetScript("OnHide", function() BAB:SaveProfile() end)

		-- this reload is for IsSpellKnown which is not available on variables_loaded. 
		BAB:LoadProfile(BABAurasCurrentProfile)

		
		BAB:OpenOptions()
        BAB:CloseOptions()
		BABAurasOptionSettings.MainOptions.frame:SetScript("OnShow", function(self,text) 
			PlaySound("igCharacterInfoOpen")
		end)
		BABAurasOptionSettings.MainOptions.frame:SetScript("OnHide", function(self,text) 
			if BAB.DB.MainOptions.configMode == true then
				BAB:CloseOptions()
				BABAurasOptionSettings.MainOptions.frame:Hide()
				BAB.DB.MainOptions.configMode = false
				BAB:SaveProfile("web ticket")
			end
			PlaySound("igCharacterInfoClose")
		end)
		tinsert(UISpecialFrames, BABAurasOptionSettings.MainOptions.frame:GetName())
		BAB:UpdateAuras()
    end

end
---
function BAB:SetupAll()
	BAB:ClearLines()
    for i, v in pairs(BAB.DB.Bars) do
		v.update = true
        BAB:SetupBar(i)
        BAB:SetupSection(i)
        BAB:CalculatePoints(i)
        BAB:DrawPoints(i)
        BAB:SetButtonsOnBar(i)
        BAB:RotateButtonsOnBar(i)
        for b, bv in pairs(v.Buttons) do
            BAB:SetButtonShape(bv)
        end
    end
end
---
function BAB:InitializeBar(pBarName)
	BAB.DB.Bars[pBarName].update = true
	BAB:ClearLines(nil, pBarName)
    BAB:SetupBar(pBarName)
    BAB:ShowBar(pBarName)
    BAB:SetupSection(pBarName)
    BAB:CalculatePoints(pBarName)
    BAB:DrawPoints(pBarName)
    BAB:RotateButtonsOnBar(pBarName)
    for i, v in pairs(BAB.DB.Bars[pBarName].Buttons) do
        BAB:SetButtonShape(v)
    end
end
---
function BAB:AddBar(pName, pNumberButtons)
	local count = 1
	while BAB.DB.Bars["NewBar"..count] do
		count = count + 1
	end
    local tBarname = pName or "NewBar"..count

    BAB:CreateBar(tBarname, (UIParent:GetWidth() / 2) - (BAB.DB.Settings.BarWidth / 2), 500, tBarname)
    BAB:SetupBar(tBarname)
    BAB:AddSection(tBarname)
    BAB.DB.Bars[tBarname].buttonWidth = 50
    BAB.DB.Bars[tBarname].buttonHeight = 50
	if not pNumberButtons then pNumberButtons = 12 end
	for x = 1, pNumberButtons do
		BAB:CreateButton(tBarname, BAB:CreateExtraButton(tBarname, x):GetName(), x) 
	end
	BAB:ClearLines()
    BAB:SetupSection(tBarname)
    BAB:CalculatePoints(tBarname)
    BAB:DrawPoints(tBarname)
    BAB:SetButtonsOnBar(tBarname)
    BAB:RotateButtonsOnBar(tBarname)
    for b, bv in pairs(BAB.DB.Bars[tBarname].Buttons) do
        BAB:SetButtonShape(bv)
    end

	BAB:SelectBar()
            
    BAB:SaveProfile("add bar", BABAurasCurrentProfile)
    BAB:LoadProfile(BABAurasCurrentProfile)
	return tBarname
end


function BAB:RemoveBar(pBarname, pIsPlayerInitiated)
	
	if BAB:GetNumberOfBars() <= 1 and pIsPlayerInitiated then
		print(BAB.L["BAB: You can't remove the last bar."])
		return false
	end

	if BAB.DB.Bars[pBarname] then
		for i, v in pairs(BAB.DB.Bars[pBarname].Buttons) do
			v.frame.ag:Stop()
		end
		BAB:HideBar(pBarname)
		BAB.DB.Bars[pBarname].Buttons = nil
		BAB.DB.Bars[pBarname].Sections = nil
		BAB.DB.Bars[pBarname] = nil
		BAB:SelectBar()

		BAB:SaveProfile("remove bar", BABAurasCurrentProfile)
		BAB:LoadProfile(BABAurasCurrentProfile)
	end
end
function BAB:AddButton(pBarname)
	if BAB.DB.Bars[pBarname] then
		BAB.DB.Bars[pBarname].update = true
		local tButton = BAB:CreateExtraButton(pBarname, BAB:GetNumberOfButtons(pBarname) + 1)
		tButton:Hide()
		BAB:CreateButton(pBarname, tButton:GetName(), BAB:GetNumberOfButtons(pBarname) + 1) 
		BAB:SetButtonsOnBar(pBarname)
		BAB:RotateButtonsOnBar(pBarname)
		BAB:SaveProfile("add but")
	end
end
----
function BAB:SetupSection(pBarName, pSectionNumber)
    if BAB.DB.MainOptions.configMode == false then
        for b, a in pairs(BAB.DB.Bars) do
            if a.Sections then
                for i, v in pairs(a.Sections) do
                    v.Knot1.frame:Hide()
                    v.Knot2.frame:Hide()
                    v.Vector1.frame:Hide()
                    v.Vector2.frame:Hide()
                    v.Vector1.lineToKnot:Hide()
                    v.Vector2.lineToKnot:Hide()
                    v.Vector1.lineToVector2:Hide()
                end
            end
        end
    else
        local a, r, g, b = BAB.DB.Settings.VectorLineColor.a, BAB.DB.Settings.VectorLineColor.r, BAB.DB.Settings.VectorLineColor.g, BAB.DB.Settings.VectorLineColor.b
		local tBarFrame = BAB.DB.Bars[pBarName].frame
        if BAB.DB.Bars[pBarName].sectionCount then
			for i = 1, BAB.DB.Bars[pBarName].sectionCount do
				local v = BAB.DB.Bars[pBarName].Sections[i]
				v.Knot1.frame:ClearAllPoints()
				v.Knot2.frame:ClearAllPoints()
				v.Vector1.frame:ClearAllPoints()
				v.Vector2.frame:ClearAllPoints()
				v.Knot1.frame:SetPoint("CENTER", tBarFrame, "BOTTOMLEFT", v.Knot1.x, v.Knot1.y)
				v.Knot2.frame:SetPoint("CENTER", tBarFrame, "BOTTOMLEFT", v.Knot2.x, v.Knot2.y)
				v.Vector1.frame:SetPoint("CENTER", tBarFrame, "BOTTOMLEFT", v.Vector1.x, v.Vector1.y)
				v.Vector2.frame:SetPoint("CENTER", tBarFrame, "BOTTOMLEFT", v.Vector2.x, v.Vector2.y)
				v.Knot1.frame:Show()
				v.Knot2.frame:Show()
				v.Vector1.frame:Show()
				v.Vector2.frame:Show()

				local tAlphaBack, tAlphaBorder = 0, 0
				if BAB.DB.MainOptions.selectedBar == pBarName then
					tAlphaBack, tAlphaBorder = 1, 1
				else
					tAlphaBack, tAlphaBorder = 0.3, 0.5
				end

				if BAB.DB.Bars[pBarName].calcStyle == 2 then
					if i == 1 then
						v.Knot1.frame:SetBackdropColor(0, 1,0, tAlphaBack)
					else
						v.Knot1.frame:SetBackdropColor(1, 0,0, tAlphaBack)
					end
					if i == BAB.DB.Bars[pBarName].sectionCount then
						v.Knot2.frame:SetBackdropColor(0, 1,0, tAlphaBack)
					else
						v.Knot2.frame:SetBackdropColor(1, 0,0, tAlphaBack)
					end
				else
					v.Knot1.frame:SetBackdropColor(0, 1,0, tAlphaBack)
					v.Knot2.frame:SetBackdropColor(0, 1,0, tAlphaBack)
				end
				v.Vector1.frame:SetBackdropColor(1, 0,0, tAlphaBack)
				v.Vector2.frame:SetBackdropColor(1, 0,0, tAlphaBack)

				v.Knot1.frame:SetBackdropBorderColor(1, 1,1, tAlphaBorder)
				v.Knot2.frame:SetBackdropBorderColor(1, 1,1, tAlphaBorder)
				v.Vector1.frame:SetBackdropBorderColor(1, 1,1, tAlphaBorder)
				v.Vector2.frame:SetBackdropBorderColor(1, 1,1, tAlphaBorder)
				BAB:DrawLine(v.Knot1.x, v.Knot1.y, v.Vector1.x, v.Vector1.y, 1, a, r, g, b, tBarFrame, v.Vector1.lineToKnot)
				BAB:DrawLine(v.Knot2.x, v.Knot2.y, v.Vector2.x, v.Vector2.y, 1, a, r, g, b, tBarFrame, v.Vector2.lineToKnot)
				BAB:DrawLine(v.Vector1.x, v.Vector1.y, v.Vector2.x, v.Vector2.y, 1, a, r, g, b, tBarFrame, v.Vector1.lineToVector2)

			end
		end
    end
end
----
function BAB:RemoveSection(pBarName)
    if BAB.DB.Bars[pBarName].sectionCount then
		if BAB.DB.Bars[pBarName].sectionCount <= 1 then
			print(BAB.L["BAB: You can't remove the last section from a bar."])
			return false
		end

        if BAB.DB.Bars[pBarName].sectionCount > 1 then -- at least one section must be there
            local tSection = BAB.DB.Bars[pBarName].Sections[BAB.DB.Bars[pBarName].sectionCount]
            tSection.Knot1.frame:ClearAllPoints()
            tSection.Knot2.frame:ClearAllPoints()
            tSection.Vector1.frame:ClearAllPoints()
            tSection.Vector2.frame:ClearAllPoints()
            tSection.Knot1.frame:Hide()
            tSection.Knot2.frame:Hide()
            tSection.Vector1.frame:Hide()
            tSection.Vector2.frame:Hide()
            tSection.Vector1.lineToKnot:Hide()
            tSection.Vector2.lineToKnot:Hide()
            tSection.Vector1.lineToVector2:Hide()
            table.remove(BAB.DB.Bars[pBarName].Sections, BAB.DB.Bars[pBarName].sectionCount)
            BAB.DB.Bars[pBarName].sectionCount = BAB.DB.Bars[pBarName].sectionCount - 1
            BAB:SetupSection(pBarName)
            BAB:CalculatePoints(pBarName)
            BAB:DrawPoints(pBarName)
            BAB:RotateButtonsOnBar(pBarName)
        end
    end
	if BABAurasOptionSettings.BarOptions.frame and _G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"] and BAB.DB.MainOptions.selectedBar and BAB.DB.Bars[BAB.DB.MainOptions.selectedBar] then
		if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) <= BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].minSections then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Disable()
		end
		if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) > 7 then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Enable()
		end
	end
end
----
function BAB:AddSection(pBarName)
    if not BAB.DB.Bars[pBarName] then return false end
	if BAB.DB.Bars[pBarName].sectionCount and BAB.DB.Bars[pBarName].sectionCount >= 8 then
		print(BAB.L["BAB: You can't add more than 8 sections to a bar."])
		return false
	end

    local pSectionNumber
    local tBarFrame = BAB.DB.Bars[pBarName].frame
	local tBar = BAB.DB.Bars[pBarName]

    if not tBar.Sections then
        pSectionNumber = 1
        tBar.Sections = {}
        tinsert(tBar.Sections, 1, {
            Knot1 = {
                x = 0, 
                y = (BAB.DB.Settings.BarHeight / 2) - 1, 
                frame = BAB:CreateKnotFrame(tBarFrame, "Knot1", pSectionNumber), 
                bar = pBarName, 
                section = pSectionNumber, 
                isMoving = false,
                knot1 = true,
                }, 
            Knot2 = {
                x = tBar.width, 
                y = BAB.DB.Settings.BarHeight / 2, 
                frame = BAB:CreateKnotFrame(tBarFrame, "Knot2", pSectionNumber), 
                bar = pBarName, 
                section = pSectionNumber, 
                isMoving = false,
                },
            Vector1 = {
                x = tBar.width / 5, 
                y = BAB.DB.Settings.BarHeight / 2, 
                frame = BAB:CreateVectorFrame(tBarFrame, "Vector1", pSectionNumber, pBarName), 
                bar = pBarName, 
                section = pSectionNumber,
                lineToKnot = _G[tBarFrame:GetName()..pSectionNumber.."Vector1ToKnot"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector1ToKnot","ARTWORK"),
                lineToVector2 = _G[tBarFrame:GetName()..pSectionNumber.."Vector1ToV2"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector1ToV2","OVERLAY"),
                isMoving = false
                },
            Vector2 = {
                x = tBar.width - (tBar.width / 5), 
                y = BAB.DB.Settings.BarHeight / 2, 
                frame = BAB:CreateVectorFrame(tBarFrame, "Vector2", pSectionNumber, pBarName), 
                bar = pBarName, 
                section = pSectionNumber,
                lineToKnot = _G[tBarFrame:GetName()..pSectionNumber.."Vector2ToKnot"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector2ToKnot","OVERLAY"),
                isMoving = false
                }
        })
    else

        

        pSectionNumber = getn(tBar.Sections) + 1
        
		tBar.Sections[pSectionNumber - 1].Knot2.frame.db.isMoving = true 
		tBar.Sections[pSectionNumber - 1].Knot2.frame:GetScript("OnUpdate")(tBar.Sections[pSectionNumber - 1].Knot2.frame, true, true) 
        tBar.Sections[pSectionNumber - 1].Knot2.frame.db.isMoving = false 


		local tCenterXV1, tCenterYV1 = BAB:FindPointOnLine(
														tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y, 
														BAB:LineLenght(tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y) +
														40)
		local tCenterXV2, tCenterYV2 = BAB:FindPointOnLine(
														tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y, 
														BAB:LineLenght(tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y) +
														80)
		local tCenterXK2, tCenterYK2 = BAB:FindPointOnLine(
														tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y, 
														BAB:LineLenght(tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y) +
														120)

		if BAB.DB.Bars[pBarName].calcStyle == 1 then
			tCenterXV1 = ((tBar.Sections[pSectionNumber - 1].Vector2.x - tBar.Sections[pSectionNumber - 1].Knot2.x) * -1)	+ tBar.Sections[pSectionNumber - 1].Knot2.x
			tCenterYV1 = ((tBar.Sections[pSectionNumber - 1].Vector2.y - tBar.Sections[pSectionNumber - 1].Knot2.y) * -1)	+ tBar.Sections[pSectionNumber - 1].Knot2.y

			tCenterXV2, tCenterYV2 = BAB:FindPointOnLine(
														tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y, 
														BAB:LineLenght(tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y) +
														220)
			tCenterXK2, tCenterYK2 = BAB:FindPointOnLine(
														tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y, 
														BAB:LineLenght(tBar.Sections[pSectionNumber - 1].Vector2.x, 
														tBar.Sections[pSectionNumber - 1].Vector2.y, 
														tBar.Sections[pSectionNumber - 1].Knot2.x, 
														tBar.Sections[pSectionNumber - 1].Knot2.y) +
														300)
		end

        tinsert(tBar.Sections, getn(tBar.Sections) + 1, 
        {
            Knot1 =  {
                x = tBar.Sections[pSectionNumber - 1].Knot2.x, 
                y = tBar.Sections[pSectionNumber - 1].Knot2.y, 
                frame = BAB:CreateKnotFrame(tBarFrame, "Knot1", pSectionNumber), 
                bar = pBarName, 
                section = pSectionNumber, 
                isMoving = false,
                knot1 = true,
                },
            Knot2 = {
                x = tCenterXK2, 
                y = tCenterYK2, 
                frame = BAB:CreateKnotFrame(tBarFrame, "Knot2", pSectionNumber), 
                bar = pBarName, 
                section = pSectionNumber, 
                isMoving = false,
                },
            Vector1 = {
                x =	tCenterXV1,
                y =	tCenterYV1,
                frame = BAB:CreateVectorFrame(tBarFrame, "Vector1", pSectionNumber, pBarName),
                bar = pBarName, section = pSectionNumber, 
                lineToKnot = _G[tBarFrame:GetName()..pSectionNumber.."Vector1ToKnot"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector1ToKnot","OVERLAY"),
                lineToVector2 = _G[tBarFrame:GetName()..pSectionNumber.."Vector1ToV2"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector1ToV2","OVERLAY"),
                isMoving = false
                },
            Vector2 = {
                x = tCenterXV2,  
                y = tCenterYV2, 
                frame = BAB:CreateVectorFrame(tBarFrame, "Vector2", pSectionNumber, pBarName), 
                bar = pBarName, section = pSectionNumber, 
                lineToKnot = _G[tBarFrame:GetName()..pSectionNumber.."Vector2ToKnot"] or tBar.overlayFrame:CreateTexture(tBarFrame:GetName()..pSectionNumber.."Vector2ToKnot","OVERLAY"),
                isMoving = false
                }
        })
    end

    tBar.Sections[pSectionNumber].Knot1.frame.db = tBar.Sections[pSectionNumber].Knot1
    tBar.Sections[pSectionNumber].Knot2.frame.db = tBar.Sections[pSectionNumber].Knot2
    tBar.Sections[pSectionNumber].Vector1.frame.db =  tBar.Sections[pSectionNumber].Vector1
    tBar.Sections[pSectionNumber].Vector2.frame.db =  tBar.Sections[pSectionNumber].Vector2
    tBar.sectionCount = pSectionNumber
	BAB:InitializeBar(pBarName)

	if pSectionNumber > 1 and BABAurasOptionSettings.MainOptions.frame and tBar.Sections[pSectionNumber].Knot2.frame and tBar.Sections[pSectionNumber].Knot2.frame:GetScript("OnDragStart") and tBar.Sections[pSectionNumber].Knot2.frame.db then
		tBar.Sections[pSectionNumber].Knot2.frame:GetScript("OnDragStart")(tBar.Sections[pSectionNumber].Knot2.frame)
		tBar.Sections[pSectionNumber].Knot2.frame:GetScript("OnDragStop")(tBar.Sections[pSectionNumber].Knot2.frame)
	end

	if BABAurasOptionSettings.BarOptions.frame and BABAurasOptionSettings.BarOptions.frame.GetName and _G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"] and BAB.DB.MainOptions.selectedBar and BAB.DB.Bars[BAB.DB.MainOptions.selectedBar] then
		if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) > BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].minSections then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Enable()
		end
		if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) > 7 then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Enable()
		end
	end
end
----
function BAB:UpdateSections()
    for a, b in pairs(BAB.DB.Bars) do
        if b.Sections then
			local tBarFrame = b.frame
            for i, v in pairs(b.Sections) do
                v.Knot1.frame = BAB:CreateKnotFrame(tBarFrame, "Knot1", i) 
                v.Knot2.frame = BAB:CreateKnotFrame(tBarFrame, "Knot2", i) 
                v.Vector1.frame = BAB:CreateVectorFrame(tBarFrame, "Vector1", i)
                v.Vector2.frame = BAB:CreateVectorFrame(tBarFrame, "Vector2", i) 
                v.Knot1.frame.db = v.Knot1
                v.Knot2.frame.db = v.Knot2
                v.Vector1.frame.db =  v.Vector1
                v.Vector2.frame.db =  v.Vector2
				v.Vector1.lineToKnot = _G[tBarFrame:GetName()..i.."Vector1ToKnot"] or tBarFrame:CreateTexture(tBarFrame:GetName()..i.."Vector1ToKnot","OVERLAY")
                v.Vector2.lineToKnot = _G[tBarFrame:GetName()..i.."Vector2ToKnot"] or tBarFrame:CreateTexture(tBarFrame:GetName()..i.."Vector2ToKnot","OVERLAY")
                v.Vector1.lineToVector2 = _G[tBarFrame:GetName()..i.."Vector1ToV2"] or tBarFrame:CreateTexture(tBarFrame:GetName()..i.."Vector1ToV2","OVERLAY")

                v.Knot1.frame.db = v.Knot1
                v.Knot2.frame.db = v.Knot2
                v.Vector1.frame.db =  v.Vector1
                v.Vector2.frame.db =  v.Vector2
            end
        end
    end
    for a, b in pairs(BAB.DB.Bars) do
        if b.Sections then
            for i, v in pairs(b.Sections) do
                BAB:SetupSection(a, i)
            end
        end
    end
end
----
function BAB:CreateVectorFrame(tParentObj, pVectorName, pSectionNumber, pBarName)
    local tObj = _G[tParentObj:GetName()..pSectionNumber..pVectorName] or CreateFrame("Frame", tParentObj:GetName()..pSectionNumber..pVectorName ,tParentObj)

    tObj:SetFrameLevel(200)
    tObj:SetFrameStrata("HIGH")
    tObj:SetHeight(15)
    tObj:SetWidth(15)
    tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 }})
    tObj:SetBackdropColor(1, 0,0, 1)
    tObj:SetBackdropBorderColor(0, 0, 1, 1)
    tObj:EnableMouse(true)
    tObj:SetMovable(true)
    tObj:SetScript("OnDragStart", function(self) 
        self.db.isMoving = true
        self:StartMoving() 
        if  BAB.DB.MainOptions.selectedBar ~= self.db.bar then
            BAB:SelectBar(self.db.bar)
        end
		self.db.oldx  = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
		self.db.oldy  = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
    end)
    tObj:SetScript("OnDragStop", function(self) 
		BAB.DB.Bars[self.db.bar].update = true
        self:StopMovingOrSizing() 
        self.db.isMoving = false
		if BAB.DB.Bars[self.db.bar].CirclePermanent == 1 then

			local cx, cy = GetCursorPosition()
			if self.db.cx  ~= cx or self.db.cy ~= cy then
				self.db.cx = cx 
				self.db.cy = cy 
				cx = (cx / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].x
				cy = (cy / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].y
				local centerx, centery = BAB.DB.Bars[self.db.bar].width / 2, BAB.DB.Bars[self.db.bar].height / 2
				if BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
					BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx, cy) / tModifiers[getn(BAB.DB.Bars[self.db.bar].Sections)].a
					if BAB.DB.Bars[self.db.bar].CircleRadius < 20 then BAB.DB.Bars[self.db.bar].CircleRadius = 20 end
				else
					BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2)
				end
				BAB:AutoSetupBarShape(self.db.bar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[self.db.bar].CircleRadius, true) 
				BAB:SetupSection(self.db.bar)
				BAB:CalculatePoints(self.db.bar)
				BAB:DrawPoints(self.db.bar)
				BAB:RotateButtonsOnBar(self.db.bar)

				if BAB.DB.Bars[self.db.bar].autoButton == 1 then
					BAB:AutoButtons(self.db.bar)
				end
			end
		else
			if not self.db.oldx then
				self.db.oldx = floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) 
				self.db.oldy = floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2)
			end
			if floor(self.db.oldx) ~= floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) or floor(self.db.oldy) ~= floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2) or pUserInitiated then
				BAB.DB.Bars[self.db.bar].update = true
				self.db.oldx = self.db.x
				self.db.oldy = self.db.y

				self.db.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
				self.db.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2

				if getn(BAB.DB.Bars[self.db.bar].Sections) > 1 and BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
					if string.find(self.db.frame:GetName(), "Vector2") and BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1] then
						local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.x
						local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.y

						BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.x
						BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.y
					end						
					if string.find(self.db.frame:GetName(), "Vector1") and BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1] then
						local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.x
						local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.y

						BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.x
						BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.y

					end
					if BAB.DB.Bars[tParentObj.barName].linked == 1 then
						if string.find(self.db.frame:GetName(), "Vector2") and self.db.section == getn(BAB.DB.Bars[self.db.bar].Sections) then
							local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.x
							local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.y

							BAB.DB.Bars[self.db.bar].Sections[1].Vector1.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.x
							BAB.DB.Bars[self.db.bar].Sections[1].Vector1.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.y
						end						
						if string.find(self.db.frame:GetName(), "Vector1") and self.db.section == 1 then
							local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.x
							local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.y

							BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.x
							BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.y

						end
					end
				end

				BAB:SetupSection(self.db.bar)
				BAB:CalculatePoints(self.db.bar)
				--BAB:DrawPoints(self.db.bar)
				BAB:RotateButtonsOnBar(self.db.bar)

				if BAB.DB.Bars[self.db.bar].autoButton == 1 then
					BAB:AutoButtons(self.db.bar)
				end
			end
		end
        BAB:SaveProfile("v OnDragStop")
		BAB:DrawPoints(self.db.bar)
        if  BAB.DB.MainOptions.selectedBar ~= self.db.bar then
            BAB:SelectBar(self.db.bar)
        end
		BAB:EnableDisableOptions()
    end)
    tObj:SetScript("OnUpdate", function(self,  elapsed, pUserInitiated) 
        if self.db then
            if self.db.isMoving == true then
				if BAB.DB.Bars[self.db.bar].CirclePermanent == 1 then

					local cx, cy = GetCursorPosition()
					if self.db.cx  ~= cx or self.db.cy ~= cy then
						self.db.cx = cx 
						self.db.cy = cy 
						cx = (cx / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].x
						cy = (cy / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].y
						local centerx, centery = BAB.DB.Bars[self.db.bar].width / 2, BAB.DB.Bars[self.db.bar].height / 2
						if BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
							BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx, cy) / tModifiers[getn(BAB.DB.Bars[self.db.bar].Sections)].a
							if BAB.DB.Bars[self.db.bar].CircleRadius < 20 then BAB.DB.Bars[self.db.bar].CircleRadius = 20 end
						else
							BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2)
						end
						BAB:AutoSetupBarShape(self.db.bar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[self.db.bar].CircleRadius, true) 
					end
				else
					if not self.db.oldx then
						self.db.oldx = floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) 
						self.db.oldy = floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2)
					end
					if floor(self.db.oldx) ~= floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) or floor(self.db.oldy) ~= floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2) or pUserInitiated then
						BAB.DB.Bars[self.db.bar].update = true
						self.db.oldx = self.db.x
						self.db.oldy = self.db.y

						self.db.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
						self.db.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2

						if getn(BAB.DB.Bars[self.db.bar].Sections) > 1 and BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
							if string.find(self.db.frame:GetName(), "Vector2") and BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1] then
								local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.x
								local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.y

								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.x
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.y
							end						
							if string.find(self.db.frame:GetName(), "Vector1") and BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1] then
								local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.x
								local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.y

								BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.x
								BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.y

							end
							if BAB.DB.Bars[tParentObj.barName].linked == 1 then
								if string.find(self.db.frame:GetName(), "Vector2") and self.db.section == getn(BAB.DB.Bars[self.db.bar].Sections) then
									local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.x
									local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot2.frame.db.y

									BAB.DB.Bars[self.db.bar].Sections[1].Vector1.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.x
									BAB.DB.Bars[self.db.bar].Sections[1].Vector1.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot2.frame.db.y
								end						
								if string.find(self.db.frame:GetName(), "Vector1") and self.db.section == 1 then
									local tCenterX = self.db.x - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.x
									local tCenterY = self.db.y - BAB.DB.Bars[self.db.bar].Sections[self.db.section].Knot1.frame.db.y

									BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.frame.db.x = (tCenterX * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.x
									BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.frame.db.y = (tCenterY * -1) + BAB.DB.Bars[self.db.bar].Sections[self.db.section ].Knot1.frame.db.y

								end
							end
						end

						BAB:SetupSection(self.db.bar)
						BAB:CalculatePoints(self.db.bar)
						BAB:DrawPoints(self.db.bar)
						BAB:RotateButtonsOnBar(self.db.bar)
						BAB.DB.CalculatedDirectionButtonTable = nil

						if BAB.DB.Bars[self.db.bar].autoButton == 1 then
							BAB:AutoButtons(self.db.bar)
						end
						BAB:EnableDisableOptions()
					end
				end

            end
        end
    end)
    tObj:SetMovable(true)
    tObj:EnableMouse(true)
    tObj:SetClampedToScreen(true)
    tObj:RegisterForDrag("LeftButton")

    return tObj
end
----
function BAB:CreateKnotFrame(tParentObj, tKnotName, pSectionNumber)

    local tObj = _G[tParentObj:GetName()..pSectionNumber..tKnotName] or CreateFrame("Frame", tParentObj:GetName()..pSectionNumber..tKnotName ,tParentObj)

    tObj:SetFrameStrata("HIGH")
    tObj:SetFrameLevel(200)

	if ((pSectionNumber == 1 and tKnotName == "Knot1") or (pSectionNumber == BAB.DB.Bars[tParentObj.barName].sectionCount and tKnotName == "Knot2")) and BAB.DB.Bars[tParentObj.barName].calcStyle == 2 then
	    tObj:SetHeight(15)
	    tObj:SetWidth(15)
		tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 }})
		tObj:SetBackdropColor(0, 1,0, 1)
	else
		tObj:SetHeight(15)
		tObj:SetWidth(15)
		tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 }})
		tObj:SetBackdropColor(1, 0,0, 1)
		tObj:SetBackdropBorderColor(0, 0, 1, 1)
	end

    tObj:EnableMouse(true)
    tObj:SetMovable(true)
    tObj:SetScript("OnDragStart", function(self) 
        self.db.isMoving = true
        self:StartMoving() 
        if  BAB.DB.MainOptions.selectedBar ~= self.db.bar then
            BAB:SelectBar(self.db.bar)
        end
        self.db.oldx  = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
        self.db.oldy  = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
    end)
    tObj:SetScript("OnDragStop", function(self) 
        self.db.isMoving = false
		BAB.DB.Bars[self.db.bar].update = true
        self:StopMovingOrSizing() 
        self.db.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
        self.db.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
        self.db.oldx  = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
        self.db.oldy  = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
        if self.db.section > 1 and string.find(self.db.frame:GetName(), "Knot1") then
            BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Knot2.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
            BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Knot2.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
        else
            if string.find(self.db.frame:GetName(), "Knot2") and BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1] then
                BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Knot1.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
                BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Knot1.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
            end
        end

		if BAB.DB.Bars[self.db.bar].CirclePermanent == 1 and not pUserInitiated then
			local cx, cy = GetCursorPosition()
			cx = (cx / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].x
			cy = (cy / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].y
			local centerx, centery = BAB.DB.Bars[self.db.bar].width / 2, BAB.DB.Bars[self.db.bar].height / 2
			if BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
				BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2)
			else
				BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2) * tModifiers[getn(BAB.DB.Bars[self.db.bar].Sections)].a
			end
			if BAB.DB.Bars[self.db.bar].CircleRadius < 20 then BAB.DB.Bars[self.db.bar].CircleRadius = 20 end
			BAB:AutoSetupBarShape(self.db.bar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[self.db.bar].CircleRadius) 
		end

        BAB:SetupSection(self.db.bar)
        BAB:CalculatePoints(self.db.bar)
        --BAB:DrawPoints(self.db.bar, true)
        BAB:RotateButtonsOnBar(self.db.bar)
        BAB:SaveProfile("k OnDragstop")
		BAB:DrawPoints(self.db.bar, true)
        if  BAB.DB.MainOptions.selectedBar ~= self.db.bar then
            BAB:SelectBar(self.db.bar)
        end
		BAB:EnableDisableOptions()
    end)
    tObj:SetScript("OnUpdate", function(self, elapsed, pUserInitiated) 
        if self.db then
            if self.db.isMoving == true then
				if BAB.DB.Bars[self.db.bar].CirclePermanent == 1 and not pUserInitiated then
					local cx, cy = GetCursorPosition()
					if self.db.cx  ~= cx or self.db.cy ~= cy then
						self.db.cx = cx 
						self.db.cy = cy 
						cx = (cx / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].x
						cy = (cy / UIParent:GetEffectiveScale() ) - BAB.DB.Bars[self.db.bar].y
						local centerx, centery = BAB.DB.Bars[self.db.bar].width / 2, BAB.DB.Bars[self.db.bar].height / 2

						if BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
							BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2)
						else
							BAB.DB.Bars[self.db.bar].CircleRadius = BAB:LineLenght(centerx, centery, cx + self:GetWidth() / 2, cy - self:GetHeight() / 2) * tModifiers[getn(BAB.DB.Bars[self.db.bar].Sections)].a
						end
						if BAB.DB.Bars[self.db.bar].CircleRadius < 20 then BAB.DB.Bars[self.db.bar].CircleRadius = 20 end


						if BAB.DB.Bars[self.db.bar].CircleRadius < 20 then BAB.DB.Bars[self.db.bar].CircleRadius = 20 end
						BAB:AutoSetupBarShape(self.db.bar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[self.db.bar].CircleRadius) 
					end
				else
					if not self.db.oldx then
						self.db.oldx = floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) 
						self.db.oldy = floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2)
					end
					if floor(self.db.oldx) ~= floor(self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2) or floor(self.db.oldy) ~= floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2)  or pUserInitiated then
						BAB.DB.Bars[self.db.bar].update = true
						self.db.oldx = self.db.x
						self.db.oldy = self.db.y


						self.db.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
						self.db.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2

						if self.db.section > 1 and string.find(self.db.frame:GetName(), "Knot1") then
							BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Knot2.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
							BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Knot2.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
						else
							if string.find(self.db.frame:GetName(), "Knot2") and BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1] then
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Knot1.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Knot1.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
							end
						end

						if BAB.DB.Bars[tParentObj.barName].linked == 1 then
							if self.db.section == 1 and string.find(self.db.frame:GetName(), "Knot1") then
								BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Knot2.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
								BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Knot2.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
							end
							if self.db.section == BAB.DB.Bars[self.db.bar].Sections and  string.find(self.db.frame:GetName(), "Knot2") then
								BAB.DB.Bars[self.db.bar].Sections[1].Knot1.x = self:GetLeft() - self:GetParent():GetLeft() + self:GetWidth() / 2
								BAB.DB.Bars[self.db.bar].Sections[1].Knot1.y = self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight() - self:GetHeight() / 2
							end
						end

						if getn(BAB.DB.Bars[self.db.bar].Sections) > 1 and BAB.DB.Bars[tParentObj.barName].calcStyle == 1 then
							local xDif, yDif = self.db.oldx - self.db.x, self.db.oldy - self.db.y

							if self.db.section == 1 and string.find(self.db.frame:GetName(), "Knot2") then
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.y - yDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.y - yDif
							end
							if self.db.section > 1 and string.find(self.db.frame:GetName(), "Knot1") then
								BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section - 1].Vector2.y - yDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector1.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector1.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector1.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector1.y - yDif
							end
							if self.db.section > 1 and self.db.section < getn(BAB.DB.Bars[self.db.bar].Sections) and string.find(self.db.frame:GetName(), "Knot2") then
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section + 1].Vector1.y - yDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.x = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.x - xDif
								BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.y = BAB.DB.Bars[self.db.bar].Sections[self.db.section].Vector2.y - yDif
							end
							if BAB.DB.Bars[tParentObj.barName].linked == 1 then
								if (self.db.section == 1 and string.find(self.db.frame:GetName(), "Knot1")) or (self.db.section == getn(BAB.DB.Bars[self.db.bar].Sections) and string.find(self.db.frame:GetName(), "Knot2")) then
									BAB.DB.Bars[self.db.bar].Sections[1].Vector1.x = BAB.DB.Bars[self.db.bar].Sections[1].Vector1.x - xDif
									BAB.DB.Bars[self.db.bar].Sections[1].Vector1.y = BAB.DB.Bars[self.db.bar].Sections[1].Vector1.y - yDif
									BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.x = BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.x - xDif
									BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.y = BAB.DB.Bars[self.db.bar].Sections[getn(BAB.DB.Bars[self.db.bar].Sections)].Vector2.y - yDif
								end
							end
						end

						BAB:SetupSection(self.db.bar)
						BAB:CalculatePoints(self.db.bar)
						BAB:DrawPoints(self.db.bar)
						BAB:RotateButtonsOnBar(self.db.bar)
						BAB.DB.CalculatedDirectionButtonTable = nil

						if BAB.DB.Bars[self.db.bar].autoButton == 1 then
							BAB:AutoButtons(self.db.bar)
						end
						BAB:EnableDisableOptions()
					end
				end



            end
        end
    end)
    tObj:SetMovable(true)
    tObj:EnableMouse(true)
    tObj:SetClampedToScreen(true)
    tObj:RegisterForDrag("LeftButton")

    return tObj
end
----
function BAB:ShowBar(pBarName)
    if not pBarName then
        for i, v in pairs(BAB.DB.Bars) do
            v.frame:Show()
        end
    else
        if BAB.DB.Bars[pBarName] then
            BAB.DB.Bars[pBarName].frame:Show()
        end
    end
end
----
--function BAB:HideBar(pBarName)
    --if not pBarName then
        --for i, v in pairs(BAB.DB.Bars) do
            --v.frame:Hide()
        --end
    --else
        --if BAB.DB.Bars[pBarName] then
            --BAB.DB.Bars[pBarName].frame:Hide()
        --end
    --end
--end
---
local PreCalc = {["sin"] = {}, ["cos"] = {}} 
do 
    for x = -720, 720 do 
        PreCalc.sin[x] = sin(x) 
        PreCalc.cos[x] = cos(x) 
    end 
end
function BAB:RotateButtonsOnBar(pBarName, pButtonNr)
	if BAB.DB.Bars[pBarName].isActive == 0 then
		local tStart = 1
		local tEnd = getn(BAB.DB.Bars[pBarName].Buttons) --BAB:GetNumberOfButtons(pBarName)
		if pButtonNr then
			tStart = pButtonNr
			tEnd = tStart
		end
		for x = tStart, tEnd  do
			local tButton = BAB.DB.Bars[pBarName].Buttons[x]
		end
	else

		local tStart = 1
		local tEnd = BAB:GetNumberOfButtons(pBarName)
		--tEnd = BAB:GetMaxButtonsOnBar(pBarName)
		if pButtonNr then
			tStart = pButtonNr 
			tEnd = tStart
		end
		for x = tStart, tEnd  do
			local tButton = BAB.DB.Bars[pBarName].Buttons[x]
			local tFrame = tButton.frame
			local tDegree = BAB.DB.Bars[pBarName].Buttons[x].degree

			if tFrame.ag:IsPaused() == true then 
				tFrame.ag:Stop() 
				tFrame.a1:Stop() 
			end
		
			tFrame.a1:SetDegrees(tDegree) 
			tFrame:SetAttribute("degree", tDegree)
			--tButton.BABCenterFS:SetText(tDegree)
			local tHK, tN, tC = _G[tFrame:GetName().."HotKey"], _G[tFrame:GetName().."Name"], _G[tFrame:GetName().."Count"]
			if tHK  then
				local tsx, tsy, tex, tey = 0, 0, BAB.DB.Bars[tButton.bar].buttonWidth / 5, BAB.DB.Bars[tButton.bar].buttonWidth / 3
				local new_x = floor(tsx + PreCalc.cos[tDegree] * (tex - tsx) - PreCalc.sin[tDegree] * (tey - tsy))
				local new_y = floor(tsy + PreCalc.sin[tDegree] * (tex - tsx) + PreCalc.cos[tDegree] * (tey - tsy))
				tHK:ClearAllPoints()
				tHK:SetPoint("CENTER", tFrame, "CENTER", new_x, new_y) 
			end
			if tN then
				local tsx, tsy, tex, tey = 0, 0, BAB.DB.Bars[tButton.bar].buttonWidth / 8, BAB.DB.Bars[tButton.bar].buttonWidth / 4
				local new_x = floor(tsx + PreCalc.cos[tDegree + 180] * (tex - tsx) - PreCalc.sin[tDegree + 180] * (tey - tsy))
				local new_y = floor(tsy + PreCalc.sin[tDegree + 180] * (tex - tsx) + PreCalc.cos[tDegree + 180] * (tey - tsy))
				tN:ClearAllPoints()
				tN:SetPoint("CENTER", tFrame, "CENTER", new_x, new_y) 
			end
			if tC then
				local tsx, tsy, tex, tey = 0, 0, BAB.DB.Bars[tButton.bar].buttonWidth / 4, BAB.DB.Bars[tButton.bar].buttonWidth / 4
				local new_x = floor(tsx + PreCalc.cos[tDegree + 270] * (tex - tsx) - PreCalc.sin[tDegree + 270] * (tey - tsy))
				local new_y = floor(tsy + PreCalc.sin[tDegree + 270] * (tex - tsx) + PreCalc.cos[tDegree + 270] * (tey - tsy))
				tC:ClearAllPoints()
				tC:SetPoint("CENTER", tFrame, "CENTER", new_x, new_y) 
			end
			if tFrame.ag:IsPlaying() == false then 
				tFrame.ag:Play() 
				tFrame.a1:Play() 
			end
			
		end


	end
end
---
function BAB:CleanPoints(pBarName)
    --local nPoints = {}
    --local ox, oy = BAB.DB.Bars[pBarName].Points[1].x, BAB.DB.Bars[pBarName].Points[1].y
    --for x = 1, getn(BAB.DB.Bars[pBarName].Points) do
		--local tPoint = BAB.DB.Bars[pBarName].Points[x]
--
		--if tPoint.x ~= ox or tPoint.y  ~= oy then
--
			--local tDiffx, tDify = 0, 0
			--if tPoint.x < ox then tDiffx = ox - tPoint.x else tDiffx = tPoint.x - ox end
			--if tPoint.y < oy then tDiffy = oy - tPoint.y else tDiffy = tPoint.y - oy end
			--if tDiffx > 1 or tDiffy > 1 then 
				----print(x, tDiffx, tDiffy) 
				--BAB:DrawVirtualLine(ox, oy, tPoint.x, tPoint.y, nPoints)	
			--else
	            --tinsert(nPoints, tPoint)
			--end
        --end
		--
        --ox = tPoint.x 
        --oy = tPoint.y
    --end
	----print(getn(nPoints), getn(BAB.DB.Bars[pBarName].Points))
    --BAB.DB.Bars[pBarName].Points = nPoints
end
---
function BAB:GetMaxButtonsOnBar(pBarName)
	if BAB.DB.Bars[pBarName].buttonWidth then

		local tNumberPoints = getn(BAB.DB.Bars[pBarName].Points)
		local tWidth = BAB.DB.Bars[pBarName].buttonWidth 
		local oStart = 1 
		local oEnd = tWidth 
		local tCount = 0
		local tMaxLength = (tNumberPoints - 1)
		local tCounter = 2

		while tCounter < tMaxLength do
			if BAB:LineLenght(BAB.DB.Bars[pBarName].Points[oStart].x, BAB.DB.Bars[pBarName].Points[oStart].y, BAB.DB.Bars[pBarName].Points[tCounter].x, BAB.DB.Bars[pBarName].Points[tCounter].y) >= tWidth + BAB.DB.Bars[pBarName].buttonPadding then
				tCount = tCount + 1
				oStart = tCounter
			end
			tCounter = tCounter + 1
		end

		return tCount 
	else
		return getn(BAB.DB.Bars[pBarName].Buttons)
	end
end
--- 
local function CalculateBeta(tsx, tsy, tex, tey, pBarName)
    return -(floor(deg(atan2(tex - tsx, tey - tsy) ) - 90))
end
--
function BAB:SetButtonsOnBar(pBarName, pFin)
	if BAB.DB.Bars[pBarName].isActive == 1 then --and BAB.DB.Bars[pBarName].update == true then
		local tNumberPoints = getn(BAB.DB.Bars[pBarName].Points)
		local tWidth = BAB.DB.Bars[pBarName].buttonWidth 
		local oStart = 1 
		local oEnd = tWidth 

		local tHigh = 1
		local tLow = 1

		if BAB:GetNumberOfButtons(pBarName) > 0 then
			for x = 1, getn(BAB.DB.Bars[pBarName].Buttons)  do
				BAB.DB.Bars[pBarName].Buttons[x].frame:Hide()
				if oStart < (tNumberPoints - BAB.DB.Bars[pBarName].buttonPadding - tWidth - 1) then
					local tPs = oStart
					for y = oStart, tNumberPoints   do
						if BAB:LineLenght(BAB.DB.Bars[pBarName].Points[tPs].x, BAB.DB.Bars[pBarName].Points[tPs].y, BAB.DB.Bars[pBarName].Points[y].x, BAB.DB.Bars[pBarName].Points[y].y) >= tWidth + BAB.DB.Bars[pBarName].buttonPadding then
							oEnd = y
							break
						end
					end

					if BAB.DB.Bars[pBarName].Buttons[x] and oEnd + (floor((oEnd - oStart)  / 2)) < (tNumberPoints  ) and oEnd > oStart then
						if BAB.DB.Bars[pBarName].Buttons[x].unused == false or BAB.DB.Bars[pBarName].autoButton == 1 then
							BAB.DB.Bars[pBarName].Buttons[x].unused = false 
							BAB.DB.Bars[pBarName].Buttons[x].x = BAB.DB.Bars[pBarName].Points[oEnd ].x 
							BAB.DB.Bars[pBarName].Buttons[x].y = BAB.DB.Bars[pBarName].Points[oEnd ].y
							BAB.DB.Bars[pBarName].Buttons[x].degree = floor(CalculateBeta( 
								BAB.DB.Bars[pBarName].Points[oStart + (floor((oEnd - oStart)  / 2))].x,
								BAB.DB.Bars[pBarName].Points[oStart + (floor((oEnd - oStart)  / 2))].y,
								BAB.DB.Bars[pBarName].Points[oEnd + (floor((oEnd - oStart)  / 2))].x,
								BAB.DB.Bars[pBarName].Points[oEnd + (floor((oEnd - oStart)  / 2))].y,
								pBarName
							))

							local tTmp
							if BAB.DB.Bars[pBarName].Buttons[x - 1] then
								if BAB.DB.Bars[pBarName].Buttons[x - 1].degree < 0  and BAB.DB.Bars[pBarName].Buttons[x].degree > 0 then
									tTmp = BAB.DB.Bars[pBarName].Buttons[x].degree - (360 + BAB.DB.Bars[pBarName].Buttons[x - 1].degree)
								else
									tTmp = BAB.DB.Bars[pBarName].Buttons[x].degree - BAB.DB.Bars[pBarName].Buttons[x - 1].degree
								end
							elseif BAB.DB.Bars[pBarName].Buttons[x + 1] then
								if BAB.DB.Bars[pBarName].Buttons[x].degree < 0  and BAB.DB.Bars[pBarName].Buttons[x + 1].degree > 0 then
									tTmp = BAB.DB.Bars[pBarName].Buttons[x + 1].degree - (360 + BAB.DB.Bars[pBarName].Buttons[x].degree)
								else
									tTmp = BAB.DB.Bars[pBarName].Buttons[x + 1].degree - BAB.DB.Bars[pBarName].Buttons[x].degree
								end
							else
								tTmp = 0
							end
							if tTmp < -90 then tTmp = 360 + tTmp end
							BAB.DB.Bars[pBarName].Buttons[x].shapeMod = tTmp

							

							--local ltMod = -(tTmp / 360) * 3
							--if ltMod < 0 then ltMod = ltMod * -1 end
							--local tButtonsSizeModificator = BAB.DB.MainOptions.MaxShapeFactor / ltMod
							--if tButtonsSizeModificator < tLow then tLow = tButtonsSizeModificator end
							--if tButtonsSizeModificator > tHigh then tHigh = tButtonsSizeModificator end

							BAB:SetupButton(pBarName, x, pFin)
						
							BAB:ShowButton(pBarName, x)
						end
					else
						if BAB.DB.Bars[pBarName].Buttons[x].unused == false then BAB:ResetButton(BAB.DB.Bars[pBarName].Buttons[x]) end
					end
					oStart = oEnd
				else
					if BAB.DB.Bars[pBarName].Buttons[x].unused == false then BAB:ResetButton(BAB.DB.Bars[pBarName].Buttons[x]) end
				end
			end
		end
--BAB.DB.Bars[pBarName].smoothe = 1
		if BAB.DB.Bars[pBarName].smoothe == 1 then BAB:SmootheButtons(pBarName) end
		BAB.DB.Bars[pBarName].update = false

--if tLow < 0 then tLow = 1 end
--if tLow > 1 then tLow = 1 end
--print(50 * tLow)
--if floor(50 * tLow) > 20 and floor(50 * tLow) < 50 then
	--BAB.DB.Bars[pBarName].buttonWidth = floor(50 * tLow)
	--BAB.DB.Bars[pBarName].buttonHeight = BAB.DB.Bars[pBarName].buttonWidth
	--BAB:CalculatePoints(pBarName)
	--BAB:DrawPoints(pBarName)
	--BAB:RotateButtonsOnBar(pBarName)
----elseif tLow == 1 then
	----BAB.DB.Bars[pBarName].buttonWidth = 50
--end
--elseif tHigh and tHigh > 1 then
	--
	----if floor(BAB.DB.Bars[pBarName].buttonWidth * tHigh) > 50 then
		----tHigh = 50 / BAB.DB.Bars[pBarName].buttonWidth
		----BAB.DB.Bars[pBarName].buttonWidth = floor(BAB.DB.Bars[pBarName].buttonWidth * tHigh)
		----BAB.DB.Bars[pBarName].buttonHeight = BAB.DB.Bars[pBarName].buttonWidth
		----BAB:CalculatePoints(pBarName)
		----BAB:DrawPoints(pBarName)
		----BAB:RotateButtonsOnBar(pBarName)
	----end
	----
--end


	end
	


		--BAB.DB.Bars[pBarName].buttonWidth = floor(BAB.DB.Bars[pBarName].buttonWidth * tButtonSizeModificator)
		--BAB.DB.Bars[pBarName].buttonHeight = BAB.DB.Bars[pBarName].buttonWidth
--
		--BAB:CalculatePoints(pBarName)
		--BAB:DrawPoints(pBarName)
		--BAB:RotateButtonsOnBar(pBarName)
	--
	
	--return tHighestButtonSizeModificator
end
---
function BAB:ResetButton(pButton)
	pButton.x = 0 
	pButton.y = 0
	pButton.degree = 0
	pButton.shapeMod = 0
	BAB:HideButton(pButton.bar, pButton.number)
	pButton.frame.ag:Stop()
	BAB:HideButton(pButton.bar, pButton.number)
	pButton.unused = true
end
---- 
function BAB:SmootheButtons(pBarName)
	if BAB.DB.Bars[pBarName].isActive == 1 then

		local a = {}
		for	x = 1, getn(BAB.DB.Bars[pBarName].Buttons)  do
			tinsert(a, BAB.DB.Bars[pBarName].Buttons[x].degree)
		end

		a = BAB:smooth(a)

		for i = 1,#a do
		   BAB.DB.Bars[pBarName].Buttons[i].degree = a[i]
		end

	end
end
---
function BAB:smooth_helper(a, b, weight)
   local direction = 1
   if (a-b)%360 < 180 then
      direction = -1
   end
   
   local delta = math.abs(a-b) % 360
   if delta > 180 then 
      delta = 360 - delta
   end
   
   return (a + direction * delta * weight) % 360
end
--
function BAB:round(n)
   return floor(n+0.5)
end
--
function BAB:smooth(tbl) 
   ret =  {}
   local avg = 0
   
   local tmp = {tbl[1], tbl[1]}
   for i=1,#tbl do
      tinsert (tmp, tbl[i])
   end
   tinsert (tmp, tbl[#tbl])
   tinsert (tmp, tbl[#tbl])
   
   for i=3,#tbl+2 do
      avg = tmp[i]
      avg = BAB:smooth_helper (avg, tmp[i-1], 0.15)
      avg = BAB:smooth_helper (avg, tmp[i+1], 0.15)
      avg = BAB:smooth_helper (avg, tmp[i-2], 0.05)
      avg = BAB:smooth_helper (avg, tmp[i+2], 0.05)
      
      tinsert (ret, BAB:round(avg))
   end
   
   return ret
end
--
function BAB:DrawPoints(pBarName, pFin)
    if BAB.DB.MainOptions.configMode == false then
        BAB:ClearLines()
    else    
		local tStep = BAB.DB.Settings.SingleDrawLineLenght
        if pBarName then --and BAB.DB.Bars[pBarName].update == true then
			BAB.DB.Bars[pBarName].update = true
			if BAB.DB.GlobalSettings.ShowCurveProgression == 1 then
				BAB:ClearLines(nil, pBarName)
				local b, tr, tg, tb, ta = pBarName
				if BAB.DB.MainOptions.selectedBar == b then
					tr, tg, tb, ta, tw = 1,1,1,1, 2
				else
					tr, tg, tb, ta, tw = 1,1,1,0.35, 1
				end
				local tTo = getn(BAB.DB.Bars[b].Points) - tStep
				for x = 1, tTo, tStep do
					BAB:DrawLine(BAB.DB.Bars[b].Points[x].x, BAB.DB.Bars[b].Points[x].y, BAB.DB.Bars[b].Points[x + tStep].x, BAB.DB.Bars[b].Points[x + tStep].y, tw,ta, tr, tg, tb, BAB.DB.Bars[b].frame, nil)
				end
				BAB:DrawLine(BAB.DB.Bars[b].Points[getn(BAB.DB.Bars[b].Points) - tStep].x, BAB.DB.Bars[b].Points[getn(BAB.DB.Bars[b].Points) - tStep].y, BAB.DB.Bars[b].Points[getn(BAB.DB.Bars[b].Points)].x, BAB.DB.Bars[b].Points[getn(BAB.DB.Bars[b].Points)].y, tw,ta, tr, tg, tb, BAB.DB.Bars[b].frame, nil)
			end
            BAB:SetButtonsOnBar(pBarName, pFin)
        else
            local tr, tg, tb, ta
            for b, v in pairs(BAB.DB.Bars) do
				v.update = true
				if BAB.DB.GlobalSettings.ShowCurveProgression == 1 then
					BAB:ClearLines(nil, b)
					if BAB.DB.MainOptions.selectedBar == b then
						tr, tg, tb, ta, tw = 1,1,1,1, 2
					else
						tr, tg, tb, ta, tw = 1,1,1,0.35, 1
					end
					for x = 1, getn(BAB.DB.Bars[b].Points) - tStep, tStep do
					   BAB:DrawLine(BAB.DB.Bars[b].Points[x].x, BAB.DB.Bars[b].Points[x].y, BAB.DB.Bars[b].Points[x + tStep].x, BAB.DB.Bars[b].Points[x + tStep].y, tw,ta, tr, tg, tb, BAB.DB.Bars[b].frame, nil)
					end
				end
                BAB:SetButtonsOnBar(b, pFin)
            end
        end
    end

end

---
local factorialLookup = {} 
factorialLookup[0] = 1.0
factorialLookup[1] = 1.0
factorialLookup[2] = 2.0
factorialLookup[3] = 6.0
factorialLookup[4] = 24.0
factorialLookup[5] = 120.0
factorialLookup[6] = 720.0
factorialLookup[7] = 5040.0
factorialLookup[8] = 40320.0
factorialLookup[9] = 362880.0
factorialLookup[10] = 3628800.0
factorialLookup[11] = 39916800.0
factorialLookup[12] = 479001600.0
factorialLookup[13] = 6227020800.0
factorialLookup[14] = 87178291200.0
factorialLookup[15] = 1307674368000.0
factorialLookup[16] = 20922789888000.0
factorialLookup[17] = 355687428096000.0
factorialLookup[18] = 6402373705728000.0
factorialLookup[19] = 121645100408832000.0
factorialLookup[20] = 2432902008176640000.0
factorialLookup[21] = 51090942171709440000.0
factorialLookup[22] = 1124000727777607680000.0
factorialLookup[23] = 25852016738884976640000.0
factorialLookup[24] = 620448401733239439360000.0
factorialLookup[25] = 15511210043330985984000000.0
factorialLookup[26] = 403291461126605635584000000.0
factorialLookup[27] = 10888869450418352160768000000.0
factorialLookup[28] = 304888344611713860501504000000.0
factorialLookup[29] = 8841761993739701954543616000000.0
factorialLookup[30] = 265252859812191058636308480000000.0
factorialLookup[31] = 8222838654177922817725562880000000.0
factorialLookup[32] = 263130836933693530167218012160000000.0

local function factorial(n)
    if n < 0 or n > 32 then
		print(string.format("DEBUG: factorial(%s) failed"), n)
	end
    return factorialLookup[n]
end
---
local function Bernstein(n, i, t)
    local basis
    local ti
    local tni

    if t == 0.0 and i == 0 then
        ti = 1.0
    else 
        ti = pow(t, i)
	end
    if n == i and t == 1.0 then
        tni = 1.0
    else 
        tni = pow((1 - t), (n - i))
	end
    basis = factorial(n) / (factorial(i) * factorial(n - i)) * ti * tni
    return basis
end
---
function BAB:CalculatePointsBernstein(pBarName) 
	local tCPoints = {}
	local tAddCount = 0
	for i = 1, BAB.DB.Bars[pBarName].sectionCount do
		local v = BAB.DB.Bars[pBarName].Sections[i]
		if i == 1 then
			tCPoints[tAddCount] = v.Knot1.x
			tCPoints[tAddCount + 1] = v.Knot1.y
			tCPoints[tAddCount + 2] = v.Vector1.x
			tCPoints[tAddCount + 3] = v.Vector1.y
			tCPoints[tAddCount + 4] = v.Vector2.x
			tCPoints[tAddCount + 5] = v.Vector2.y
			tCPoints[tAddCount + 6] = v.Knot2.x
			tCPoints[tAddCount + 7] = v.Knot2.y
			tAddCount = tAddCount + 8
		else
			tCPoints[tAddCount] = v.Vector1.x
			tCPoints[tAddCount + 1] = v.Vector1.y
			tCPoints[tAddCount + 2] = v.Vector2.x
			tCPoints[tAddCount + 3] = v.Vector2.y
			tCPoints[tAddCount + 4] = v.Knot2.x
			tCPoints[tAddCount + 5] = v.Knot2.y
			tAddCount = tAddCount + 6
		end
	end

	local POINTS_ON_CURVE = BAB.DB.Settings.BernsteinPoints --4000
	local cpts = (POINTS_ON_CURVE) / 2
	local p = {}

	local npts = ((getn(tCPoints) + 1) / 2) 
	local icount, jcount
	local step, t

	icount = 0
	t = 0
	step = 1.0 / (cpts - 1)

	for i1 = 0, cpts  do
		if (1.0 - t) < 5e-6 then
			t = 1.0
		end
		jcount = 0
		p[icount] = 0.0
		p[icount + 1] = 0.0
		for i = 0, npts - 1 do
			local basis = Bernstein(npts - 1, i, t)
			p[icount] = p[icount] + basis * tCPoints[jcount]
			p[icount + 1] = p[icount + 1] + basis * tCPoints[jcount + 1]
			jcount = jcount + 2
		end

		icount = icount + 2
		t = t + step
	end

	local floor = floor
	local tCount = 1
	local tTo = getn(p) - 2
	for x = 0, tTo , 2 do
		BAB.DB.Bars[pBarName].Points[getn(BAB.DB.Bars[pBarName].Points) + 1] = {
			x = floor(p[x]), 
			y = floor(p[x+1]),
			texture = nil, 
			}
		--BAB:DrawLine(floor(p[x]), floor(p[x+1]), floor(p[x+2]), floor(p[x+3]), 4, 1, 1, 0, 0, BAB.DB.Bars["NewBar1"].frame, nil)
	end

	--one more if circle permanent and not linked
	if BAB.DB.Bars[pBarName].CirclePermanent == 1 and BAB.DB.Bars[pBarName].linked == 0  then 
		for x = 1, BAB.DB.Bars[pBarName].buttonWidth do
			BAB.DB.Bars[pBarName].Points[getn(BAB.DB.Bars[pBarName].Points) + 1] = {
				x = BAB.DB.Bars[pBarName].Points[x].x, 
				y = BAB.DB.Bars[pBarName].Points[x].y,
				texture = nil, 
				}
		end
	end


	BAB:CleanPoints(pBarName)
end
---
function BAB:CalculatePoints(pBarName) 
    if not BAB.DB.Bars[pBarName] then return false end

	for i, v in pairs(BAB.DB.Bars[pBarName].Points) do
		BAB:ClearLines(v.texture, pBarName)
	end

	for k in pairs(BAB.DB.Bars[pBarName].Points) do
		BAB.DB.Bars[pBarName].Points[k] = nil
	end

	if BAB.DB.Bars[pBarName].sectionCount then

		if BAB.DB.Bars[pBarName].calcStyle == 2 then
			BAB:CalculatePointsBernstein(pBarName)
		else
			-- cubic bezier
			local floor = floor
			local getn = getn
			for i = 1, BAB.DB.Bars[pBarName].sectionCount do
				local v = BAB.DB.Bars[pBarName].Sections[i]

				local P0x = v.Knot1.x
				local P1x = v.Vector1.x
				local P2x = v.Vector2.x
				local P3x = v.Knot2.x
				local P0y = v.Knot1.y
				local P1y = v.Vector1.y
				local P2y = v.Vector2.y
				local P3y = v.Knot2.y
                        
				local oresultX, oresultY = P0x, P0y
            
				local cx = 3 * (P1x - P0x)
				local bx = 3 * (P2x - P1x) - cx
				local ax = P3x - P0x - cx - bx
				local cy = 3 * (P1y - P0y)
				local by = 3 * (P2y - P1y) - cy
				local ay = P3y - P0y - cy - by

				for t = 0, 1, (1 / BAB.DB.Settings.BezierPoints)  do
					local tSquared = t * t
					local tCubed = tSquared * t
					local resultX = floor((ax * tCubed) + (bx * tSquared) + (cx * t) + P0x)
					local resultY = floor((ay * tCubed) + (by * tSquared) + (cy * t) + P0y)

					if oresultX ~= resultX or oresultY ~= resultY then
						BAB.DB.Bars[pBarName].Points[getn(BAB.DB.Bars[pBarName].Points) + 1] = {
							x = resultX, 
							y = resultY, 
							texture = nil, 
							}
					end
					oresultX, oresultY = resultX, resultY
				end
			end

			local tCount = 1
			if BAB.DB.Bars[pBarName].linked == 1 then
				while tCount < ( BAB.DB.Bars[pBarName].buttonWidth) do
					BAB.DB.Bars[pBarName].Points[getn(BAB.DB.Bars[pBarName].Points) + 1] = BAB.DB.Bars[pBarName].Points[tCount] 
					--{
						--x = BAB.DB.Bars[pBarName].Points[tCount].x, 
						--y = BAB.DB.Bars[pBarName].Points[tCount].y, 
						--texture = nil, 
						--}
					tCount = tCount + 1
				end
			end

			--one more if circle permanent and not linked
			if BAB.DB.Bars[pBarName].CirclePermanent == 1 and BAB.DB.Bars[pBarName].linked == 0  then 
				for x = 1, BAB.DB.Bars[pBarName].buttonWidth do
					BAB.DB.Bars[pBarName].Points[getn(BAB.DB.Bars[pBarName].Points) + 1] = {
						x = BAB.DB.Bars[pBarName].Points[x].x, 
						y = BAB.DB.Bars[pBarName].Points[x].y,
						texture = nil, 
						}
				end
			end

			BAB:CleanPoints(pBarName)
		end
	end
end
---- 
function BAB:UpdateBars()
    for i, v in pairs(BAB.DB.Bars) do
        v.frame = _G["BABAurasBar"..v.name] 
		if not v.frame then
			v.frame = CreateFrame("Frame", "BABAurasBar"..i, UIParent)
			v.frame.barName = i
			v.backgroundTexture = BAB.DB.Bars[i].frame:CreateTexture("BABAurasBar"..i.."BackgroundTexture","BACKGROUND")
			v.nameFS = BAB.DB.Bars[i].frame:CreateFontString("BABAurasBar"..i.."BarName", "OVERLAY")
			v.anchorFS = BAB.DB.Bars[i].frame:CreateFontString("BABAurasBar"..i.."AnchorFS", "OVERLAY")
			v.underlayFrame = CreateFrame("Frame", "BABAurasBar"..i.."Underlay", UIParent)
			v.underlayFrame:SetFrameStrata("BACKGROUND")
			v.overlayFrame = CreateFrame("Frame", "BABAurasBar"..i.."Overlay", UIParent)
			v.textOverlay = CreateFrame("Frame", "BABAurasBar"..i.."textOverlay", UIParent)

			if BAB.MSQ then 
				local tG = BAB.MSQ:Group("BABAuras", v.shownName) 
				if tG then
					BAB.DB.MasqueGroups[i] = tG
				end
			end

		end
        v.underlayFrame = _G["BABAurasBar"..v.name.."Underlay"] or CreateFrame("Frame", "BABAurasBar"..v.name.."Underlay", UIParent)
		v.underlayFrame:SetFrameStrata("BACKGROUND")
        v.overlayFrame = _G["BABAurasBar"..v.name.."Overlay"] or CreateFrame("Frame", "BABAurasBar"..v.name.."Overlay", UIParent)
        v.backgroundTexture = _G["BABAurasBar"..v.name.."BackgroundTexture"] or v.frame:CreateTexture("BABAurasBar"..v.name.."BackgroundTexture","BACKGROUND")
		v.backgroundTexture:SetParent(v.frame)
        v.nameFS = _G["BABAurasBar"..v.name.."BarName"] or v.frame:CreateFontString("BABAurasBar"..v.name.."BarName", "OVERLAY")
		v.nameFS:SetParent(v.frame)
        v.anchorFS = _G["BABAurasBar"..v.name.."AnchorFS"] or v.frame:CreateFontString("BABAurasBar"..v.name.."AnchorFS", "OVERLAY")
		v.anchorFS:SetParent(v.frame)
        v.textOverlay = _G["BABAurasBar"..v.name.."textOverlay"] or v.frame:CreateFrame("BABAurasBar"..v.name.."textOverlay", "OVERLAY")
		v.textOverlay:SetParent(v.frame)
		v.underlayFrame:SetParent(v.frame)


    end
end
---- 
function BAB:CreateBar(pBarName, pX, pY, pOrgBarName)
    if not BAB.DB.Bars[pBarName] then
        BAB.DB.Bars[pBarName] = {
            name = pBarName,
            Buttons = {},
            Points = {},
            height = BAB.DB.Settings.BarHeight,
            width = BAB.DB.Settings.BarWidth,
            x = pX, 
            y = pY, 
            scale = BAB.DB.Settings.BarScale,
            color = BAB.DB.Settings.BarColor,
            buttonPadding = BAB.DB.Settings.buttonPadding,
            buttonShape = BABAURAS_CONST_BUTTON_SHAPE_DYNAMIC,
            smoothe = 0,
			showBackdropTexture = 0,
			backdropTexture = "",--BAB.DB.Settings.defaultBarBackdropTexture,
			backdropColor = BAB.DB.Settings.defaultBarBackdropColor,
			overlayFrame = nil,
			underlayFrame = nil,
			anchorFS = nil,
			isActive = 1,
			textOverlay = nil,
			showHotKey = 1,
			showCount = 1,
			showActionName = 1,
			calcStyle = 2,
			autoButton = 1,
			linked = 0,
			minSections = 1,
			CirclePermanent = 0,
			CircleRadius = 150,
			shownName = pBarName,
			autoFontSize = 0,
			showBorder = 1,
			showBackground = 1,
			showFlash = 1,
			showCooldown = 1,
        }
		local tBar = BAB.DB.Bars[pBarName]
		tBar.update = true
        tBar.frame = _G["BABAurasBar"..pBarName] or CreateFrame("Frame", "BABAurasBar"..pBarName, UIParent)
        tBar.frame.barName = pBarName
        tBar.backgroundTexture = _G["BABAurasBar"..tBar.name.."BackgroundTexture"] or  tBar.frame:CreateTexture("BABAurasBar"..tBar.name.."BackgroundTexture","BACKGROUND")
		tBar.backgroundTexture:SetParent(tBar.frame)
        tBar.nameFS = _G["BABAurasBar"..pBarName.."BarName"] or tBar.frame:CreateFontString("BABAurasBar"..pBarName.."BarName", "OVERLAY")
		tBar.nameFS:SetParent(tBar.frame)
        tBar.anchorFS = _G["BABAurasBar"..pBarName.."AnchorFS"] or tBar.frame:CreateFontString("BABAurasBar"..pBarName.."AnchorFS", "OVERLAY")
		tBar.anchorFS:SetParent(tBar.frame)
        tBar.overlayFrame = _G["BABAurasBar"..pBarName.."Overlay"] or CreateFrame("Frame", "BABAurasBar"..pBarName.."Overlay", UIParent)
        tBar.textOverlay = _G["BABAurasBar"..pBarName.."textOverlay"] or CreateFrame("Frame", "BABAurasBar"..pBarName.."textOverlay", UIParent)
		tBar.textOverlay:SetParent(tBar.frame)
        tBar.underlayFrame = _G["BABAurasBar"..pBarName.."Underlay"] or CreateFrame("Frame", "BABAurasBar"..pBarName.."Underlay", UIParent)
		tBar.underlayFrame:SetFrameStrata("LOW")
		--tBar.underlayFrame:SetParent(tBar.frame)


    end

    if BAB.MSQ then 
        local tG = BAB.MSQ:Group("BABAuras", BAB.DB.Bars[pBarName].shownName) 
        if tG then
            BAB.DB.MasqueGroups[pBarName] = tG
			--BAB:StoreMasqueData(pBarName)
        end
    end
    
    return BAB.DB.Bars[pBarName].frame
end
----
function BAB:SetupBar(pBarName)
	local tBar = BAB.DB.Bars[pBarName]
    if tBar then
		local tBFrame = tBar.frame

		if tBar.isActive == 0 then
			BAB:HideBar(pBarName)			
		else
			tBFrame:SetFrameStrata("LOW")
			tBFrame:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", tBar.x, tBar.y)
			tBFrame:SetHeight(tBar.height)
			tBFrame:SetWidth(tBar.width)
			tBFrame:SetScale(tBar.scale)
        
			tBFrame:SetScript("OnDragStart", function(self) 
				self.BABisMoving = true
				self:StartMoving() 
			end)
			tBFrame:SetScript("OnDragStop", function(self)
				if  BAB.DB.MainOptions.selectedBar ~= self.barName then
					BAB:SelectBar(self.barName)
				end
				self:StopMovingOrSizing() 
				BAB.DB.Bars[self.barName].x = self:GetLeft() 
				BAB.DB.Bars[self.barName].y = self:GetTop() - BAB.DB.Bars[self.barName].height
				self:ClearAllPoints()
				self:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", BAB.DB.Bars[self.barName].x, BAB.DB.Bars[self.barName].y)
				BAB:SaveProfile("setup bar") 
				self.BABisMoving = false
			end)
			tBFrame:SetScript("OnUpdate", function(self) 
				if self.BABisMoving then 
					BAB.DB.Bars[self.barName].x = self:GetLeft() 
					BAB.DB.Bars[self.barName].y = self:GetTop() - BAB.DB.Bars[self.barName].height
				end
			end)
			tBFrame:SetScript("OnMouseDown", function(self) 
				if BAB.DB.MainOptions.selectedBar ~= self.barName then
					BAB:SelectBar(self.barName)
				end
			end)
			tBFrame:SetMovable(true)
			tBFrame:EnableMouse(true)
			--tBFrame:SetClampedToScreen(true)
			tBFrame:RegisterForDrag("LeftButton")
			tBFrame:Show()

			tBFrame:SetBackdrop({bgFile="", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 }})
			tBFrame:SetBackdropColor(1, 0,0, 0)
			tBFrame:SetBackdropBorderColor(1, 1, 1, 1)

			tBar.overlayFrame:SetPoint("BOTTOMLEFT", tBFrame, "BOTTOMLEFT")
			tBar.overlayFrame:SetPoint("TOPRIGHT", tBFrame, "TOPRIGHT")
			tBar.overlayFrame:SetFrameStrata("TOOLTIP")
			tBar.overlayFrame:Show()

			tBar.underlayFrame:SetPoint("BOTTOMLEFT", tBFrame, "BOTTOMLEFT")
			tBar.underlayFrame:SetPoint("TOPRIGHT", tBFrame, "TOPRIGHT")
			tBar.underlayFrame:SetFrameStrata("LOW")
			if tBar.showBackdropTexture == 1 then
				tBar.underlayFrame:Show()
			else
				tBar.underlayFrame:Hide()
			end
			
			tBar.textOverlay:SetPoint("BOTTOMLEFT", tBFrame, "BOTTOMLEFT")
			tBar.textOverlay:SetPoint("TOPRIGHT", tBFrame, "TOPRIGHT")
			tBar.textOverlay:SetFrameStrata("TOOLTIP")
			tBar.textOverlay:Show()
			

			tBar.backgroundTexture:SetTexture("Interface\\Tooltips\\UI-Tooltip-Background")
			tBar.backgroundTexture:SetVertexColor(tBar.color.r, tBar.color.g, tBar.color.b, tBar.color.a)
			tBFrame:SetBackdropColor(1, 0,0, 0)
			tBFrame:SetBackdropBorderColor(1,1,1, tBar.color.a)
			tBar.backgroundTexture:ClearAllPoints()
			tBar.backgroundTexture:SetPoint("TOPLEFT", tBFrame, "TOPLEFT")
			tBar.backgroundTexture:SetPoint("BOTTOMRIGHT", tBFrame, "BOTTOMRIGHT")
			tBar.backgroundTexture:Show()

			tBar.nameFS:SetFontObject(GameFontNormalLarge)
			tBar.nameFS:SetFont("Fonts\\ARIALN.TTF", 16)
			tBar.nameFS:SetText(tBar.shownName.." ("..BAB:GetNumberOfButtons(pBarName)..")") 
			tBar.nameFS:SetTextColor(tBar.color.r, tBar.color.g, tBar.color.b, tBar.color.a )
			tBar.nameFS:SetJustifyH("CENTER") 
			tBar.nameFS:SetJustifyV("MIDDLE") 
			tBar.nameFS:SetPoint("TOPLEFT", tBFrame, "TOPLEFT", 0, 20) 
			tBar.nameFS:Show()

			tBar.anchorFS:SetFontObject(GameFontNormalLarge)
			tBar.anchorFS:SetFont("Fonts\\ARIALN.TTF", 140, "THICKOUTLINE")

			tBar.anchorFS:SetText(tBar.name.." Anchor") 
			tBar.anchorFS:SetTextColor(tBar.color.r, tBar.color.g, tBar.color.b, tBar.color.a * .3)
			tBar.anchorFS:SetJustifyH("CENTER") 
			tBar.anchorFS:SetJustifyV("MIDDLE") 
			tBar.anchorFS:Show()
			tBar.anchorFS:SetPoint("TOPLEFT", tBFrame, "TOPLEFT", 0, 0) 
			tBar.anchorFS:SetPoint("BOTTOMRIGHT", tBFrame, "BOTTOMRIGHT", 0, 0) 

			if BAB.DB.MainOptions.configMode == false then
				tBFrame:SetMovable(false)
				tBFrame:EnableMouse(false)
				tBar.backgroundTexture:SetVertexColor(tBar.color.r, tBar.color.g, tBar.color.b, 0)
				tBFrame:SetBackdropColor(1, 0,0, 0)
				tBFrame:SetBackdropBorderColor(1, 1, 1, 0)
				tBar.anchorFS:Hide()
				tBar.nameFS:Hide()
			end
        
			--for i, v in pairs(tBar.Buttons) do
				--if i <= BAB:GetMaxButtonsOnBar(pBarName) then
					--BAB:SetupButton(pBarName, i)
					--BAB:ShowButton(pBarName, i)
				--end
			--end

			local tC = 1
			while _G["LAB10ActionButtonOverlay"..tC] do
				if _G["LAB10ActionButtonOverlay"..tC]:IsVisible() then
					_G["LAB10ActionButtonOverlay"..tC]:Hide()
					_G["LAB10ActionButtonOverlay"..tC]:Show()
				end
				tC = tC + 1
			end

		--BAB:CreateFlyout(pBarName)

		end
		
		if BAB.DB.MasqueGroups[pBarName] then
			if MasqueDB then
				local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
				if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.Masque.Disabled == false and
					MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.BAB.Disabled == false and
					MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..BAB.DB.Bars[pBarName].shownName].Disabled == false 
				then
					BAB.DB.MasqueGroups[pBarName]:ReSkin() 
				else
					BAB.DB.MasqueGroups[pBarName].Disable(BAB.DB.MasqueGroups[pBarName])
				end
			end
		end

		BAB:ResizeButtonTextLabels(pBarName)
	end
end
----
function BAB:GetNumberOfButtons(pBarName)
	if pBarName and BAB.DB.Bars[pBarName] then
		local tRet = nil
		for x = 1, getn(BAB.DB.Bars[pBarName].Buttons) do
			if BAB.DB.Bars[pBarName].Buttons[x].unused == true and tRet == nil then
				tRet = x - 1
			end
		end
		return tRet or getn(BAB.DB.Bars[pBarName].Buttons)
	else
		return 0
	end
end
----
function BAB:HideBar(pBarName)
	if pBarName then
		BAB.DB.Bars[pBarName].frame:Hide()
		BAB.DB.Bars[pBarName].overlayFrame:Hide()
		BAB.DB.Bars[pBarName].underlayFrame:Hide()
		for i, v in pairs(BAB.DB.Bars[pBarName].Buttons) do
			BAB:HideButton(pBarName, i)
		end
		for i, v in pairs(BAB.DB.Bars[pBarName].Sections) do
			v.Vector1.frame:Hide()
			v.Vector2.frame:Hide()
			v.Knot1.frame:Hide()
			v.Knot2.frame:Hide()
			v.Vector1.lineToKnot:Hide()
			v.Vector1.lineToVector2:Hide()
			v.Vector2.lineToKnot:Hide()
		end
	else
		for bi, bv in pairs(BAB.DB.Bars) do
			bv.frame:Hide()
			--bv.overlayFrame:Hide()
			--bv.underlayFrame:Hide()
			--for i, v in pairs(bv.Buttons) do
			--print(i,v, bi)
				--BAB:HideButton(bi, i)
			--end
			--for i, v in pairs(bv.Sections) do
				--v.Vector1.frame:Hide()
				--v.Vector2.frame:Hide()
				--v.Knot1.frame:Hide()
				--v.Knot2.frame:Hide()
				--v.Vector1.lineToKnot:Hide()
				--v.Vector1.lineToVector2:Hide()
				--v.Vector2.lineToKnot:Hide()
			--end
		end
	end
end
---
function BAB:MasqueCallback(pGroup, pskinnID, pGloss, pBackdrop, pColors) 
	local tBarName = nil
	for i,v in pairs(BAB.DB.Bars) do
		if v.shownName == pGroup then tBarName = i end
	end

	if pGroup then
		if tBarName and BAB.DB.Bars[tBarName] then
			for i, v in pairs(BAB.DB.Bars[tBarName].Buttons) do
				local t = _G[v.name.."Icon"]
				if t then
					v.masqueTexCoords = { t:GetTexCoord()}
				end
			end
		else
			error("BAB Error: This should not happen - BAB:MasqueCallback: BAB.DB.Bars[pGroup] = nil, "..pGroup, tBarName)
		end
	else
		for bi, bv in pairs(BAB.DB.Bars) do
			for i, v in pairs(bv.Buttons) do
				local t = _G[v.name.."Icon"]
				if t then
					v.masqueTexCoords = { t:GetTexCoord()}
				end
			end
		end
	end

	local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
	local tCount = 0
	if MasqueDB.profileKeys[tUnit] then
		if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.Masque.Disabled == false then
			tCount = tCount + 1
			if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.BAB.Disabled == false then
				tCount = tCount + 1
				for i, v in pairs(BAB.DB.Bars) do
					if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..v.shownName] then
						if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..v.shownName].Disabled == false then
							tCount = tCount + 1
						end
					end
				end
			end
		end
	end
	BAB.DB.MasqueUpdateHack = tCount

    if tBarName and BAB:GetNumberOfButtons(tBarName) then
        for x = 1, BAB:GetNumberOfButtons(tBarName) do
            BAB:SetupButton(tBarName, x)
            
        end
    end
end
---
function BAB:SlashHandler(msg)
	if not InCombatLockdown() then
		--msg = string.lower(msg)
		--local fields = split(msg, " ")
		--if #fields == 0 or (fields[1] == "?" or fields[1] == "help") then
			--print("Slash commands:")
		--elseif fields[1] == "show" or fields[1] == "s" then
		--end
		if BAB.DB.MainOptions.configMode == false then BAB:OpenOptions() else BAB:CloseOptions() end
	end
end
---
function BAB:GetNumberOfBars()
	local tCount = 0
	for _ in pairs(BAB.DB.Bars) do
		tCount = tCount + 1
	end
	return tCount
end
---
do 
	--preload ant/cd textures
    f = CreateFrame("Frame", "BABAurasMain", UIParent)
	for x = 2, 23 do
		local t = f:CreateTexture("antspreload"..x)
		t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\IconAlertAnts"..x..".tga")
		t:SetAllPoints()
		t:Show()
	end
	for x = 249, 0 do 
		local t = f:CreateTexture("cdsquarepreload"..x)
		t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\CD\\"..x..".tga")
		t:Show()
	end
	for x = 249, 0 do 
		local t = f:CreateTexture("cdroundpreload"..x)
		t:SetTexture("Interface\\AddOns\\BABAuras\\textures\\round\\CD\\"..x..".tga")
		t:Show()
	end

    f:SetScript("OnEvent", function(...) 
        BAB:OnEvent(...) 
    end)
    f:SetScript("OnUpdate", function(...) 
		if MasqueDB then
			if BAB.DB.MasqueUpdateHack > 0 then
				local tUnit = GetUnitName("player", true).." - "..GetRealmName() 
				if MasqueDB.profileKeys[tUnit] then
					local tCount = 0
					if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.Masque.Disabled == false then
						tCount = tCount + 1
						if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.BAB.Disabled == false then
							tCount = tCount + 1
							for i, v in pairs(BAB.DB.Bars) do
								if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..v.shownName] then
									if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..v.shownName].Disabled == false then
										tCount = tCount + 1
									end
								end
							end
						end
					end
					if BAB.DB.MasqueUpdateHack ~= tCount then
						BAB.DB.MasqueUpdateHack = 0
						for i, v in pairs(BAB.DB.Bars) do
							if BAB:GetNumberOfButtons(i) then
								for x = 1, BAB:GetNumberOfButtons(i) do
									BAB:SetupButton(i, x)
								end
							end
						end

					end
				end
			end
		end
    end)

    f:RegisterEvent("PLAYER_ENTERING_WORLD")
    f:RegisterEvent("VARIABLES_LOADED")		
	f:RegisterEvent('PLAYER_REGEN_ENABLED')
	f:RegisterEvent('PLAYER_REGEN_DISABLED')
	f:RegisterEvent('PET_BATTLE_OPENING_START')
	f:RegisterEvent('PET_BATTLE_CLOSE')
	f:RegisterEvent('UNIT_AURA')
	f:RegisterEvent('UNIT_INVENTORY_CHANGED')
--	f:RegisterAllEvents() 
end




----------------------TEMP GLOBAL FOR DEBUGING
--_G["BAB"] = BAB
----------------------





