--[[--------------------------------------------------------------------------------
Copyright (c) Ulf Dornheck Busscher (Duugu) <duugu@duugu.com>
All rights reserved.
----------------------------------------------------------------------------------]]

local _, BAB = ...
local _G = _G
local floor = math.floor
local getn = table.getn
local tinsert = table.insert
local find = string.find
local pairs = pairs

local GetUnitName = GetUnitName
local GetRealmName = GetRealmName
local CreateFrame = CreateFrame
---
function BAB:IsLocalProfile(pProfileName)
	if not pProfileName then return nil end
	if find(pProfileName, BAB.L["Char: "]) and find(pProfileName, " %(") and  find(pProfileName, "%)") then
		return true
	end		
end
---
function BAB:CopyProfile(pTargetProfile, pSourceProfile)
    BAB:ClearLines()
    for i, v in pairs(BAB.DB.Bars) do
		BAB:HideBar(i) 
		for a, b in pairs(v.Buttons) do
			b.frame.ag:Stop()
			BAB:HideButton(i, a)
			b.frame:SetState(0, "empty")
			b.frame:ClearBindings()
		end
    end
	BABAurasPerCharacterBarData[pTargetProfile] = nil
	if not BAB:IsLocalProfile(pTargetProfile) then
		BABAurasProfiles[pTargetProfile] = nil
		BABAurasProfiles[pTargetProfile] = {}
		if BAB:IsLocalProfile(pSourceProfile) then
			BABAurasProfiles[pTargetProfile].Bars = BAB:TableCopy(BABAurasProfilesCharacter[pSourceProfile].Bars, true)   
			BABAurasProfiles[pTargetProfile].GlobalSettings = BAB:TableCopy(BABAurasProfilesCharacter[pSourceProfile].GlobalSettings, true)   
		else
			BABAurasProfiles[pTargetProfile].Bars = BAB:TableCopy(BABAurasProfiles[pSourceProfile].Bars, true)   
			BABAurasProfiles[pTargetProfile].GlobalSettings = BAB:TableCopy(BABAurasProfiles[pSourceProfile].GlobalSettings, true)   
		end
	else
		BABAurasProfilesCharacter[pTargetProfile] = nil
		BABAurasProfilesCharacter[pTargetProfile] = {}
		if BAB:IsLocalProfile(pSourceProfile) then
			BABAurasProfilesCharacter[pTargetProfile].Bars = BAB:TableCopy(BABAurasProfilesCharacter[pSourceProfile].Bars, true)   
			BABAurasProfilesCharacter[pTargetProfile].GlobalSettings = BAB:TableCopy(BABAurasProfilesCharacter[pSourceProfile].GlobalSettings, true)   
		else
			BABAurasProfilesCharacter[pTargetProfile].Bars = BAB:TableCopy(BABAurasProfiles[pSourceProfile].Bars, true)   
			BABAurasProfilesCharacter[pTargetProfile].GlobalSettings = BAB:TableCopy(BABAurasProfiles[pSourceProfile].GlobalSettings, true)   
		end
	end
	
    BABAurasCurrentProfile = pTargetProfile
    BAB:UpdateBars()
    BAB:UpdateSections()
    BAB:UpdateButtons()
    for i, v in pairs(BAB.DB.Bars) do
		BAB:InitializeBar(i)
    end
    BAB:SelectBar()

	BABAurasPerCharacterBarData[pTargetProfile] = BAB:TableCopy(BABAurasPerCharacterBarData[pSourceProfile], true)   
	BAB:LoadProfile(pTargetProfile)
		local t = next(BAB.DB.Bars) 
		BAB:SelectBar(t)

end
---
function BAB:LoadProfile(pProfileName)
-------------------------
BAB:DeleteMasqueGroups()
	BAB.DB.CalculatedDirectionButtonTable = nil

    BAB:ClearLines()
    for i, v in pairs(BAB.DB.Bars) do
		v.update = true
		BAB:HideBar(i) 
		for a, b in pairs(v.Buttons) do
			b.frame.ag:Stop()
			BAB:HideButton(i, a)
			b.cooldownTexture:Hide()
			--b.frame:SetState(0, "empty")
			--b.frame:ClearBindings()
		end
    end
	if not BAB:IsLocalProfile(pProfileName) then
		if not BABAurasProfiles[pProfileName] then
			BABAurasProfiles[pProfileName] = {}
			BABAurasProfiles[pProfileName].Bars = BAB:TableCopy(BABAurasProfiles["Base"].Bars, true)   
			BABAurasProfiles[pProfileName].GlobalSettings = BAB:TableCopy(BABAurasProfiles["Base"].GlobalSettings, true)   
		end
		BAB.DB.Bars = BAB:TableCopy(BABAurasProfiles[pProfileName].Bars, true)        
		BAB.DB.GlobalSettings = BAB:TableCopy(BABAurasProfiles[pProfileName].GlobalSettings, true)        
		BABAurasCurrentProfile = pProfileName
	else
		if not BABAurasProfilesCharacter[pProfileName] then
			BABAurasProfilesCharacter[pProfileName] = {}
			BABAurasProfilesCharacter[pProfileName].Bars = BAB:TableCopy(BABAurasProfiles["Base"].Bars, true)   
			BABAurasProfilesCharacter[pProfileName].GlobalSettings = BAB:TableCopy(BABAurasProfiles["Base"].GlobalSettings, true)   
		end
		BAB.DB.Bars = BAB:TableCopy(BABAurasProfilesCharacter[pProfileName].Bars, true)        
		BAB.DB.GlobalSettings = BAB:TableCopy(BABAurasProfilesCharacter[pProfileName].GlobalSettings, true)        
		BABAurasCurrentProfile = pProfileName
	end

    BAB:UpdateBars()
    BAB:UpdateSections()
    BAB:UpdateButtons()
    for i, v in pairs(BAB.DB.Bars) do
		BAB:InitializeBar(i)
    end
    BAB:SelectBar()


	--local tFlyBtn, tFlyNr = nil, nil

	if not BABAurasPerCharacterBarData[pProfileName] then
		BABAurasPerCharacterBarData[pProfileName] = {
			[1] = {}, 
			[2] = {}
		}
		for bari, barv in pairs(BAB.DB.Bars) do
			BABAurasPerCharacterBarData[BABAurasCurrentProfile][currentTalentSpec][bari] = {}
			BAB:RotateButtonsOnBar(bari)
		end 
	else
		for bari, barv in pairs(BAB.DB.Bars) do
			for btni, btnv in pairs(barv.Buttons) do
				if btnv.frame.overlay and btnv.frame.overlay:IsVisible() then btnv.frame.overlay:Hide() btnv.frame.overlay:Show() end

				if not BABAurasPerCharacterBarData[BABAurasCurrentProfile][currentTalentSpec][bari] then 
					BABAurasPerCharacterBarData[BABAurasCurrentProfile][currentTalentSpec][bari] = {}
				end

			end
			BAB:RotateButtonsOnBar(bari)

		end 
	end

	----------------------------------
	BAB:LoadMasqueGroups()
	BAB:LoadMasqueData()
	BAB:ApplyMasque()


	BAB:EnableDisableOptions()
	BAB:UpdateAuras()
end
---
function BAB:ResetProfile(pProfileName)
	if BAB.DB.InCombatLockdown == false then
--BAB:DeleteMasqueGroups()

		BAB.DB.CalculatedDirectionButtonTable = nil

		--print(BAB.L["Reset profile: "]..pProfileName)
		for bari, barv in pairs(BAB.DB.Bars) do
			for btni, btnv in pairs(barv.Buttons) do
				BAB:ResetButton(btnv)
				btnv.cooldownTexture:Hide()
				--btnv.frame:SetState(0, "empty")
				--btnv.frame:ClearBindings()
			end
		end 
		BABAurasPerCharacterBarData[pProfileName] = nil

		BAB:ClearLines()
		if BAB:IsLocalProfile(pProfileName) then
			BABAurasProfilesCharacter[pProfileName] = {}
			BABAurasProfilesCharacter[pProfileName].Bars = BAB:TableCopy(BABAurasProfiles["Base"].Bars, true)        
			BABAurasProfilesCharacter[pProfileName].GlobalSettings = BAB:TableCopy(BABAurasProfiles["Base"].GlobalSettings, true)        
		else
			BABAurasProfiles[pProfileName] = {}
			BABAurasProfiles[pProfileName].Bars = BAB:TableCopy(BABAurasProfiles["Base"].Bars, true)        
			BABAurasProfiles[pProfileName].GlobalSettings = BAB:TableCopy(BABAurasProfiles["Base"].GlobalSettings, true)        
		end

		BAB:UpdateBars()
		BAB:UpdateSections()
		BAB:UpdateButtons()
    
		for i, v in pairs(BAB.DB.Bars) do
			BAB:InitializeBar(i)
		end
    
		BAB:LoadProfile(pProfileName)
		BAB:SaveProfile("reset", pProfileName)


	end
	BAB:UpdateAuras()
end
---
function BAB:SaveProfile(caller)
--print(BAB.L["Save profile: "]..BABAurasCurrentProfile, caller)
	BAB:StoreMasqueData()
	--BAB:CalculateDirectionButtonTable()
	BAB.DB.CalculatedDirectionButtonTable = nil
	if BAB:IsLocalProfile(BABAurasCurrentProfile) then
		BABAurasProfilesCharacter[BABAurasCurrentProfile].Bars = BAB:TableCopy(BAB.DB.Bars, true)   
		BABAurasProfilesCharacter[BABAurasCurrentProfile].GlobalSettings = BAB:TableCopy(BAB.DB.GlobalSettings, true)   
	else
		BABAurasProfiles[BABAurasCurrentProfile].Bars = BAB:TableCopy(BAB.DB.Bars, true)   
		BABAurasProfiles[BABAurasCurrentProfile].GlobalSettings = BAB:TableCopy(BAB.DB.GlobalSettings, true)   
	end
	
	for bari, barv in pairs(BAB.DB.Bars) do
		BABAurasPerCharacterBarData[BABAurasCurrentProfile][currentTalentSpec][bari] = {}
	end 
	BAB:UpdateAuras()
end
---
function BAB:CloseOptions(pBarOnly)
    if not pBarOnly then
        BAB.DB.MainOptions.configMode = false
        BABAurasOptionSettings.MainOptions.frame:Hide()
        if BABAurasOptionSettings.BarOptions.frame then BABAurasOptionSettings.BarOptions.frame:Hide() end
    else
        if BABAurasOptionSettings.BarOptions.frame then BABAurasOptionSettings.BarOptions.frame:Hide() end
    end
    BAB:SelectBar()
    BAB:SetupAll()
	BAB:UpdateAuras()
end
---
function BAB:DropDown_OnLoad(self, frame, source, onClickFunc)
	local tname = frame:GetName()
	UIDropDownMenu_Initialize(self, function()
        local info = UIDropDownMenu_CreateInfo();
        for i=1, getn(source), 1 do
            info.text = source[i];
            info.value = value
            info.arg1 = frame
            info.arg2 = onClickFunc
            info.func = function(self, frame, onClickFunc) BAB:DropDownButton_OnClick(self, frame, onClickFunc) end;
            info.checked = nil;
            UIDropDownMenu_AddButton(info);
        end
    end);
	UIDropDownMenu_SetWidth(self, 80);
	UIDropDownMenu_SetButtonWidth(self, 24);
	UIDropDownMenu_JustifyText(frame, "LEFT")
end

function BAB:DropDownButton_OnClick(self, frame, onClickFunc)
	UIDropDownMenu_SetSelectedID(frame, self:GetID());
    onClickFunc(self:GetText())
end
------------
function BAB:OpenOptions(pBar)
    BAB.DB.MainOptions.configMode = true
    for i, v in pairs(BAB.DB.Bars) do
        BAB:SetupBar(i)
        BAB:SetupSection(i)
        BAB:DrawPoints(i)
        BAB:RotateButtonsOnBar(i)
    end
    
    local tNew = false
    if not BABAurasOptionSettings.MainOptions.frame then
        tNew = true
    else
        if not BABAurasOptionSettings.MainOptions.frame[0] then
            tNew = true
        end
    end

    if tNew == true then
        local tObj = _G["BABAurasOptions"] or CreateFrame("Frame", "BABAurasOptions" ,UIParent)
        tObj:SetFrameStrata("MEDIUM")
        tObj:SetHeight(BABAurasOptionSettings.MainOptions.height)
        tObj:SetWidth(BABAurasOptionSettings.MainOptions.width)
        tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
        tile = false, tileSize = 1, edgeSize = 10, insets = { left = 0, right = 0, top = 0, bottom = 0 }})
        tObj:SetBackdropColor(0, 0, 0, 1)
        tObj:EnableMouse(true)
        tObj:SetMovable(true)
		tObj:EnableKeyboard(true)
		tObj:ClearAllPoints()
		tObj:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.MainOptions.posX, BABAurasOptionSettings.MainOptions.posY)
        tObj:SetScript("OnDragStart", function(self) 
            self.BABisMoving = true
            if BABAurasOptionSettings.BarOptions.frame:IsVisible() then
                --self.BABoffX = BABAurasOptionSettings.BarOptions.frame:GetLeft() - self:GetLeft()
                --self.BABoffY = BABAurasOptionSettings.BarOptions.frame:GetTop() - self:GetTop()
            end
            self:StartMoving() 
        end)
        tObj:SetScript("OnDragStop", function(self) 
            self.BABisMoving = nil
            self:StopMovingOrSizing() 
            BABAurasOptionSettings.MainOptions.posX, BABAurasOptionSettings.MainOptions.posY = floor(self:GetLeft() - self:GetParent():GetLeft()), floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight())
            if BABAurasOptionSettings.BarOptions.frame:IsVisible() then
                --BABAurasOptionSettings.BarOptions.frame:ClearAllPoints()
                --BABAurasOptionSettings.BarOptions.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.MainOptions.posX + self.BABoffX, BABAurasOptionSettings.MainOptions.posY + self.BABoffY)
            end
            --BAB:SaveProfile("open options")
        end)
        tObj:SetScript("OnUpdate", function(self) 
            if self.BABisMoving then 
                BABAurasOptionSettings.MainOptions.posX, BABAurasOptionSettings.MainOptions.posY = floor(self:GetLeft() - self:GetParent():GetLeft()), floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight())
                if BABAurasOptionSettings.BarOptions.frame:IsVisible() then
                    --BABAurasOptionSettings.BarOptions.frame:ClearAllPoints()
                    --BABAurasOptionSettings.BarOptions.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.MainOptions.posX + self.BABoffX, BABAurasOptionSettings.MainOptions.posY + self.BABoffY)
                end
            end
        end)
        tObj:SetClampedToScreen(true)
        tObj:RegisterForDrag("LeftButton")
        BABAurasOptionSettings.MainOptions.frame = tObj
        
--------------------------------------------------------------------------------
        local tFS = tObj:CreateFontString("BABAurasOptionsName", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 18)
        tFS:SetText(BAB.L["BAB Main Options"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 5, -5) 
        tFS:Show()
		local tCurrentYPos = 0
		tCurrentYPos = tCurrentYPos - 20

        ---- build main options
		local tLine = tObj:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(187) tLine:SetPoint("TOPLEFT", tObj, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20

        local tFS = tObj:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Profiles"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 20

        local tOp1 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."Profiles", BABAurasOptionSettings.MainOptions.frame, "UIDropDownMenuTemplate")
        tOp1:SetFrameStrata("HIGH")
        tOp1:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", -10, tCurrentYPos)
        tOp1.UpdateFunc = function()
            local selected, tContent, tCount = nil, {}, 0
            for i, v in pairs(BABAurasProfiles) do
                if i ~= "Base" then
                    table.insert(tContent, i)
                    tCount = tCount + 1
                end
                if i == BABAurasCurrentProfile then
                    selected = tCount 
                end
            end
            for i, v in pairs(BABAurasProfilesCharacter) do
                if i ~= "Base" then
                    tinsert(tContent, i)
                    tCount = tCount + 1
                end
                if i == BABAurasCurrentProfile then
                    selected = tCount 
                end
            end
            BAB:DropDown_OnLoad(tOp1, tOp1, tContent, function(pProfileName)
                BAB:LoadProfile(pProfileName)
                BAB:OpenOptions()
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tOp1, 165)
            UIDropDownMenu_SetWidth(tOp1, 165)
            if selected then
                UIDropDownMenu_SetSelectedName(_G["BABAurasOptionsProfiles"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G["BABAurasOptionsProfiles"], tContent[selected ], 1);
            end
        end      
		tCurrentYPos = tCurrentYPos - 35        

        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNew", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["New Global Profile"])
        tOp3:SetScript("OnClick", function(...) 
            local tName = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()
            if tName ~= "" and not BABAurasProfiles[tName] then
				BAB:LoadProfile(_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText())
				BAB:SaveProfile()
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."Profiles"]:UpdateFunc()
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:SetText("")
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:ClearFocus()
				BAB:OpenOptions()
				BAB:EnableDisableOptions()
			else
				print(string.format(BAB.L["The profile name '%s'is already in use. Please select a unique Name."], _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()))
			end
        end)
     	tCurrentYPos = tCurrentYPos - 18        

        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNewLocal", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["New Character Profile"])
        tOp3:SetScript("OnClick", function(...) 
            local tName = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()
			local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
			if tName ~= "" and not BABAurasProfilesCharacter[BAB.L["Char: "]..tName.." ("..tUnit..")"] then
				BAB:LoadProfile(BAB.L["Char: "]..tName.." ("..tUnit..")")
				BAB:SaveProfile()
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."Profiles"]:UpdateFunc()
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:SetText("")
				_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:ClearFocus()
				BAB:OpenOptions()
				BAB:EnableDisableOptions()
			else
				print(string.format(BAB.L["The profile name '%s'is already in use for this character. Please select a unique Name."], _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()))
			end
        end)
		tCurrentYPos = tCurrentYPos - 20       

        local tFS = BABAurasOptionSettings.MainOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Profile Name"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 8, tCurrentYPos - 3) 
        tFS:Show()
        local tOp2 = BAB:CreateEditboxFrame(BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit", BABAurasOptionSettings.MainOptions.frame, 80)
        tOp2:SetFrameStrata("HIGH")
        tOp2:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 50, tCurrentYPos)
        tOp2:SetWidth(130)
        _G[tOp2:GetName().."Middle"]:SetWidth(130)
        tOp2:SetMaxLetters(50);
        tOp2:SetText("");
		tOp2:SetScript("OnEnterPressed", function(self, ...) self:ClearFocus() BAB:EnableDisableOptions()  end)
		tOp2:SetScript("OnEscapePressed", function(self, ...) self:SetText("") self:ClearFocus() BAB:EnableDisableOptions()  end)
		tOp2:SetScript("OnTabPressed", function(self, ...) self:ClearFocus() BAB:EnableDisableOptions()  end)
		tOp2:SetScript("OnTextChanged", function(self, ...) BAB:EnableDisableOptions() end)
		tCurrentYPos = tCurrentYPos - 25        

        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileReset", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Reset Current Profile"])
        tOp3:SetScript("OnClick", function(...) 
            BAB:ResetProfile(BABAurasCurrentProfile)
            BAB:OpenOptions()
			BAB:EnableDisableOptions() 
        end)
		tCurrentYPos = tCurrentYPos - 25        

		-- copy profile
        local tFS = BABAurasOptionSettings.MainOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Copy From"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 8, tCurrentYPos - 8) 
        tFS:Show()
        local CopyProfile = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile", BABAurasOptionSettings.MainOptions.frame, "UIDropDownMenuTemplate")
        CopyProfile:SetFrameStrata("HIGH")
        CopyProfile:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 50, tCurrentYPos)
        CopyProfile.UpdateFunc = function()
            local selected, tContent, tCount = nil, {}, 0
            for i, v in pairs(BABAurasProfiles) do
                if i ~= "Base" and i ~= BABAurasCurrentProfile then
                    tinsert(tContent, i)
                    tCount = tCount + 1
                end
            end
            for i, v in pairs(BABAurasProfilesCharacter) do
                if i ~= "Base" and i ~= BABAurasCurrentProfile then
                    tinsert(tContent, i)
                    tCount = tCount + 1
                end
            end
            BAB:DropDown_OnLoad(CopyProfile, CopyProfile, tContent, function(pProfileName)
                --BAB:LoadProfile(pProfileName)
                --BAB:OpenOptions()
				BAB:CopyProfile(BABAurasCurrentProfile, pProfileName)
				UIDropDownMenu_ClearAll(_G[BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile"])
            end)
            UIDropDownMenu_SetButtonWidth(CopyProfile, 105)
            UIDropDownMenu_SetWidth(CopyProfile, 105)
            if selected then
                --UIDropDownMenu_SetSelectedName(_G[BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile"], tContent[selected ], 1);
                --UIDropDownMenu_SetSelectedValue(_G[BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile"], tContent[selected ], 1);
            end
			BAB:EnableDisableOptions() 
        end      
		tCurrentYPos = tCurrentYPos - 25        










		--buff bars
        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."AddBarBuff", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Add Buff Bar"])
        tOp3:SetScript("OnClick", function(...) 
			BAB:SelectBar(BAB:AddBar(BAB.L["Buff"].." "..(BAB.DB.GlobalSettings.numberBuffBars + 1), 8))
			BAB.DB.GlobalSettings.numberBuffBars = BAB.DB.GlobalSettings.numberBuffBars + 1
			BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)
		tCurrentYPos = tCurrentYPos - 20

        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarBuff", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(string.format(BAB.L["Delete Buff Bar %s"], BAB.DB.GlobalSettings.numberBuffBars))
		tOp3:Disable()
        tOp3:SetScript("OnClick", function(...) 
			if BAB.DB.GlobalSettings.numberBuffBars > 1 then
				BAB:SelectBar(BAB.L["Buff"].." "..(BAB.DB.GlobalSettings.numberBuffBars))
				if BAB.DB.MainOptions.selectedBar then
					if BAB.MSQ then 
						local tG = BAB.MSQ:Group("BABAuras", BAB.DB.MainOptions.selectedBar) 
						if tG then
							tG:Delete()
							local tCount = 1
							for i, v in pairs(BAB.DB.Bars) do
								if i == BAB.DB.MainOptions.selectedBar then
									table.remove(BAB.DB.MasqueGroups, tCount)
								end
								tCount = tCount + 1
							end
						end
					end
					BAB:RemoveBar(BAB.DB.MainOptions.selectedBar, true)
					BAB.DB.GlobalSettings.numberBuffBars = BAB.DB.GlobalSettings.numberBuffBars - 1
					BAB:SaveProfile()
					local t = next(BAB.DB.Bars) 
					BAB:SelectBar(t)
				end
				BAB:EnableDisableOptions() 
			end
        end)
		tCurrentYPos = tCurrentYPos - 25    
		
		
		
		
		
		--debuff bars
        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."AddBarDebuff", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Add Debuff Bar"])
        tOp3:SetScript("OnClick", function(...) 
			BAB:SelectBar(BAB:AddBar(BAB.L["Debuff"].." "..(BAB.DB.GlobalSettings.numberDebuffBars + 1), 8))
			BAB.DB.GlobalSettings.numberDebuffBars = BAB.DB.GlobalSettings.numberDebuffBars + 1
			BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)
		tCurrentYPos = tCurrentYPos - 20

        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarDebuff", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(string.format(BAB.L["Delete Debuff Bar %s"], BAB.DB.GlobalSettings.numberDebuffBars))
		tOp3:Disable()
        tOp3:SetScript("OnClick", function(...) 
			if BAB.DB.GlobalSettings.numberDebuffBars > 1 then
				BAB:SelectBar(BAB.L["Debuff"].." "..(BAB.DB.GlobalSettings.numberDebuffBars))
				if BAB.DB.MainOptions.selectedBar then
					if BAB.MSQ then 
						local tG = BAB.MSQ:Group("BABAuras", BAB.DB.MainOptions.selectedBar) 
						if tG then
							tG:Delete()
							local tCount = 1
							for i, v in pairs(BAB.DB.Bars) do
								if i == BAB.DB.MainOptions.selectedBar then
									table.remove(BAB.DB.MasqueGroups, tCount)
								end
								tCount = tCount + 1
							end
						end
					end
					BAB:RemoveBar(BAB.DB.MainOptions.selectedBar, true)
					BAB.DB.GlobalSettings.numberDebuffBars = BAB.DB.GlobalSettings.numberDebuffBars - 1
					BAB:SaveProfile()
					local t = next(BAB.DB.Bars) 
					BAB:SelectBar(t)
				end
				BAB:EnableDisableOptions() 
			end
        end)
		tCurrentYPos = tCurrentYPos - 25    		
		
		
		
		
		
		
		    

		local tLine = tObj:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(187) tLine:SetPoint("TOPLEFT", tObj, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20

        local tFS = tObj:CreateFontString("sectionHeader3", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Buff Bar Fill Direction"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "BuffDirectionLeft",  BAB.L["Left"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.BuffFillDirection = 1 
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "BuffDirectionRight",  BAB.L["Right"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 55, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.BuffFillDirection = 2
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "BuffDirectionCenter",  BAB.L["Center"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 105, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.BuffFillDirection = 3
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		tCurrentYPos = tCurrentYPos - 25

        local tFS = tObj:CreateFontString("sectionHeader3", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Debuff Bar Fill Direction"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "DebuffDirectionLeft",  BAB.L["Left"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.DebuffFillDirection = 1 
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "DebuffDirectionRight",  BAB.L["Right"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 55, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.DebuffFillDirection = 2
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "DebuffDirectionCenter",  BAB.L["Center"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 105, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.DebuffFillDirection = 3
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		tCurrentYPos = tCurrentYPos - 25



        local tFS = tObj:CreateFontString("sectionHeader3", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Enchant Bar Fill Direction"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "EnchantDirectionLeft",  BAB.L["Left"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.EnchantFillDirection = 1 
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "EnchantDirectionRight",  BAB.L["Right"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 55, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.EnchantFillDirection = 2
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
		tCurrentYPos = tCurrentYPos - 25







        local tFS = tObj:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Buff Sort"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

        local tOp1buffSort = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."buffSort", BABAurasOptionSettings.MainOptions.frame, "UIDropDownMenuTemplate")
        tOp1buffSort:SetFrameStrata("HIGH")
        tOp1buffSort:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", -10, tCurrentYPos)
        tOp1buffSort.UpdateFunc = function()
            local selected, tContent, tCount = 1, {}, 0
            for i, v in pairs(BAB.DB.Settings.sortStyles) do
                tinsert(tContent, v)
                tCount = tCount + 1
                if i == BAB.DB.GlobalSettings.buffSort then
                    selected = tCount 
                end
            end

            BAB:DropDown_OnLoad(tOp1buffSort, tOp1buffSort, tContent, function(pShape)
				for i, v in pairs(BAB.DB.Settings.sortStyles) do
					if v == pShape then
						BAB.DB.GlobalSettings.buffSort = i
					end
				end
                BAB:SaveProfile("buffSort")
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tOp1buffSort, 165)
            UIDropDownMenu_SetWidth(tOp1buffSort, 165)
            if selected then
                UIDropDownMenu_SetSelectedName(_G["BABAurasOptionsbuffSort"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G["BABAurasOptionsbuffSort"], tContent[selected ], 1);
            end
        end      
		tCurrentYPos = tCurrentYPos - 30

    local tFS = tObj:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Debuff Sort"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

        local tOp1debuffSort = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."debuffSort", BABAurasOptionSettings.MainOptions.frame, "UIDropDownMenuTemplate")
        tOp1debuffSort:SetFrameStrata("HIGH")
        tOp1debuffSort:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", -10, tCurrentYPos)
        tOp1debuffSort.UpdateFunc = function()
            local selected, tContent, tCount = 1, {}, 0
            for i, v in pairs(BAB.DB.Settings.sortStyles) do
                tinsert(tContent, v)
                tCount = tCount + 1
                if i == BAB.DB.GlobalSettings.debuffSort then
                    selected = tCount 
                end
            end

            BAB:DropDown_OnLoad(tOp1debuffSort, tOp1debuffSort, tContent, function(pShape)
				for i, v in pairs(BAB.DB.Settings.sortStyles) do
					if v == pShape then
						BAB.DB.GlobalSettings.debuffSort = i
					end
				end
                BAB:SaveProfile("debuffSort")
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tOp1debuffSort, 165)
            UIDropDownMenu_SetWidth(tOp1debuffSort, 165)
            if selected then
                UIDropDownMenu_SetSelectedName(_G["BABAurasOptionsdebuffSort"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G["BABAurasOptionsdebuffSort"], tContent[selected ], 1);
            end
        end      
		tCurrentYPos = tCurrentYPos - 20



		--[[
        tOp3 = CreateFrame("Button", BABAurasOptionSettings.MainOptions.frame:GetName().."Togglekeybind", BABAurasOptionSettings.MainOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(184)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Toggle keybind mode"])
        tOp3:SetScript("OnClick", function(...) 
			BAB.DB.LKB:Toggle()
			BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)
		tCurrentYPos = tCurrentYPos - 25        
		]]
		local tLine = tObj:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(187) tLine:SetPoint("TOPLEFT", tObj, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20

        local tFS = tObj:CreateFontString("sectionHeader3", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Stuff"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 15

		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.MainOptions.frame, "ShowCurveProgression",  BAB.L["Show Curve Progression"]) 
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.GlobalSettings.ShowCurveProgression = 1 
			else
				BAB.DB.GlobalSettings.ShowCurveProgression = 0 
				for b, v in pairs(BAB.DB.Bars) do
					BAB:ClearLines(nil, b)
				end
			end
            BAB:SaveProfile()
			BAB:EnableDisableOptions() 
        end)   
----------------------------------------
		BABAurasOptionSettings.MainOptions.height = (tCurrentYPos - 50) * -1
		BABAurasOptionSettings.MainOptions.frame:SetHeight((tCurrentYPos - 50) * -1)
    end

    BABAurasOptionSettings.MainOptions.frame:ClearAllPoints()
    BABAurasOptionSettings.MainOptions.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.MainOptions.posX, BABAurasOptionSettings.MainOptions.posY)
    -- build BarOptions 
    if tNew == true  then
        local tObj = CreateFrame("Frame", "BABAurasBarOptions" ,UIParent)
         --tObj:ClearAllPoints()
         --tObj:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.MainOptions.posX + BABAurasOptionSettings.BarOptions.width, BABAurasOptionSettings.MainOptions.posY)
        tObj:SetFrameStrata("HIGH")
        tObj:SetHeight(BABAurasOptionSettings.BarOptions.height)
        tObj:SetWidth(BABAurasOptionSettings.BarOptions.width)
        tObj:SetBackdrop({bgFile="Interface\\Tooltips\\UI-Tooltip-Background", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", 
        tile = false, tileSize = 1, edgeSize = 10, insets = { left = 0, right = 0, top = 0, bottom = 0 }})
        tObj:SetBackdropColor(0, 0, 0, 1)
        tObj:SetClampedToScreen(true)
        tObj:EnableMouse(true)
        tObj:SetMovable(true)
		tObj:ClearAllPoints()
		tObj:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY)
        tObj:SetScript("OnDragStart", function(self) 
            self.BABisMoving = true
            self:StartMoving() 
        end)
        tObj:SetScript("OnDragStop", function(self) 
            self.BABisMoving = nil
            self:StopMovingOrSizing() 
            BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY = floor(self:GetLeft() - self:GetParent():GetLeft()), floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight())
--            BAB:SaveProfile("open options")
        end)
        tObj:SetScript("OnUpdate", function(self) 
            if self.BABisMoving then 
                BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY = floor(self:GetLeft() - self:GetParent():GetLeft()), floor(self:GetTop() - self:GetParent():GetTop() + self:GetParent():GetHeight())
            end
        end)

--
        --tObj:SetScript("OnDragStart", function(self) 
            --self.BABisMoving = true
            --self:StartMoving() 
        --end)
        --tObj:SetScript("OnDragStop", function(self) 
            --self:StopMovingOrSizing() 
             --self:OnUpdate()
            --self.BABisMoving = nil
             --self:ClearAllPoints()
             --self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY)
            --BAB:SaveProfile("BABAurasBarOptions drags")
        --end)
        --local tFunc = function(self) 
            --if self.BABisMoving then
                --local cx, cy = GetCursorPosition()
                --cx = cx / UIParent:GetEffectiveScale() 
                --cy = cy / UIParent:GetEffectiveScale() 
                ----local ex, ey = 0, 0
                ----local dl, dr, dt, db = 0,0,0,0
                     ----
                ----dl = floor(BABAurasOptionSettings.MainOptions.posX - cx)
                ----dr = floor(cx - BABAurasOptionSettings.MainOptions.posX - BABAurasOptionSettings.MainOptions.frame:GetWidth())
                ----dt = floor(cy - BABAurasOptionSettings.MainOptions.posY)
                ----if cy > BABAurasOptionSettings.MainOptions.posY - BABAurasOptionSettings.MainOptions.frame:GetHeight() then
                    ----db = floor((BABAurasOptionSettings.MainOptions.posY - BABAurasOptionSettings.MainOptions.frame:GetHeight() - cy) )
                ----else
                    ----db = floor(BABAurasOptionSettings.MainOptions.posY - BABAurasOptionSettings.MainOptions.frame:GetHeight() - cy)
                ----end
                ----if dl > dr and dl > dt and dl > db then
                    ----ex, ey =  BABAurasOptionSettings.MainOptions.posX - self:GetWidth(),  BABAurasOptionSettings.MainOptions.posY
                ----elseif dr > dl and dr > dt and dr > db then
                    ----ex, ey = BABAurasOptionSettings.MainOptions.posX + BABAurasOptionSettings.MainOptions.frame:GetWidth(), BABAurasOptionSettings.MainOptions.posY
                ----elseif dt > dl and dt > dr and dt > db then
                    ----ex, ey= BABAurasOptionSettings.MainOptions.posX,  BABAurasOptionSettings.MainOptions.posY + self:GetHeight()
                ----else
                    ----ex, ey = BABAurasOptionSettings.MainOptions.posX,  BABAurasOptionSettings.MainOptions.posY -  BABAurasOptionSettings.MainOptions.frame:GetHeight()
                ----end
                --self:ClearAllPoints()
                --self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", cx, cy)
                --BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY = ex, ey
            --end
        --end
        --tObj:SetScript("OnUpdate", tFunc)
        --tObj.OnUpdate = tFunc
        tObj:RegisterForDrag("LeftButton")

        BABAurasOptionSettings.BarOptions.frame = tObj
		local tCurrentYPos = -7 
-----------------------------------------------------------------------------------       
        local tFS = tObj:CreateFontString("BABAurasBarOptionsName", "OVERLAY")   
        BABAurasOptionSettings.BarOptions.frame.nameFS = tFS
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 16)
        tFS:SetText("") 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 5, tCurrentYPos) 
        tFS:Show()
		      
        local tOp1BarSelect = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect", BABAurasOptionSettings.BarOptions.frame, "UIDropDownMenuTemplate")
        tOp1BarSelect:SetFrameStrata("HIGH")
        tOp1BarSelect:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 70, tCurrentYPos + 4)
        tOp1BarSelect.UpdateFunc = function()
			--load/update
            local selected, tContent, tCount = nil, {}, 0
            for i, v in pairs(BAB.DB.Bars) do
                tinsert(tContent, v.shownName)
                tCount = tCount + 1
                if i == BAB.DB.MainOptions.selectedBar then
                    selected = tCount 
                end
            end
            BAB:DropDown_OnLoad(tOp1BarSelect, tOp1BarSelect, tContent, function(pBarname)
				local tCount = 0
				for i, v in pairs(BAB.DB.Bars) do
					if v.shownName == pBarname then
						BAB:SelectBar(i)
					end
				end
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tOp1BarSelect, 150)
            UIDropDownMenu_SetWidth(tOp1BarSelect, 150)
            if selected then
                UIDropDownMenu_SetSelectedName(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect"], tContent[selected ], 1);
            end
        end   
		tCurrentYPos = tCurrentYPos - 15


		local tLine = BABAurasOptionSettings.BarOptions.frame:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(250) tLine:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Pivot Points"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 20

		--add/remove section
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd",BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        tOp3:SetWidth(120)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Add Section"])
        tOp3:SetScript("OnClick", function(...) 
			local tP = 0
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 1 then 
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 0 
				tP = 1 
			end
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 1 then
				local tLength = BAB:LineLenghtBresenham(
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector2.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector2.frame.db.y,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.y)
				local tNewx, tNewy = BAB:FindPointOnLine(
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector2.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector2.frame.db.y,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.y,
					tLength / 4)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.x = tNewx
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.y = tNewy
				BAB:SetupSection(BAB.DB.MainOptions.selectedBar)
			end
            BAB:AddSection(BAB.DB.MainOptions.selectedBar)
			BAB:RePlaceKnotsVectors(BAB.DB.MainOptions.selectedBar)
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 1 then
				--knot2 von letzten auf 1/3 zu vect2 von letzten setzen
				local tLength = BAB:LineLenghtBresenham(
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Knot2.frame.db.y,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector1.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections)].Vector1.frame.db.y)
				local tNewx, tNewy = BAB:FindPointOnLine(
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.y,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.x,
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.y,
					tLength / 2)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.x = tNewx
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.y = tNewy

				BAB:SetupSection(BAB.DB.MainOptions.selectedBar)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.oldx = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.oldy = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.isMoving = true
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame:GetScript("OnUpdate")(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.isMoving = false
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.oldx = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.oldy = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.isMoving = true
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame:GetScript("OnUpdate")(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.isMoving = false
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
			if tP == 1 then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 1
				BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius, true) 
			end 
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
            BAB:SaveProfile("Add Section")

			BAB:EnableDisableOptions()
        end)

        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove",BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 130, tCurrentYPos)
        tOp3:SetWidth(120)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Remove Section"])
        tOp3:SetScript("OnClick", function(...) 
            BAB:RemoveSection(BAB.DB.MainOptions.selectedBar)
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 1 then
				BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius, true) 
			else
				BAB:RePlaceKnotsVectors(BAB.DB.MainOptions.selectedBar)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.oldx = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.oldy = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.isMoving = true
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame:GetScript("OnUpdate")(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Knot1.frame.db.isMoving = false
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.oldx = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.oldy = 0
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.isMoving = true
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame:GetScript("OnUpdate")(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[1].Vector1.frame.db.isMoving = false
			end
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
            BAB:SaveProfile("SectionRemove")
			BAB:EnableDisableOptions()
        end)
		tCurrentYPos = tCurrentYPos - 30
        
        --calc style
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Curve Calculation Style"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 125, tCurrentYPos) 
        tFS:Show()
        local tCalcStyleDD = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."CalcStyle", BABAurasOptionSettings.BarOptions.frame, "UIDropDownMenuTemplate")
        tCalcStyleDD:SetFrameStrata("HIGH")
        tCalcStyleDD:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", -10, tCurrentYPos + 6)
        tCalcStyleDD.UpdateFunc = function()
            local selected, tContent, tCount = 1, {}, 0
            for i, v in pairs(BAB.DB.Settings.calcStyles) do
                tinsert(tContent, v)
                tCount = tCount + 1
                if i == BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].calcStyle then
                    selected = tCount 
                end
            end
            BAB:DropDown_OnLoad(tCalcStyleDD, tCalcStyleDD, tContent, function(pShape)
				local tP = 0
				if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 1 then 
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 0 
					tP = 1 
				end
				for i, v in pairs(BAB.DB.Settings.calcStyles) do
					if v == pShape then
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].calcStyle = i
					end
				end
				if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].calcStyle ~= 1 then
					if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 1 then
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked = 0
						_G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"]:SetChecked(false)
						BAB:LinkBar(BAB.DB.MainOptions.selectedBar, false) 
					end
				end

	            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)
				BAB:SetupBar(BAB.DB.MainOptions.selectedBar)

				for x = 1, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].sectionCount - 1 do
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame.db.isMoving = true
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame.db.oldx = 0
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame.db.oldy = 0
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame:GetScript("OnUpdate")(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame)
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections[x].Vector2.frame.db.isMoving = false
				end
				if tP == 1 then
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 1
					BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius, true) 
				end 
                BAB:SaveProfile("CalcStyle")
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tCalcStyleDD, 100)
            UIDropDownMenu_SetWidth(tCalcStyleDD, 100)
            if selected then
                UIDropDownMenu_SetSelectedName(_G["BABAurasBarOptionsCalcStyle"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G["BABAurasBarOptionsCalcStyle"], tContent[selected ], 1);
            end
        end      
		tCurrentYPos = tCurrentYPos - 20

		--linked
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "linked",  BAB.L["linked"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB:LinkBar(BAB.DB.MainOptions.selectedBar, true)
				BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius) 
			else
				BAB:LinkBar(BAB.DB.MainOptions.selectedBar, false)
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
            BAB:SaveProfile("linked")
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 20        


		local tLine = BABAurasOptionSettings.BarOptions.frame:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(250) tLine:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Bar"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 20
		--[[
		--edit barname
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Bar Name"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 190, tCurrentYPos - 4) 
        tFS:Show()
        local tOp2 = BAB:CreateEditboxFrame(BABAurasOptionSettings.BarOptions.frame:GetName().."shownName", BABAurasOptionSettings.BarOptions.frame, 20)
        tOp2:SetFrameStrata("HIGH")
        tOp2:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 15, tCurrentYPos)
        tOp2:SetWidth(160)
        _G[tOp2:GetName().."Middle"]:SetWidth(160)
        tOp2:SetMaxLetters(25);
        tOp2:SetText("150");
        local function set(self, ...) 
            local t = self:GetText()
            if t ~= ""  then
				local tFound = nil
				for i, v in pairs(BAB.DB.Bars) do
					if v.shownName == self:GetText() and i ~= BAB.DB.MainOptions.selectedBar then
						tFound = true
					end
				end
				if not tFound then
					self:ClearFocus()
					BAB:DeleteMasqueGroups()
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName = t
					BAB:LoadMasqueGroups()
					BAB:LoadMasqueData()
					BAB:ApplyMasque()
					BAB:SaveProfile()
					_G[BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect"]:UpdateFunc()
					BAB:EnableDisableOptions()
					self:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName)
				else
					print(BAB.L["Bar name already in use. Please choose a unique name."])
				end

            end
        end
		tOp2:SetScript("OnEditFocusLost", function(self, ...) self:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName) end)
		tOp2:SetScript("OnEnterPressed", set)
		tOp2:SetScript("OnEscapePressed", function(self, ...) self:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName) self:ClearFocus() end)
		tOp2:SetScript("OnTabPressed", set)
		tCurrentYPos = tCurrentYPos - 22
		
        -- ShowBackground
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "ShowBackground",  BAB.L["Show Background"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackdropTexture = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackdropTexture = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("ShowBackground")
			BAB:EnableDisableOptions()
        end)        

		--bar background color
        local tBackgroundColor = CreateFrame("Frame", BABAurasOptionSettings.BarOptions.frame:GetName().."tBackgroundColor", BABAurasOptionSettings.BarOptions.frame)
        tBackgroundColor:SetFrameStrata("HIGH")
        tBackgroundColor:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 135, tCurrentYPos - 3)
        tBackgroundColor:SetWidth(20)
        tBackgroundColor:SetHeight(20)
		tBackgroundColor:SetBackdrop({bgFile="Interface\\AddOns\\BABAuras\\Textures\\BarBackdrop", edgeFile="Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 0, edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 }})
		if BAB.DB.MainOptions.selectedBar then 
			tBackgroundColor:SetBackdropColor(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b)
		end
        tBackgroundColor:EnableMouse(true)
		tBackgroundColor:SetScript("OnMouseUp", function(self) 
			local callback = function(color)
				local r,g,b,a 
				if color then
					r,g,b,a = unpack(color)
				else
					r,g,b = ColorPickerFrame:GetColorRGB()
					a = OpacitySliderFrame:GetValue()
				end
				--ColorPickerFrame:SetColorRGB(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b = r, g, b
				_G[BABAurasOptionSettings.BarOptions.frame:GetName().."tBackgroundColor"]:SetBackdropColor(r, g, b)
				BAB:SaveProfile("tBackgroundColor")
				BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
				BAB:EnableDisableOptions()
			end
			local cancel = function(color)
				ColorPickerFrame:SetColorRGB(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b)
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b = ColorPickerFrame.previousValues
				BAB:SaveProfile("tBackgroundColor")
				BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
				BAB:EnableDisableOptions()
			end
			
			ColorPickerFrame:SetColorRGB(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b)
			--ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = (a ~= nil), a
			ColorPickerFrame.previousValues = {BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b, 1}
			ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = callback, callback, cancel
			ColorPickerFrame:Hide()
			ColorPickerFrame:Show()
        end)       
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Background Color"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 159, tCurrentYPos - 8) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 23
		]]
		--reset bar
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonResetBar",BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        tOp3:SetWidth(245)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Reset Bar"])
        tOp3:SetScript("OnClick", function(...) 
            BAB:ResetBar(BAB.DB.MainOptions.selectedBar)
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
            BAB:SaveProfile("AutoButton")
			BAB:EnableDisableOptions()
        end)
		tCurrentYPos = tCurrentYPos - 16
        




		local tLine = BABAurasOptionSettings.BarOptions.frame:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(250) tLine:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString("sectionHeader1", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Buttons"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 20

        --button size
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Button Size"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
        local tOp2 = BAB:CreateEditboxFrame(BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth", BABAurasOptionSettings.BarOptions.frame, 20)
        tOp2:SetFrameStrata("HIGH")
        tOp2:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 40, tCurrentYPos - 15)
        tOp2:SetWidth(20)
        _G[tOp2:GetName().."Middle"]:SetWidth(20)
        tOp2:SetMaxLetters(3);
        tOp2:SetText("");
		local function set(self, ...)
            local t = floor(self:GetText())
            if t < 20  then
               t = 20
               self:SetText(t)
            end
            if t > 99 then
               t = 99
               self:SetText(t)
            end
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth = t
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonHeight = t
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("ButtonWidth ofl")
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
			self:ClearFocus()
		end
        tOp2:SetScript("OnEditFocusLost", set) 
		tOp2:SetScript("OnEnterPressed", set)
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth+", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 70, tCurrentYPos - 14)
        tOp3:SetWidth(25)
        tOp3:SetText(BAB.L["+"])
        tOp3:SetScript("OnClick", function(...) 
            local t = floor(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:GetText())
            if t < 99 then
               t = t + 1 
            end
            _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:SetText(t)
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth = t
			BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonHeight = BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("ButtonWidth")
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
        end)        
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth-", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos - 14)
        tOp3:SetWidth(25)
        tOp3:SetText(BAB.L["-"])
        tOp3:SetScript("OnClick", function(...) 
            local t = floor(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:GetText())
            if t > 20 then
               t = t - 1 
            end
            _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:SetText(t)
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth = t
			BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonHeight = BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth
            BAB:SaveProfile("ButtonWidth")
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
        end)        

        --button padding
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetFontObject(GameFontNormalSmall)
        tFS:SetText(BAB.L["Button Padding"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 105, tCurrentYPos) 
        tFS:Show()
        local tOp2 = BAB:CreateEditboxFrame(BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding", BABAurasOptionSettings.BarOptions.frame, 20)
        tOp2:SetFrameStrata("HIGH")
        tOp2:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 140, tCurrentYPos - 15)
        tOp2:SetWidth(20)
        _G[tOp2:GetName().."Middle"]:SetWidth(20)
        tOp2:SetMaxLetters(3);
        tOp2:SetText("");
        local function set(self, ...) 
            local t = floor(self:GetText())
            if t < -9 then
               t = -9
               self:SetText(t)
            end
            if t > 99 then
               t = 99
               self:SetText(t)
            end
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding = t
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("ButtonPadding")
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
        end  
        tOp2:SetScript("OnEditFocusLost", set) 
		tOp2:SetScript("OnEnterPressed", set)

        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding+", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 170, tCurrentYPos - 14)
        tOp3:SetWidth(25)
        tOp3:SetText(BAB.L["+"])
        tOp3:SetScript("OnClick", function(...) 
            local t = floor(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:GetText())
            if t < 99 then
               t = t + 1 
            end
            _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:SetText(t)
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding = t
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("ButtonPadding")
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
        end)        
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding-", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 105, tCurrentYPos - 14)
        tOp3:SetWidth(25)
        tOp3:SetText(BAB.L["-"])
        tOp3:SetScript("OnClick", function(...) 
            local t = floor(_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:GetText())
            if t > -9 then
               t = t - 1 
            end
            _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:SetText(t)
            BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding = t
            BAB:SaveProfile("ButtonPadding")
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:EnableDisableOptions()
        end)         
		tCurrentYPos = tCurrentYPos - 39
		       
        --shape
        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Button Shape"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 105, tCurrentYPos - 7) 
        tFS:Show()
        local tButtonShapeDD = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonShape", BABAurasOptionSettings.BarOptions.frame, "UIDropDownMenuTemplate")
        tButtonShapeDD:SetFrameStrata("HIGH")
        tButtonShapeDD:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", -10, tCurrentYPos)
        tButtonShapeDD.UpdateFunc = function()
            local selected, tContent, tCount = 1, {}, 0
            for i, v in pairs(BAB.DB.Settings.buttonShapes) do
                tinsert(tContent, v)
                tCount = tCount + 1
                if i == BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonShape then
                    selected = tCount 
                end
            end
            BAB:DropDown_OnLoad(tButtonShapeDD, tButtonShapeDD, tContent, function(pShape)
				for i, v in pairs(BAB.DB.Settings.buttonShapes) do
					if v == pShape then
						BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonShape = i
					end
				end
                BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
                BAB:DrawPoints()
				
				BAB:ButtonUpdateTexture()
                BAB:SaveProfile("ButtonShape")
				BAB:EnableDisableOptions()
            end)
            UIDropDownMenu_SetButtonWidth(tButtonShapeDD, 80)
            UIDropDownMenu_SetWidth(tButtonShapeDD, 80)
            if selected then
                UIDropDownMenu_SetSelectedName(_G["BABAurasBarOptionsButtonShape"], tContent[selected ], 1);
                UIDropDownMenu_SetSelectedValue(_G["BABAurasBarOptionsButtonShape"], tContent[selected ], 1);
            end
        end      
		tCurrentYPos = tCurrentYPos - 28


		--add/remove button
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonAdd",BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        tOp3:SetWidth(120)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Add Button"])
        tOp3:SetScript("OnClick", function(...) 
            BAB:AddButton(BAB.DB.MainOptions.selectedBar)
            BAB:SaveProfile("AutoButton")
			BAB:EnableDisableOptions()
        end)
        local tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonRemove",BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 130, tCurrentYPos)
        tOp3:SetWidth(120)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Remove Button"])
        tOp3:SetScript("OnClick", function(...) 
			BAB:ResetButton(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Buttons[BAB:GetNumberOfButtons(BAB.DB.MainOptions.selectedBar)])
            BAB:SaveProfile("AutoButton")
			BAB:EnableDisableOptions()
        end)
		tCurrentYPos = tCurrentYPos - 18

        -- auto button
		local autoButton = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "AutoButton",  BAB.L["Auto Button"])
        autoButton:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        autoButton:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton = 1 
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("AutoButton")
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 23

		--showHotKey,
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showHotKey",  BAB.L["HotKey"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showHotKey = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showHotKey = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("showHotKey")
			BAB:EnableDisableOptions()
        end)        
		--showCount,
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showCount",  BAB.L["Count"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 125, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCount = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCount = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("showCount")
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 20
		--[[showActionName,
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showActionName",  BAB.L["ActionName"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 185, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == 1 then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showActionName = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showActionName = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile("showActionName")
			BAB:EnableDisableOptions()
        end)        
		]]
		--auto font size
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "autoFontSize",  BAB.L["Auto Size Button Labels"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoFontSize = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoFontSize = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile()
			BAB:EnableDisableOptions()
        end)        
		--tCurrentYPos = tCurrentYPos - 15



		--- new buff
		--border 
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showBorder",  BAB.L["Show Border"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 125, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBorder  = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBorder  = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile()
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 20
				--background 
				--[[
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showBackground",  BAB.L["Show Background"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == 1 then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackground  = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackground  = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile()
			BAB:EnableDisableOptions()
        end)        
		--tCurrentYPos = tCurrentYPos - 15
		]]
		--flash 
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showFlash",  BAB.L["Flash On End"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showFlash  = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showFlash  = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile()
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 20
		--cooldown 
		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "showCooldown",  BAB.L["Show Animated Cooldown"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCooldown  = 1 
			else
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCooldown  = 0 
			end
            BAB:SetupBar(BAB.DB.MainOptions.selectedBar)
            BAB:InitializeBar(BAB.DB.MainOptions.selectedBar)  
            BAB:SaveProfile()
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 15




		------ auto bar shapes
		local tLine = BABAurasOptionSettings.BarOptions.frame:CreateTexture(nil,"ARTWORK") tLine:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line1.tga")
		tLine:SetHeight(30) tLine:SetWidth(250) tLine:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 5, tCurrentYPos)  tLine:SetVertexColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a) tLine:Show()
		tCurrentYPos = tCurrentYPos - 20
        local tFS = tObj:CreateFontString("sectionHeaderAutoshapes", "OVERLAY")         
        tFS:SetFontObject(GameFontNormalLarge)
		tFS:SetFont("Fonts\\ARIALN.TTF", 14)
        tFS:SetText(BAB.L["Automatic Bar Shapes"]) 
        tFS:SetTextColor(BABAurasOptionSettings.MainOptions.headerColor.r, BABAurasOptionSettings.MainOptions.headerColor.g, BABAurasOptionSettings.MainOptions.headerColor.b, BABAurasOptionSettings.MainOptions.headerColor.a)
        tFS:SetJustifyH("CENTER") 
        tFS:SetJustifyV("MIDDLE") 
        tFS:SetPoint("TOPLEFT", tObj, "TOPLEFT", 8, tCurrentYPos) 
        tFS:Show()
		tCurrentYPos = tCurrentYPos - 20

		-- circle
        tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."AutoCircle", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(50)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Circle"])
        tOp3:SetScript("OnClick", function(...) 
			BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius) 
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
			BAB:SaveProfile("AutoCircle")
			BAB:EnableDisableOptions()
        end)

        local tFS = BABAurasOptionSettings.BarOptions.frame:CreateFontString(nil, "OVERLAY")         
        tFS:SetFontObject(GameFontNormalSmall)
		tFS:SetFont("Fonts\\ARIALN.TTF", 12)
        tFS:SetText(BAB.L["Radius"]) 
        tFS:SetTextColor(1, 1, 1, 1)
        tFS:SetJustifyH("LEFT") 
        tFS:SetJustifyV("TOP") 
        tFS:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 125, tCurrentYPos - 4) 
        tFS:Show()
        local tOp2 = BAB:CreateEditboxFrame(BABAurasOptionSettings.BarOptions.frame:GetName().."CircleRadius", BABAurasOptionSettings.BarOptions.frame, 20)
        tOp2:SetFrameStrata("HIGH")
        tOp2:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 74, tCurrentYPos)
        tOp2:SetWidth(40)
        _G[tOp2:GetName().."Middle"]:SetWidth(40)
        tOp2:SetMaxLetters(4);
        tOp2:SetText("150");
        local function set(self, ...) 
            local t = floor(self:GetText())
            if t < 20  then
               t = 20
               self:SetText(t)
            end
            if t > 999 then
               t = 999
               self:SetText(t)
            end
			self:ClearFocus()
			BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius = t
			BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius) 
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
				BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
			end
            BAB:SaveProfile("CircleRadius")
			BAB:EnableDisableOptions()
        end
		tOp2:SetScript("OnEditFocusLost", set)
		tOp2:SetScript("OnEnterPressed", set)
		tOp2:SetScript("OnEscapePressed", function(self, ...) self:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius) self:ClearFocus() end)
		tOp2:SetScript("OnTabPressed", set)

		local Smoothe = BAB:CreateOptionsCheckButton(BABAurasOptionSettings.BarOptions.frame, "AutoCirclePermanent",  BAB.L["AutoCirclePermanent"])
        Smoothe:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 167, tCurrentYPos + 2)
        Smoothe:SetScript("OnClick", function(self, ...) 
			if self:GetChecked() == true then
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].minSections = 2
				BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_CIRCLE, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius, true) 
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 1
			else
				if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 0 then
					BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].minSections = 1
				end
				BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent = 0
			end
            BAB:SaveProfile("AutoCirclePermanent")
			BAB:EnableDisableOptions()
        end)        
		tCurrentYPos = tCurrentYPos - 27

		--line
        tOp3 = CreateFrame("Button", BABAurasOptionSettings.BarOptions.frame:GetName().."AutoLine", BABAurasOptionSettings.BarOptions.frame, "UIPanelButtonTemplate")
        tOp3:SetFrameStrata("HIGH")
        tOp3:SetPoint("TOPLEFT", BABAurasOptionSettings.BarOptions.frame, "TOPLEFT", 6, tCurrentYPos)
        tOp3:SetWidth(50)
		tOp3:GetFontString():SetFont("Fonts\\ARIALN.TTF", 12)
        tOp3:SetText(BAB.L["Line"])
        tOp3:SetScript("OnClick", function(...) 
			if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 0 then 
				BAB:AutoSetupBarShape(BAB.DB.MainOptions.selectedBar, BABAURAS_CONST_LINE) 
				if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
					BAB:AutoButtons(BAB.DB.MainOptions.selectedBar)
				end
				BAB:SaveProfile()
			end
			BAB:EnableDisableOptions()
        end)
		tCurrentYPos = tCurrentYPos - 27


----------------------------------------------------------------------
		_G["BABAurasBarOptions"]:SetHeight((tCurrentYPos - 10) * -1)
    end    
-----------------------------------------------------------------
    -- set main options
    -- profiles
    local tOp1 = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."Profiles"]
    tOp1.UpdateFunc()
    local tOp1 = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."buffSort"]
    tOp1.UpdateFunc()
    local tOp1 = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."debuffSort"]
    tOp1.UpdateFunc()

    local tOp1 = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile"]
    tOp1.UpdateFunc()
	if BAB.DB.GlobalSettings.ShowCurveProgression == 1 then
	    _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ShowCurveProgression"]:SetChecked(true)
	else
	    _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ShowCurveProgression"]:SetChecked(false)
	end
-----------------------------------------------------------------
    BABAurasOptionSettings.MainOptions.frame:Show()

    if BAB.DB.MainOptions.selectedBar then
        BABAurasOptionSettings.BarOptions.frame:ClearAllPoints()
        --BABAurasOptionSettings.BarOptions.frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", BABAurasOptionSettings.BarOptions.posX, BABAurasOptionSettings.BarOptions.posY)
		BABAurasOptionSettings.BarOptions.frame:SetPoint("TOPLEFT", BABAurasOptionSettings.MainOptions.frame, "TOPRIGHT")
        BABAurasOptionSettings.BarOptions.frame.nameFS:SetText(BAB.L["Bar Options: "])
        
		local tOp1 = _G[BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect"]
        tOp1:UpdateFunc()

-----------------------------------------------------------------
		------ set bar options
		--ButtonWidth
        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth)
        --ButtonPadding
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding)
        --ButtonShape
		local tOp1 = _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonShape"]
        tOp1:UpdateFunc()
		--Smoothe
		--[[
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackdropTexture == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ShowBackground"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."ShowBackground"]:SetChecked(false)
		end
		--bar backgroundcolor
		if BAB.DB.MainOptions.selectedBar then 
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."tBackgroundColor"]:SetBackdropColor(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.r, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.g, BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].backdropColor.b)
		end
		]]
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showHotKey == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showHotKey"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showHotKey"]:SetChecked(false)
		end
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCount == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCount"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCount"]:SetChecked(false)
		end
		--[[if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showActionName == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showActionName"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showActionName"]:SetChecked(false)
		end]]

		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoButton"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoButton"]:SetChecked(false)
		end

        --CalcStyle
		local tOp1 = _G[BABAurasOptionSettings.BarOptions.frame:GetName().."CalcStyle"]
        tOp1:UpdateFunc()

		--bar auto shapes
		--circle
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."CircleRadius"]:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CircleRadius)
		
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoCirclePermanent"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoCirclePermanent"]:SetChecked(false)
		end

		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"]:SetChecked(false)
		end

		--_G[BABAurasOptionSettings.BarOptions.frame:GetName().."shownName"]:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName)

		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoFontSize == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."autoFontSize"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."autoFontSize"]:SetChecked(false)
		end

		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBorder == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBorder"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBorder"]:SetChecked(false)
		end
		--[[
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showBackground == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBackground"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBackground"]:SetChecked(false)
		end
		]]
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showFlash == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showFlash"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showFlash"]:SetChecked(false)
		end
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].showCooldown == 1 then
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCooldown"]:SetChecked(true)
		else
	        _G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCooldown"]:SetChecked(false)
		end

-----------------------------------------------------------------
		BAB:EnableDisableOptions()
        BABAurasOptionSettings.BarOptions.frame:Show()
    else
		local t = next(BAB.DB.Bars) 
		BAB:SelectBar(t)
    end
end
---
function BAB:EnableDisableOptions()
	if BABAurasOptionSettings.MainOptions.frame and BABAurasOptionSettings.MainOptions.frame.GetName then
	--MAIN OPTIONS
		--
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."Profiles"]:Enable()
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."buffSort"]:Enable()
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."debuffSort"]:Enable()


		--
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:Enable()

		--
        local tName = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()
		if tName ~= "" and not BABAurasProfiles[tName] then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNew"]:Enable()
		else
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNew"]:Disable()
		end

		--
        local tName = _G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileEdit"]:GetText()
		local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
		if tName ~= "" and not BABAurasProfilesCharacter[BAB.L["Char: "]..tName.." ("..tUnit..")"] then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNewLocal"]:Enable()
		else
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileNewLocal"]:Disable()
		end
		
		--
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ProfileReset"]:Enable()

		--
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."CopyProfile"]:Enable()

		--
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."AddBarBuff"]:Enable()
		_G[BABAurasOptionSettings.MainOptions.frame:GetName().."AddBarDebuff"]:Enable()

		--
        _G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarBuff"]:SetText(string.format(BAB.L["Delete Buff Bar %s"], BAB.DB.GlobalSettings.numberBuffBars))
		if BAB.DB.GlobalSettings.numberBuffBars > 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarBuff"]:Enable() 
		else 
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarBuff"]:Disable() 
		end

        _G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarDebuff"]:SetText(string.format(BAB.L["Delete Debuff Bar %s"], BAB.DB.GlobalSettings.numberDebuffBars))
		if BAB.DB.GlobalSettings.numberDebuffBars > 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarDebuff"]:Enable() 
		else 
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."RemoveBarDebuff"]:Disable() 
		end

		--
		if BAB.DB.GlobalSettings.BuffFillDirection == 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionLeft"]:SetChecked(true)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionRight"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionCenter"]:SetChecked(false)
		elseif BAB.DB.GlobalSettings.BuffFillDirection == 2 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionLeft"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionRight"]:SetChecked(true)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionCenter"]:SetChecked(false)
		elseif BAB.DB.GlobalSettings.BuffFillDirection == 3 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionLeft"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionRight"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."BuffDirectionCenter"]:SetChecked(true)
		end
		--
		if BAB.DB.GlobalSettings.DebuffFillDirection == 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionLeft"]:SetChecked(true)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionRight"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionCenter"]:SetChecked(false)
		elseif BAB.DB.GlobalSettings.DebuffFillDirection == 2 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionLeft"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionRight"]:SetChecked(true)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionCenter"]:SetChecked(false)
		elseif BAB.DB.GlobalSettings.DebuffFillDirection == 3 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionLeft"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionRight"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."DebuffDirectionCenter"]:SetChecked(true)
		end
		--
		if BAB.DB.GlobalSettings.EnchantFillDirection == 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."EnchantDirectionLeft"]:SetChecked(true)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."EnchantDirectionRight"]:SetChecked(false)
		elseif BAB.DB.GlobalSettings.EnchantFillDirection == 2 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."EnchantDirectionLeft"]:SetChecked(false)
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."EnchantDirectionRight"]:SetChecked(true)
		end

		--
		if BAB.DB.GlobalSettings.ShowCurveProgression == 1 then
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ShowCurveProgression"]:SetChecked(true)
		else
			_G[BABAurasOptionSettings.MainOptions.frame:GetName().."ShowCurveProgression"]:SetChecked(false)
		end

	end
	if BABAurasOptionSettings.BarOptions.frame and BAB.DB.MainOptions.selectedBar then
	--BAR OPTIONS
		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."BarSelect"]:Enable() 

		--
		if getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) < BAB.DB.Settings.maxSections then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Enable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionAdd"]:Disable()
		end

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].minSections >= getn(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].Sections) then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."SectionRemove"]:Enable()
		end

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth"]:Enable() 

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth >= BAB.DB.Settings.buttonMaxWidth then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth+"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth+"]:Enable()
		end

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonWidth <= BAB.DB.Settings.buttonMinWidth then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth-"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonWidth-"]:Enable()
		end
	    
		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding"]:Enable() 

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding >= BAB.DB.Settings.buttonMaxPadding then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding+"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding+"]:Enable()
		end

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].buttonPadding <= BAB.DB.Settings.buttonMinPadding then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding-"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonPadding-"]:Enable()
		end

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonShape"]:Enable() 

		--
--		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ShowBackground"]:Enable() 

		--
		--_G[BABAurasOptionSettings.BarOptions.frame:GetName().."tBackgroundColor"]:Enable() 

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 or (BAB:GetNumberOfButtons(BAB.DB.MainOptions.selectedBar) >= BAB:GetMaxButtonsOnBar(BAB.DB.MainOptions.selectedBar)) then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonAdd"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonAdd"]:Enable()
		end

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].autoButton == 1 or BAB:GetNumberOfButtons(BAB.DB.MainOptions.selectedBar) <= 0 then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonRemove"]:Disable()
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonRemove"]:Enable()
		end

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoButton"]:Enable() 

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."ButtonResetBar"]:Enable() 

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showHotKey"]:Enable() 

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCount"]:Enable() 

		--
		--_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showActionName"]:Enable() 

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."CalcStyle"]:Enable()

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].calcStyle ~= 1 then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"]:Disable(true)
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"].FS:SetTextColor(0.5,0.5,0.5,0.5)
		else
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"]:Enable(true)
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."linked"].FS:SetTextColor(1,1,1,1)
		end

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoCircle"]:Enable()

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."CircleRadius"]:Enable() 

		--
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoCirclePermanent"]:Enable() 

		--
		if BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].CirclePermanent == 1 or BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].linked == 1 then
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoLine"]:Disable(true) 
		else 
			_G[BABAurasOptionSettings.BarOptions.frame:GetName().."AutoLine"]:Enable(true) 
		end

		--_G[BABAurasOptionSettings.BarOptions.frame:GetName().."shownName"]:Enable() 

		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."autoFontSize"]:Enable()
		
		
		
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBorder"]:Enable()
		--_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showBackground"]:Enable()
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showFlash"]:Enable()
		_G[BABAurasOptionSettings.BarOptions.frame:GetName().."showCooldown"]:Enable()
	end

	if BAB.DB.MainOptions.selectedBar and BAB.DB.Bars[BAB.DB.MainOptions.selectedBar] then
		BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].nameFS:SetText(BAB.DB.Bars[BAB.DB.MainOptions.selectedBar].shownName.." ("..BAB:GetNumberOfButtons(BAB.DB.MainOptions.selectedBar)..")") 
	end

end
---
function BAB:CreateEditboxFrame(pName, pParent, pWidth)
    local tFrame = CreateFrame("EditBox", pName, pParent)
    tFrame:SetAutoFocus(false) 
    tFrame:SetWidth(50)
    tFrame:SetHeight(20)
    tFrame:SetPoint("TOPLEFT", 5, -5)
    local t = tFrame:CreateTexture(pName.."Left", "BACKGROUND")
    t:SetTexture("Interface\\Common\\Common-Input-Border")
    t:SetWidth(8)
    t:SetHeight(20)
    t:SetPoint("TOPLEFT", -8, 0)
    t:SetTexCoord(0, 0.0625, 0, 0.625)
    local t1 = tFrame:CreateTexture(pName.."Middle", "BACKGROUND")
    t1:SetTexture("Interface\\Common\\Common-Input-Border")
    t1:SetWidth(pWidth)
    t1:SetHeight(20)
    t1:SetPoint("LEFT", t, "RIGHT")
    t1:SetTexCoord(0.0625, 0.9375, 0, 0.625)
    local t2 = tFrame:CreateTexture(pName.."Right", "BACKGROUND")
    t2:SetTexture("Interface\\Common\\Common-Input-Border")
    t2:SetWidth(8)
    t2:SetHeight(20)
    t2:SetPoint("LEFT", t1, "RIGHT")
    t2:SetTexCoord(0.9375, 1.0, 0 ,0.625)
    tFrame:SetFontObject(ChatFontNormal)
	tFrame:SetFont("Fonts\\ARIALN.TTF", 12)
    return tFrame
end
---
function BAB:SelectBar(pBarName)
	if BAB.DB.MainOptions.selectedBar ~= pBarName then
		BAB.DB.MainOptions.selectedBar = nil
		for i, v in pairs(BAB.DB.Bars) do
			v.color = BAB.DB.Settings.BarColor
			BAB:SetupBar(i)
		end
		if pBarName then
			BAB.DB.MainOptions.selectedBar = pBarName
			BAB:OpenOptions(pBarName)    
			BAB.DB.Bars[pBarName].color = BAB.DB.Settings.BarColorSelected
			BAB:SetupBar(pBarName)
			BAB:DrawPoints(pBarName)
		end
	end
	BAB:EnableDisableOptions()
end
--
function BAB:CreateOptionsCheckButton(pParentFrame, pName, pLabelText)
    local Smoothe = CreateFrame("CheckButton", pParentFrame:GetName()..pName, pParentFrame)
    Smoothe:SetWidth(25)
    Smoothe:SetHeight(25)
	Smoothe:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up")
	Smoothe:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down")
	Smoothe:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight")
	Smoothe:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check")
	Smoothe:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
    Smoothe:SetFrameStrata("HIGH")
    Smoothe:Show()

    local tFS = Smoothe:CreateFontString(nil, "OVERLAY")         
	Smoothe.FS = tFS
	tFS:SetFont("Fonts\\ARIALN.TTF", 12)
    tFS:SetFontObject(GameFontNormalSmall)
    tFS:SetText(pLabelText) 
    tFS:SetTextColor(1, 1, 1, 1)
    tFS:SetJustifyH("LEFT") 
    tFS:SetJustifyV("TOP") 
    tFS:SetPoint("LEFT", Smoothe, "RIGHT", 0, 0) 
    tFS:Show()

	return Smoothe
end


