--[[--------------------------------------------------------------------------------
Copyright (c) Ulf Dornheck Busscher (Duugu) <duugu@duugu.com>
All rights reserved.
----------------------------------------------------------------------------------]]
local _, BAB = ...
local _G = _G
local floor = math.floor
local getn = table.getn
local tinsert = table.insert
local tremove = table.remove
local atan2 = atan2
local deg = math.deg  
local pow = math.pow
local pairs = pairs
local abs = math.abs
local sqrt = math.sqrt

local LineTextureStorage = {}

local GetCompanionInfo = GetCompanionInfo
local GetUnitName = GetUnitName
local GetRealmName = GetRealmName

--
function BAB:ModifyTexturePath(path)
	if not path then return "" end
	path = string.gsub(string.lower(path), string.lower("Interface\\Icons"), string.lower("Interface\\AddOns\\BABTexturePack\\Icons"))
	--path = string.gsub(path, "INTERFACE\\ICONS", "Interface\\AddOns\\BABTexturePack\\Icons")
	--print(path )
	return path
end
---
function BAB:FormatTimeValue(pSeconds, flag)
	local h = math.floor(pSeconds / 3600)
	local m = math.floor((math.floor(pSeconds - (h * 3600))) / 60)
	local s = pSeconds - ((h * 3600) + (m * 60))
	if pSeconds >= 3600 then
		return h.."h"
--[[		if m > 9 then
			return h..":"..m
		else
			return h..":0"..m
		end]]
	elseif pSeconds > 60 then
		local m = math.floor(pSeconds / 60) + 1
		return m.."m"
	else
		return s.."s"
	end

end

---
function BAB:TableCopy_test(t, deep, seen)
    local nt = {}
    for k, v in pairs(t) do
	--print(k,v)
        if type(v) == 'table' then
            nt[k] = BAB:TableCopy_test(v, deep, seen)
        else
            nt[k] = v
        end
    end
	
    return nt
end
---
function BAB:RePlaceKnotsVectors(pBarName)
	local tTotalLength = 0
	local tPoints = {}
	for i, v in pairs(BAB.DB.Bars[pBarName].Sections) do
		tTotalLength = tTotalLength + BAB:LineLenghtBresenham(v.Knot1.frame.db.x, v.Knot1.frame.db.y, v.Vector1.frame.db.x, v.Vector1.frame.db.y)
		BAB:DrawVirtualLine(v.Knot1.frame.db.x, v.Knot1.frame.db.y, v.Vector1.frame.db.x, v.Vector1.frame.db.y, tPoints)
		tTotalLength = tTotalLength + BAB:LineLenghtBresenham(v.Vector1.frame.db.x, v.Vector1.frame.db.y, v.Vector2.frame.db.x, v.Vector2.frame.db.y)
		BAB:DrawVirtualLine(v.Vector1.frame.db.x, v.Vector1.frame.db.y, v.Vector2.frame.db.x, v.Vector2.frame.db.y, tPoints)
		tTotalLength = tTotalLength + BAB:LineLenghtBresenham(v.Vector2.frame.db.x, v.Vector2.frame.db.y, v.Knot2.frame.db.x, v.Knot2.frame.db.y)
		BAB:DrawVirtualLine(v.Vector2.frame.db.x, v.Vector2.frame.db.y, v.Knot2.frame.db.x, v.Knot2.frame.db.y, tPoints)
	end
	tTotalLength = floor(tTotalLength / ((getn(BAB.DB.Bars[pBarName].Sections) - 1) * 3 +  4))
	local tCount = 0
	for i, v in pairs(BAB.DB.Bars[pBarName].Sections) do
		if i > 1 then
			v.Knot1.frame.db.x, v.Knot1.frame.db.y = tPoints[tTotalLength * 3 + (tCount - 1) * 3 * tTotalLength + 1].x, tPoints[tTotalLength * 3 + (tCount - 1)  * 3 * tTotalLength  + 1].y
			v.Vector1.frame.db.x, v.Vector1.frame.db.y = tPoints[tTotalLength * 1 + tCount * 3 * tTotalLength  + 1].x, tPoints[tTotalLength * 1 + tCount * 3 * tTotalLength  + 1].y
			v.Vector2.frame.db.x, v.Vector2.frame.db.y = tPoints[tTotalLength * 2 + tCount * 3 * tTotalLength  + 1].x, tPoints[tTotalLength * 2 + tCount * 3 * tTotalLength  + 1].y
			v.Knot2.frame.db.x, v.Knot2.frame.db.y = tPoints[tTotalLength * 3 + tCount * 3 * tTotalLength  + 1].x, tPoints[tTotalLength * 3 + tCount * 3 * tTotalLength  + 1].y
		elseif i == 1 then
			v.Knot1.frame.db.x, v.Knot1.frame.db.y = tPoints[tTotalLength * 0 + tCount * 4 * tTotalLength + 1].x, tPoints[tTotalLength * 0 + tCount * 4 * tTotalLength  + 1].y
			v.Vector1.frame.db.x, v.Vector1.frame.db.y = tPoints[tTotalLength * 1 + tCount * 4 * tTotalLength  + 1].x, tPoints[tTotalLength * 1 + tCount * 4 * tTotalLength  + 1].y
			v.Vector2.frame.db.x, v.Vector2.frame.db.y = tPoints[tTotalLength * 2 + tCount * 4 * tTotalLength  + 1].x, tPoints[tTotalLength * 2 + tCount * 4 * tTotalLength  + 1].y
			v.Knot2.frame.db.x, v.Knot2.frame.db.y = tPoints[tTotalLength * 3 + tCount * 4 * tTotalLength  + 1].x, tPoints[tTotalLength * 3 + tCount * 4 * tTotalLength  + 1].y
		end
		tCount = tCount + 1
	end
	BAB:SetupSection(pBarName)

	for i, v in pairs(BAB.DB.Bars[pBarName].Sections) do
		v.Vector1.frame.db.oldx = 0
		v.Vector1.frame.db.oldy = 0
		v.Vector1.frame.db.isMoving = true
		v.Vector1.frame:GetScript("OnUpdate")(v.Vector1.frame)
		v.Vector1.frame.db.isMoving = false
	end
end
---
function BAB:GetSpellIDFromSpellName(pname)
	local rSpellID = nil
	if rSpellID == nil then
		for x = 1, GetNumCompanions("MOUNT") do
			local creatureID, creatureName, spellID, icon, active = GetCompanionInfo("MOUNT", x)
			if pname == creatureName then
				rSpellID = spellID
				return rSpellID
			end
		end
	end
	return rSpellID
end
---
function split(str, sep)
        local sep, fields = sep or ":", {}
        local pattern = string.format("([^%s]+)", sep)
        str:gsub(pattern, function(c) fields[#fields+1] = c end)
        return fields
end
---
function BAB:GetButtonBackgroundTexture(pButtonObj, pButtonTable) -- -1 (left) 0 (center) 1 (right)
    local tTexture = _G[pButtonObj:GetName().."BackdropTexture"] 
	if not tTexture then
		tTexture  = BAB.DB.Bars[pButtonObj.bar].underlayFrame:CreateTexture(pButtonObj:GetName().."BackdropTexture","BACKGROUND", -8)
		local ag = tTexture.ag or tTexture:CreateAnimationGroup()
		tTexture.ag = ag
		local a1 = tTexture.a1 or ag:CreateAnimation("Rotation")
		tTexture.a1 = a1
		tTexture.a1:SetDegrees(360)
		tTexture.a1:SetDuration(0.001)
		tTexture.a1:SetEndDelay(10)
		tTexture.a1:SetScript("OnFinished", function(self, elapsed) end)    
		tTexture.a1:SetScript("OnUpdate", function(self, elapsed) if self:IsPlaying() then if self:IsDelaying() then self:Pause() end end end)    
		tTexture.ag:Play()
	end
	pButtonTable.backdropTexture = tTexture
	return tTexture
end
---
function BAB:TableCopy(t, deep, seen)
    seen = seen or {}
    if t == nil then return nil end
    if seen[t] then return seen[t] end
    local nt = {}
    for k, v in pairs(t) do
        if type(v) ~= "userdata" and k ~= "frame" and k ~= 0  then
            if deep and type(v) == 'table' and k ~= "Points" then
                nt[k] = BAB:TableCopy(v, deep, seen)
			elseif deep and type(v) == 'table' and k == "Points" then
				nt[k] = {}
            else
                nt[k] = v
            end
        end
    end
--~     setmetatable(nt, BAB:TableCopy(getmetatable(t), deep, seen))
    seen[t] = nt
    return nt
end
---
function BAB:GetLine(prt)
    local lineframe
    for i, v in pairs(LineTextureStorage) do
        if v.inUse == false then
                lineframe = v
                v.inUse = true
                v.barName = prt.barName
                v:Show()
            --lineframe = v
            --v.inUse = true
            --break
        end
    end
    if lineframe == nil then
        tinsert(LineTextureStorage, prt:CreateTexture(nil,"OVERLAY"))
        LineTextureStorage[getn(LineTextureStorage)].inUse = true
        lineframe = LineTextureStorage[getn(LineTextureStorage)]
                lineframe.inUse = true
                lineframe.barName = prt.barName
                lineframe:Show()
    end
    return lineframe
end 
----
function BAB:ClearLines(texture, pBarName)
    if texture then
        texture:Hide()
        texture.inUse = false
		texture.barName = nil
    else
        if pBarName then
            for i, v in pairs(LineTextureStorage) do
                if v.barName == pBarName then
                    v:Hide()
                    v.inUse = false
                    v.barName = nil
                end
            end
        else
            for i, v in pairs(LineTextureStorage) do
                v:Hide()
                v.inUse = false
                v.barName = nil
            end
        end
    end
end
---
local PreCalc = {["sin"] = {}, ["cos"] = {}} 
do 
    for x = -720, 720 do 
        PreCalc.sin[x] = sin(x) 
        PreCalc.cos[x] = cos(x) 
    end 
end
function BAB:DrawLine(sx, sy, ex, ey, lineW, lineAlpha, r, g, b, prt, lineframe) 
	if not sx or not sy or not ex or not ey then return nil end
    if not lineframe then
        for i, v in pairs(LineTextureStorage) do
            if v.inUse == false then
                lineframe = v
                v.inUse = true
                v.barName = prt.barName
                v:Show()
                break
            end
        end
        if lineframe == nil then
			lineframe = BAB:GetLine(prt)
        end
    end
    if sx == ex and sy == ey then 
        return nil 
    end
    local dx, dy = ex - sx, ey - sy
    local w, h = abs(dx), abs(dy)
    local d

    if w>h then 
        d = w
    else 
        d = h 
    end
	
    local tx = (sx + ex - d) / 2.0
    local ty = (sy + ey - d) / 2.0
    local a = atan2(dy, dx)
    local s = lineW * 16 / d	
    local ca = PreCalc.cos[floor(a)] / s 
    local sa = PreCalc.sin[floor(a)] / s

    if lineframe:GetTexture() ~= "Interface\\AddOns\\BABAuras\\textures\\line.tga" then lineframe:SetTexture("Interface\\AddOns\\BABAuras\\textures\\line.tga") end

    lineframe:ClearAllPoints()
    lineframe:SetPoint("BOTTOMLEFT" ,prt ,"BOTTOMLEFT", tx, ty)
    lineframe:SetPoint("TOPRIGHT", prt, "BOTTOMLEFT", tx + d, ty + d)
    local C1, C2 = (1 + sa - ca) / 2.0, (1 - sa - ca) / 2.0
    lineframe:SetTexCoord(C1, C2, -sa+C1, ca+C2, ca+C1, sa+C2, ca-sa+C1, ca+sa+C2)
    lineframe:SetVertexColor(r, g, b, lineAlpha)
    lineframe:Show()
    
    return lineframe
end
---
function BAB:DrawVirtualLine(x0, y0, x1, y1, pPoints)
	x0 = floor(x0)
	y0 = floor(y0)
	x1 = floor(x1)
	y1 = floor(y1)
    if x0 == x1 and y0 == y1 then return end
    
    dx = abs(x1-x0)
    dy = abs(y1-y0) 
    if x0 < x1 then sx = 1 else sx = -1 end
    if y0 < y1 then sy = 1 else sy = -1 end
    local err = dx-dy
    
    local tOut
    local tCount = 0
    while not tOut  do
	--print(tCount, x0, y0)
        pPoints[getn(pPoints) + 1] = {x = x0, y = y0, texture = nil}
        if x0 == x1 and y0 == y1 then
            tOut = true
        end
        e2 = 2 * err
        if e2 > -dy then 
            err = err - dy
            x0 = x0 + sx
        end
        if x0 == x1 and y0 == y1 then 
            pPoints[getn(pPoints) + 1] = {x = x0, y = y0, texture = nil}
            tOut = true
        end
        if e2 <  dx then 
            err = err + dx
            y0 = y0 + sy 
        end 
    end
end  
---
function BAB:FindPointOnLine(x0, y0, x1, y1, pNumberPoints)
    if x0 == x1 and y0 == y1 then return end
    
    dx = abs(x1-x0)
    dy = abs(y1-y0) 
    if x0 < x1 then sx = 1 else sx = -1 end
    if y0 < y1 then sy = 1 else sy = -1 end
    local err = dx-dy
    
    local tOut
    local tCount = 0
    while not tOut and tCount < pNumberPoints do
        tCount = tCount + 1
        if x0 == x1 and y0 == y1 then
            tOut = true
        end
        e2 = 2 * err
        if e2 > -dy then 
            err = err - dy
            x0 = x0 + sx
        end
        if x0 == x1 and y0 == y1 then 
	        tCount = tCount + 1
            tOut = true
        end
        if e2 <  dx then 
            err = err + dx
            y0 = y0 + sy 
        end 
    end
	return x0, y0
end  
---
function BAB:ExtendVirtualLine(x0, y0, x1, y1, nExtend)
    if x0 == x1 and y0 == y1 then return end
    dx = abs(x1-x0)
    dy = abs(y1-y0) 
    if x0 < x1 then sx = 1 else sx = -1 end
    if y0 < y1 then sy = 1 else sy = -1 end
    local err = dx-dy
    
    local tOut
    local tCount = 0
    local tHit = 5000
    while not tOut or  tHit > tCount do
        tCount = tCount + 1
        if x0 == x1 and y0 == y1 then
            tHit = tCount + nExtend
            tOut = true
        end
        e2 = 2 * err
        if e2 > -dy then 
            err = err - dy
            x0 = x0 + sx
        end
        if x0 == x1 and y0 == y1 then 
            tHit = tCount + nExtend
            tCount = tCount + 1
            tOut = true
        end
        if e2 <  dx then 
            err = err + dx
            y0 = y0 + sy 
        end 
    end
    return x0, y0
end  
---
---
function BAB:LineLenghtBresenham(x0, y0, x1, y1)
	x0, y0, x1, y1 = floor(x0), floor(y0), floor(x1), floor(y1)
    if x0 == x1 and y0 == y1 then return 0 end
    dx = abs(x1-x0)
    dy = abs(y1-y0) 
    if x0 < x1 then sx = 1 else sx = -1 end
    if y0 < y1 then sy = 1 else sy = -1 end
    local err = dx-dy
    
    local tOut
    local tCount = 0
    local tHit = 5000
    while not tOut and  tHit > tCount do
        tCount = tCount + 1
        if x0 == x1 and y0 == y1 then
            tHit = tCount 
            tOut = true
        end
        e2 = 2 * err
        if e2 > -dy then 
            err = err - dy
            x0 = x0 + sx
        end
        if x0 == x1 and y0 == y1 then 
            tHit = tCount 
            tCount = tCount + 1
            tOut = true
        end
        if e2 <  dx then 
            err = err + dx
            y0 = y0 + sy 
        end 
    end
    return tCount
end  
---
function BAB:LineLenght(sx, sy, dx, dy)
    return floor(sqrt((sx - dx) ^ 2 + (sy - dy) ^ 2))
end
---
function BAB:IntersectionPoint(x1, y1, x2, y2, x3, y3, x4, y4)
    if x1 and y1 and x2 and y2 and x3 and y3 and x4 and y4 then
        local d = ((y4 - y3) * (x2 - x1) - (x4 - x3) * (y2 - y1))
        local Ua
        local Ub 
        if d ~= 0 then
            Ua = ((x4 - x3) * (y1 - y3) - (y4 - y3) * (x1 - x3)) / d
            Ub = ((x2 - x1) * (y1 - y3) - (y2 - y1) * (x1 - x3)) / d
        end 
        if Ua and Ub then
            if Ua >= -0.0 and Ua <= 1.0 and Ub >= -0.0 and Ub <= 1.0 then
                x = x1 + Ua * (x2 - x1)
                y = y1 + Ua * (y2 - y1)
                return floor(x), floor(y), Ua
            end
        end
    end
end       
---
function BAB:CalculateDerivate(t, p) 
    local tSquared = t * t
    local s0 = -3 + 6 * t - 3 * tSquared
    local s1 = 3 - 12 * t + 9 * tSquared
    local s2 = 6 * t - 9 * tSquared
    local s3 = 3 * tSquared
    return {x = p[0].x * s0 + p[1].x * s1 + p[2].x * s2 + p[3].x * s3, y = p[0].y * s0 + p[1].y * s1 + p[2].y * s2 + p[3].y * s3}
end
---
function BAB:CalculateSecondDerivate(t, p) 
    local s0 = 6 - 6 * t
    local s1 = -12 + 18 * t
    local s2 = 6 - 18 * t
    local s3 = 6 * t
    return {x = p[0].x * s0 + p[1].x * s1 + p[2].x * s2 + p[3].x * s3, y = p[0].y * s0 + p[1].y * s1 + p[2].y * s2 + p[3].y * s3}
end
---
function BAB:CalculateRadius(t, p) 
    local d1 = BAB:CalculateDerivate(t, p)
    local d2 = BAB:CalculateSecondDerivate(t, p)
    local r1 = sqrt(pow(d1.x * d1.x + d1.y * d1.y, 3))
    local r2 = abs(d1.x * d2.y - d2.x * d1.y)
    return r1 / r2
end 
---
function BAB:AltCalculateDerivate(t, p) 
	local tx, ty = floor((1 - t)^3 * p[0].x + 3 * t * (1 - t)^2 * p[1].x + 3 * (1 - t) * t^2 * p[2].x + t^3 * p[3].x), floor((1 - t)^3 * p[0].y + 3 * t * (1 - t)^2 * p[1].y + 3 * (1 - t) * t^2 * p[2].y + t^3 * p[3].y)
    return {x = tx, y = ty}
end
---
function BAB:AltCalculateSecondDerivate(t, p) 
	local tx = floor(-3 * p[0].x * (1 - t)^2 + 3 * p[1].x * ((1 - t)^2 - (2 * t) * (1 - t)) + 3 * p[2].x * ((-t)^2 + (1 - t) * (2 * t)) + 3 * p[3].x * t^2)
	local ty = floor(-3 * p[0].y * (1 - t)^2 + 3 * p[1].y * ((1 - t)^2 - (2 * t) * (1 - t)) + 3 * p[2].y * (-t^2 + (1 - t) * (2 * t)) + 3 * p[3].y * t^2)
    return {x = tx, y = ty}
end
---
function BAB:IsMSQEnabled(pGroupBar)
	if BAB.MSQ then 
		local tEnabled = nil
		local tUnit = GetUnitName("player", true) .. " - " .. GetRealmName() 
		if MasqueDB.profileKeys[tUnit] then
			if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.Masque.Disabled == false then
				if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups.BAB.Disabled == false then
					--if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..pGroupBar] then
						if MasqueDB.profiles[MasqueDB.profileKeys[tUnit]].Groups["BABAURAS_"..pGroupBar].Disabled == false then
							tEnabled = true
						end
					--end
				end
			end
		end
		return tEnabled
	end
end
